@@ -20,7 +20,11 @@
 
 #include <mir_toolkit/mir_connection.h>
 #include <mir_toolkit/mir_surface.h>
+#include <mir_toolkit/mir_window.h>
 #include <mir_toolkit/mir_wait.h>
+#include <mir_toolkit/mir_presentation_chain.h>
+#include <mir_toolkit/rs/mir_render_surface.h>
+#include <mir_toolkit/mir_buffer.h>
 #include <mir_toolkit/mir_buffer_stream.h>
 #include <mir_toolkit/version.h>
 #include <mir_toolkit/mir_platform_message.h>
@@ -28,5 +32,8 @@
 #include <mir_toolkit/mir_cookie.h>
 #include <mir_toolkit/mir_display_configuration.h>
 #include <mir_toolkit/mir_input_device.h>
+#include <mir_toolkit/mir_error.h>
+#include <mir_toolkit/mir_persistent_id.h>
+#include <mir_toolkit/mir_window_id.h>
 
 #endif /* MIR_CLIENT_LIBRARY_H */
