@@ -18,6 +18,7 @@
 #define MIR_TOOLKIT_MIR_SCREENCAST_H_
 
 #include <mir_toolkit/client_types.h>
+#include <mir_toolkit/deprecations.h>
 
 #ifdef __cplusplus
 /**
@@ -35,7 +36,7 @@ extern "C" {
  *
  * \param [in] connection   a valid mir connection
  * \return                  A handle that can ultimately be passed to
- *                          mir_surface_create() or mir_surface_apply_spec()
+ *                          mir_create_window() or mir_window_apply_spec()
  */
 MirScreencastSpec* mir_create_screencast_spec(MirConnection* connection);
 
@@ -142,7 +143,8 @@ char const *mir_screencast_get_error_message(MirScreencast *screencast);
  */
 MirScreencast* mir_connection_create_screencast_sync(
     MirConnection* connection,
-    MirScreencastParameters* parameters);
+    MirScreencastParameters* parameters)
+MIR_FOR_REMOVAL_IN_VERSION_1("use mir_screencast_create_sync instead");
 
 /**
  * Release the specified screencast.
