@@ -24,6 +24,8 @@
 #include "mir/geometry/rectangle.h"
 #include "mir/geometry/displacement.h"
 
+#include <vector>
+
 namespace mir
 {
 namespace input
@@ -39,6 +41,26 @@ public:
      */
     virtual mir::geometry::Rectangle bounding_rectangle() const = 0;
 
+    /**
+     * \name Device State interface of InputSink
+     *
+     * In scenarios in which the system is not capable of receiving all changes as they occur,
+     * these method should be used to update the input device state as needed
+     * \{
+     */
+    /**
+     * Set all pressed scan codes.
+     * \param scan_codes currently pressed
+     */
+    virtual void key_state(std::vector<uint32_t> const& scan_codes) = 0;
+    /**
+     * Set button state of a pointing device.
+     * \param buttons mask of the buttons currently pressed
+     */
+    virtual void pointer_state(MirPointerButtons buttons) = 0;
+    /**
+     * \}
+     */
 private:
     InputSink(InputSink const&) = delete;
     InputSink& operator=(InputSink const&) = delete;
