@@ -22,13 +22,18 @@
 #include "mir_toolkit/common.h"
 #include "mir/frontend/surface_id.h"
 #include "mir/graphics/buffer_id.h"
+#include "mir/geometry/size.h"
 #include "mir/frontend/buffer_stream_id.h"
 
 #include <memory>
 #include <string>
 
+class MirInputConfig;
+
 namespace mir
 {
+class ClientVisibleError;
+
 namespace graphics
 {
 class DisplayConfiguration;
@@ -59,6 +64,8 @@ public:
     virtual std::string name() const = 0;
 
     virtual void send_display_config(graphics::DisplayConfiguration const&) = 0;
+    virtual void send_error(ClientVisibleError const&) = 0;
+    virtual void send_input_config(MirInputConfig const& config) = 0;
 
 protected:
     Session() = default;
@@ -66,6 +73,19 @@ protected:
     Session& operator=(Session const&) = delete;
 };
 
+class SessionExtensions
+{
+public:
+    virtual ~SessionExtensions() = default;
+    virtual graphics::BufferID create_buffer(geometry::Size, MirPixelFormat) = 0;
+    virtual graphics::BufferID create_buffer(geometry::Size, uint32_t native_format, uint32_t native_flags) = 0;
+protected:
+    SessionExtensions() = default;
+    SessionExtensions(SessionExtensions const&) = delete;
+    SessionExtensions& operator=(SessionExtensions const&) = delete;
+
+};
+
 }
 }
 
