@@ -26,15 +26,15 @@
 
 #include <memory>
 
+class MirPointerConfig;
+class MirTouchpadConfig;
+class MirKeyboardConfig;
+
 namespace mir
 {
 namespace input
 {
 
-class PointerConfiguration;
-class TouchpadConfiguration;
-class KeyboardConfiguration;
-
 class Device
 {
 public:
@@ -45,14 +45,14 @@ public:
     virtual std::string name() const = 0;
     virtual std::string unique_id() const = 0;
 
-    virtual mir::optional_value<PointerConfiguration> pointer_configuration() const = 0;
-    virtual void apply_pointer_configuration(PointerConfiguration const&) = 0;
+    virtual mir::optional_value<MirPointerConfig> pointer_configuration() const = 0;
+    virtual void apply_pointer_configuration(MirPointerConfig const&) = 0;
 
-    virtual mir::optional_value<TouchpadConfiguration> touchpad_configuration() const = 0;
-    virtual void apply_touchpad_configuration(TouchpadConfiguration const&) = 0;
+    virtual mir::optional_value<MirTouchpadConfig> touchpad_configuration() const = 0;
+    virtual void apply_touchpad_configuration(MirTouchpadConfig const&) = 0;
 
-    virtual optional_value<KeyboardConfiguration> keyboard_configuration() const = 0;
-    virtual void apply_keyboard_configuration(KeyboardConfiguration const&) = 0;
+    virtual optional_value<MirKeyboardConfig> keyboard_configuration() const = 0;
+    virtual void apply_keyboard_configuration(MirKeyboardConfig const&) = 0;
 private:
     Device(Device const&) = delete;
     Device& operator=(Device const&) = delete;
