@@ -23,7 +23,7 @@
 namespace mcl = mir::client;
 
 mcl::Buffer::Buffer(
-    mir_buffer_callback cb, void* context,
+    MirBufferCallback cb, void* context,
     int buffer_id,
     std::shared_ptr<ClientBuffer> const& buffer,
     MirConnection* connection,
@@ -90,24 +90,10 @@ MirGraphicsRegion mcl::Buffer::map_region()
     };
 }
 
-MirNativeBuffer* mcl::Buffer::as_mir_native_buffer() const
+void mcl::Buffer::unmap_region()
 {
-    return buffer->as_mir_native_buffer();
-}
-
-void mcl::Buffer::set_fence(MirNativeFence* native_fence, MirBufferAccess access)
-{
-    buffer->set_fence(native_fence, access);
-}
-
-MirNativeFence* mcl::Buffer::get_fence() const
-{
-    return buffer->get_fence();
-}
-
-bool mcl::Buffer::wait_fence(MirBufferAccess access, std::chrono::nanoseconds timeout)
-{
-    return buffer->wait_fence(access, timeout);
+    std::lock_guard<decltype(mutex)> lk(mutex);
+    mapped_region = nullptr;
 }
 
 MirConnection* mcl::Buffer::allocating_connection() const
@@ -149,7 +135,7 @@ char const* mcl::Buffer::error_message() const
     return "";
 }
 
-void mcl::Buffer::set_callback(mir_buffer_callback callback, void* context)
+void mcl::Buffer::set_callback(MirBufferCallback callback, void* context)
 {
     cb.set_callback([&, callback, context]{ (*callback)(reinterpret_cast<::MirBuffer*>(this), context); });
 }
