@@ -30,12 +30,25 @@ mcl::BufferFactory::AllocationRequest::AllocationRequest(
     std::shared_ptr<mcl::ClientBufferFactory> const& native_buffer_factory,
     MirConnection* connection,
     geom::Size size, MirPixelFormat format, MirBufferUsage usage,
-    mir_buffer_callback cb, void* cb_context) :
+    MirBufferCallback cb, void* cb_context) :
     native_buffer_factory(native_buffer_factory),
     connection(connection),
     size(size),
-    format(format),
-    usage(usage),
+    sw_request(SoftwareRequest{format, usage}),
+    cb(cb),
+    cb_context(cb_context)
+{
+}
+
+mcl::BufferFactory::AllocationRequest::AllocationRequest(
+    std::shared_ptr<mcl::ClientBufferFactory> const& native_buffer_factory,
+    MirConnection* connection,
+    geom::Size size, uint32_t native_format, uint32_t native_flags,
+    MirBufferCallback cb, void* cb_context) :
+    native_buffer_factory(native_buffer_factory),
+    connection(connection),
+    size(size),
+    native_request(NativeRequest{native_format, native_flags}),
     cb(cb),
     cb_context(cb_context)
 {
@@ -47,7 +60,7 @@ void mcl::BufferFactory::expect_buffer(
     geometry::Size size,
     MirPixelFormat format,
     MirBufferUsage usage,
-    mir_buffer_callback cb,
+    MirBufferCallback cb,
     void* cb_context)
 {
     std::lock_guard<decltype(mutex)> lk(mutex);
@@ -55,6 +68,20 @@ void mcl::BufferFactory::expect_buffer(
         std::make_unique<AllocationRequest>(factory, connection, size, format, usage, cb, cb_context));
 }
 
+void mcl::BufferFactory::expect_buffer(
+    std::shared_ptr<mcl::ClientBufferFactory> const& factory,
+    MirConnection* connection,
+    geometry::Size size,
+    uint32_t native_format,
+    uint32_t native_flags,
+    MirBufferCallback cb,
+    void* cb_context)
+{
+    std::lock_guard<decltype(mutex)> lk(mutex);
+    allocation_requests.emplace_back(
+        std::make_unique<AllocationRequest>(factory, connection, size, native_format, native_flags, cb, cb_context));
+}
+
 std::unique_ptr<mcl::MirBuffer> mcl::BufferFactory::generate_buffer(mir::protobuf::Buffer const& buffer)
 {
     std::lock_guard<decltype(mutex)> lk(mutex);
@@ -76,17 +103,34 @@ std::unique_ptr<mcl::MirBuffer> mcl::BufferFactory::generate_buffer(mir::protobu
     }
     else
     {
+        auto factory = (*request_it)->native_buffer_factory;
+        std::shared_ptr<mcl::ClientBuffer> client_buffer;
+        auto usage = mir_buffer_usage_hardware;
+        if ((*request_it)->native_request.is_set())
+        {
+            auto& req = (*request_it)->native_request.value();
+            client_buffer = factory->create_buffer(
+                mcl::protobuf_to_native_buffer(buffer), req.native_format, req.native_flags);
+        }
+        else if ((*request_it)->sw_request.is_set())
+        {
+            auto& req = (*request_it)->sw_request.value();
+            usage = req.usage;
+            client_buffer = factory->create_buffer(
+                mcl::protobuf_to_native_buffer(buffer), (*request_it)->size, req.format);
+        }
+        else
+        {
+            BOOST_THROW_EXCEPTION(std::logic_error("could not create buffer"));
+        }
+
         b = std::make_unique<Buffer>(
-            (*request_it)->cb, (*request_it)->cb_context,
-            buffer.buffer_id(),
-            (*request_it)->native_buffer_factory->create_buffer(
-                mcl::protobuf_to_native_buffer(buffer),
-                (*request_it)->size, (*request_it)->format),
-                (*request_it)->connection, (*request_it)->usage);
+            (*request_it)->cb, (*request_it)->cb_context, buffer.buffer_id(),
+            client_buffer, (*request_it)->connection, usage);
     }
 
     allocation_requests.erase(request_it);
-    return std::move(b);
+    return b;
 }
 
 void mcl::BufferFactory::cancel_requests_with_context(void* cancelled_context)
