@@ -19,25 +19,18 @@
 #ifndef MIR_CLIENT_CURSOR_CONFIGURATION_H_
 #define MIR_CLIENT_CURSOR_CONFIGURATION_H_
 
+#include "mir/mir_buffer_stream.h"
 #include <string>
 
-namespace mir
-{
-namespace client
-{
-class ClientBufferStream;
-}
-}
-
 // Parameters for configuring the apperance and behavior of the system cursor. 
 // Will grow to include cursors specified by raw RGBA data, hotspots, etc...
 struct MirCursorConfiguration 
 {
     MirCursorConfiguration(char const* name);
-    MirCursorConfiguration(mir::client::ClientBufferStream const* stream, int hotspot_x, int hotspot_y);
+    MirCursorConfiguration(MirBufferStream const* stream, int hotspot_x, int hotspot_y);
 
     std::string name;
-    mir::client::ClientBufferStream const* stream;
+    MirBufferStream const* stream;
     int hotspot_x;
     int hotspot_y;
 };
