@@ -34,15 +34,12 @@ public:
         MirConnection* connection,
         int id,
         std::string const& error_msg);
-    void allocate_buffer(
-        geometry::Size size, MirPixelFormat format, MirBufferUsage usage, mir_buffer_callback, void*) override;
     void submit_buffer(MirBuffer* buffer) override;
-    void release_buffer(MirBuffer* buffer) override;
-    void buffer_available(mir::protobuf::Buffer const& buffer) override;
-    void buffer_unavailable() override;
     MirConnection* connection() const override;
     int rpc_id() const override;
     char const* error_msg() const override;
+    void set_dropping_mode() override;
+    void set_queueing_mode() override;
 private:
     MirConnection* const connection_;
     int const stream_id;
