@@ -1,5 +1,5 @@
 /*
- * Copyright © 2015 Canonical Ltd.
+ * Copyright © 2015-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License version 3,
@@ -16,12 +16,13 @@
  * Author: Robert Carr <robert.carr@canonical.com>
  */
 
-#define MIR_LOG_COMPONENT "event-builders"
+#include "mir/events/event_builders.h"
 
+#define MIR_LOG_COMPONENT "event-builders"
 #include "mir/log.h"
 
-#include "mir/events/event_builders.h"
 #include "mir/events/event_private.h"
+#include "mir/events/surface_placement_event.h"
 #include "mir/cookie/blob.h"
 #include "mir/input/xkb_mapper.h"
 #include "mir/input/keymap.h"
@@ -40,28 +41,11 @@ namespace geom = mir::geometry;
 
 namespace
 {
-    mir::cookie::Blob vector_to_cookie_as_blob(std::vector<uint8_t> const& vector)
-    {
-        mir::cookie::Blob blob{{}};
 
-        if (vector.size() > blob.size())
-        {
-            throw std::runtime_error("Vector size " + std::to_string(vector.size()) +
-                                     " is larger then array size: " +
-                                     std::to_string(blob.size()));
-        }
-
-        std::copy_n(vector.begin(), vector.size(), blob.begin());
-
-        return blob;
-    }
-
-template <class T>
-T* new_event()
+template<typename Type, typename... Args>
+auto new_event(Args&&... args) -> Type*
 {
-    T* t = new T;
-
-    return t;
+    return new Type(std::forward<Args>(args)...);
 }
 
 template <class T>
@@ -69,7 +53,6 @@ mir::EventUPtr make_uptr_event(T* e)
 {
     return mir::EventUPtr(e, ([](MirEvent* e) { delete reinterpret_cast<T*>(e); }));
 }
-
 }
 
 mir::EventUPtr mev::make_event(mf::SurfaceId const& surface_id, MirOrientation orientation)
@@ -102,10 +85,24 @@ mir::EventUPtr mev::make_event(mf::SurfaceId const& surface_id, geom::Size const
     return make_uptr_event(e);
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 mir::EventUPtr mev::make_event(mf::SurfaceId const& surface_id, MirSurfaceAttrib attribute, int value)
 {
     auto e = new_event<MirSurfaceEvent>();
 
+    e->set_id(surface_id.as_value());
+    e->set_attrib(static_cast<MirWindowAttrib>(attribute));
+    e->set_value(value);
+
+    return make_uptr_event(e);
+}
+#pragma GCC diagnostic pop
+
+mir::EventUPtr mev::make_event(mf::SurfaceId const& surface_id, MirWindowAttrib attribute, int value)
+{
+    auto e = new_event<MirWindowEvent>();
+
     e->set_id(surface_id.as_value());
     e->set_attrib(attribute);
     e->set_value(value);
@@ -115,7 +112,7 @@ mir::EventUPtr mev::make_event(mf::SurfaceId const& surface_id, MirSurfaceAttrib
 
 mir::EventUPtr mev::make_event(mf::SurfaceId const& surface_id)
 {
-    auto e = new_event<MirCloseSurfaceEvent>();
+    auto e = new_event<MirCloseWindowEvent>();
 
     e->set_surface_id(surface_id.as_value());
 
@@ -126,49 +123,34 @@ mir::EventUPtr mev::make_event(
     mf::SurfaceId const& surface_id,
     int dpi,
     float scale,
+    double refresh_rate,
     MirFormFactor form_factor,
     uint32_t output_id)
 {
-    auto e = new_event<MirSurfaceOutputEvent>();
+    auto e = new_event<MirWindowOutputEvent>();
 
     e->set_surface_id(surface_id.as_value());
     e->set_dpi(dpi);
     e->set_scale(scale);
+    e->set_refresh_rate(refresh_rate);
     e->set_form_factor(form_factor);
     e->set_output_id(output_id);
 
     return make_uptr_event(e);
 }
 
-namespace
+mir::EventUPtr mev::make_event(frontend::SurfaceId const& surface_id, geometry::Rectangle placement)
 {
-// Never exposed in old event, so lets avoid leaking it in to a header now.
-enum 
-{
-    AINPUT_SOURCE_CLASS_MASK = 0x000000ff,
+    auto e = new_event<MirWindowPlacementEvent>();
 
-    AINPUT_SOURCE_CLASS_BUTTON = 0x00000001,
-    AINPUT_SOURCE_CLASS_POINTER = 0x00000002,
-    AINPUT_SOURCE_CLASS_NAVIGATION = 0x00000004,
-    AINPUT_SOURCE_CLASS_POSITION = 0x00000008,
-    AINPUT_SOURCE_CLASS_JOYSTICK = 0x00000010
-};
-enum 
-{
-    AINPUT_SOURCE_UNKNOWN = 0x00000000,
+    e->set_id(surface_id.as_value());
+    e->set_placement({
+        placement.left().as_int(),
+        placement.top().as_int(),
+        placement.size.width.as_uint32_t(),
+        placement.size.height.as_uint32_t()});
 
-    AINPUT_SOURCE_KEYBOARD = 0x00000100 | AINPUT_SOURCE_CLASS_BUTTON,
-    AINPUT_SOURCE_DPAD = 0x00000200 | AINPUT_SOURCE_CLASS_BUTTON,
-    AINPUT_SOURCE_GAMEPAD = 0x00000400 | AINPUT_SOURCE_CLASS_BUTTON,
-    AINPUT_SOURCE_TOUCHSCREEN = 0x00001000 | AINPUT_SOURCE_CLASS_POINTER,
-    AINPUT_SOURCE_MOUSE = 0x00002000 | AINPUT_SOURCE_CLASS_POINTER,
-    AINPUT_SOURCE_STYLUS = 0x00004000 | AINPUT_SOURCE_CLASS_POINTER,
-    AINPUT_SOURCE_TRACKBALL = 0x00010000 | AINPUT_SOURCE_CLASS_NAVIGATION,
-    AINPUT_SOURCE_TOUCHPAD = 0x00100000 | AINPUT_SOURCE_CLASS_POSITION,
-    AINPUT_SOURCE_JOYSTICK = 0x01000000 | AINPUT_SOURCE_CLASS_JOYSTICK,
-
-    AINPUT_SOURCE_ANY = 0xffffff00
-};
+    return make_uptr_event(e);
 }
 
 mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseconds timestamp,
@@ -178,9 +160,8 @@ mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseco
     auto e = new_event<MirKeyboardEvent>();
 
     e->set_device_id(device_id);
-    e->set_source_id(AINPUT_SOURCE_KEYBOARD);
     e->set_event_time(timestamp);
-    e->set_cookie(vector_to_cookie_as_blob(cookie));
+    e->set_cookie(cookie);
     e->set_action(action);
     e->set_key_code(key_code);
     e->set_scan_code(scan_code);
@@ -191,44 +172,34 @@ mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseco
 
 void mev::set_modifier(MirEvent& event, MirInputEventModifiers modifiers)
 {
-    switch(event.type())
-    {
-    case mir_event_type_key:
-        {
-            auto& kev = *event.to_input()->to_keyboard();
-            kev.set_modifiers(modifiers);
-            break;
-        }
-    case mir_event_type_motion:
-        {
-            auto& mev = *event.to_input()->to_motion();
-            mev.set_modifiers(modifiers);
-            break;
-        }
-    default:
-        BOOST_THROW_EXCEPTION(std::invalid_argument("Input event modifiers are only valid for pointer, key and touch events."));
-    }
+    if (event.type() == mir_event_type_input)
+        event.to_input()->set_modifiers(modifiers);
+    else
+        BOOST_THROW_EXCEPTION(std::invalid_argument("Input event modifiers are only valid for input events."));
 }
 
 void mev::set_cursor_position(MirEvent& event, mir::geometry::Point const& pos)
 {
-    if (event.type() != mir_event_type_motion &&
-        event.to_input()->to_motion()->source_id() != AINPUT_SOURCE_MOUSE &&
-        event.to_input()->to_motion()->pointer_count() == 1)
+    set_cursor_position(event, pos.x.as_int(), pos.y.as_int());
+}
+
+void mev::set_cursor_position(MirEvent& event, float x, float y)
+{
+    if (event.type() != mir_event_type_input ||
+        event.to_input()->input_type() != mir_input_event_type_pointer)
         BOOST_THROW_EXCEPTION(std::invalid_argument("Cursor position is only valid for pointer events."));
 
-    event.to_input()->to_motion()->set_x(0, pos.x.as_int());
-    event.to_input()->to_motion()->set_y(0, pos.y.as_int());
+    event.to_input()->to_pointer()->set_x(x);
+    event.to_input()->to_pointer()->set_y(y);
 }
 
 void mev::set_button_state(MirEvent& event, MirPointerButtons button_state)
 {
-    if (event.type() != mir_event_type_motion &&
-        event.to_input()->to_motion()->source_id() != AINPUT_SOURCE_MOUSE &&
-        event.to_input()->to_motion()->pointer_count() == 1)
-        BOOST_THROW_EXCEPTION(std::invalid_argument("Cursor position is only valid for pointer events."));
+    if (event.type() != mir_event_type_input ||
+        event.to_input()->input_type() != mir_input_event_type_pointer)
+        BOOST_THROW_EXCEPTION(std::invalid_argument("Updating button state is only valid for pointer events."));
 
-    event.to_input()->to_motion()->set_buttons(button_state);
+    event.to_input()->to_pointer()->set_buttons(button_state);
 }
 
 // Deprecated version with uint64_t mac
@@ -250,13 +221,12 @@ mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseco
 mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseconds timestamp,
     std::vector<uint8_t> const& cookie, MirInputEventModifiers modifiers)
 {
-    auto e = new_event<MirMotionEvent>();
+    auto e = new_event<MirTouchEvent>();
 
     e->set_device_id(device_id);
     e->set_event_time(timestamp);
-    e->set_cookie(vector_to_cookie_as_blob(cookie));
+    e->set_cookie(cookie);
     e->set_modifiers(modifiers);
-    e->set_source_id(AINPUT_SOURCE_TOUCHSCREEN);
 
     return make_uptr_event(e);
 }
@@ -277,56 +247,38 @@ mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseco
 
 void mev::add_touch(MirEvent &event, MirTouchId touch_id, MirTouchAction action,
     MirTouchTooltype tooltype, float x_axis_value, float y_axis_value,
-    float pressure_value, float touch_major_value, float touch_minor_value, float size_value)
+    float pressure_value, float touch_major_value, float touch_minor_value, float)
 {
-    auto mev = event.to_input()->to_motion();
-    auto current_index = mev->pointer_count();
-    mev->set_pointer_count(current_index + 1);
+    auto tev = event.to_input()->to_touch();
+    auto current_index = tev->pointer_count();
+    tev->set_pointer_count(current_index + 1);
 
-    mev->set_id(current_index, touch_id);
-    mev->set_tool_type(current_index, tooltype);
-    mev->set_x(current_index, x_axis_value);
-    mev->set_y(current_index, y_axis_value);
-    mev->set_pressure(current_index, pressure_value);
-    mev->set_touch_major(current_index, touch_major_value);
-    mev->set_touch_minor(current_index, touch_minor_value);
-    mev->set_size(current_index, size_value);
-    mev->set_action(current_index, action);
+    tev->set_id(current_index, touch_id);
+    tev->set_tool_type(current_index, tooltype);
+    tev->set_x(current_index, x_axis_value);
+    tev->set_y(current_index, y_axis_value);
+    tev->set_pressure(current_index, pressure_value);
+    tev->set_touch_major(current_index, touch_major_value);
+    tev->set_touch_minor(current_index, touch_minor_value);
+    tev->set_action(current_index, action);
 }
 
 mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseconds timestamp,
     std::vector<uint8_t> const& cookie, MirInputEventModifiers modifiers, MirPointerAction action,
-    MirPointerButtons buttons_pressed,                               
+    MirPointerButtons buttons_pressed,
     float x_axis_value, float y_axis_value,
     float hscroll_value, float vscroll_value,
     float relative_x_value, float relative_y_value)
 {
-    auto e = new_event<MirMotionEvent>();
-
-    auto& mev = *e->to_input()->to_motion();
-    mev.set_device_id(device_id);
-    mev.set_event_time(timestamp);
-    mev.set_cookie(vector_to_cookie_as_blob(cookie));
-    mev.set_modifiers(modifiers);
-    mev.set_source_id(AINPUT_SOURCE_MOUSE);
-    mev.set_buttons(buttons_pressed);
-
-    mev.set_pointer_count(1);
-    mev.set_action(0, action);
-    mev.set_x(0, x_axis_value);
-    mev.set_y(0, y_axis_value);
-    mev.set_dx(0, relative_x_value);
-    mev.set_dy(0, relative_y_value);
-    mev.set_hscroll(0, hscroll_value);
-    mev.set_vscroll(0, vscroll_value);
-
+    auto e = new_event<MirPointerEvent>(device_id, timestamp, modifiers, cookie, action, buttons_pressed, x_axis_value,
+                                        y_axis_value, relative_x_value, relative_y_value, vscroll_value, hscroll_value);
     return make_uptr_event(e);
 }
 
 // Deprecated version with uint64_t mac
 mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseconds timestamp,
     uint64_t /*mac*/, MirInputEventModifiers modifiers, MirPointerAction action,
-    MirPointerButtons buttons_pressed,                               
+    MirPointerButtons buttons_pressed,
     float x_axis_value, float y_axis_value,
     float hscroll_value, float vscroll_value,
     float relative_x_value, float relative_y_value)
@@ -375,15 +327,7 @@ mir::EventUPtr mev::make_event(mf::SurfaceId const& surface_id, MirInputDeviceId
                                std::string const& layout, std::string const& variant, std::string const& options)
 {
     auto e = new_event<MirKeymapEvent>();
-    auto ep = mir::EventUPtr(e, [](MirEvent* e) {
-        // xkbcommon creates the keymap through malloc
-        if (e && e->type() == mir_event_type_keymap)
-        {
-            e->to_keymap()->free_buffer();
-        }
-
-        delete e;
-    });
+    auto ep = make_uptr_event(e);
 
     auto ctx = mi::make_unique_context();
     auto map = mi::make_unique_keymap(ctx.get(), mi::Keymap{model, layout, variant, options});
@@ -394,12 +338,15 @@ mir::EventUPtr mev::make_event(mf::SurfaceId const& surface_id, MirInputDeviceId
     e->set_surface_id(surface_id.as_value());
     e->set_device_id(id);
     // TODO consider caching compiled keymaps
-    e->set_buffer(xkb_keymap_get_as_string(map.get(), XKB_KEYMAP_FORMAT_TEXT_V1));
-    e->set_size(strlen(e->to_keymap()->buffer()));
+    auto buffer = xkb_keymap_get_as_string(map.get(), XKB_KEYMAP_FORMAT_TEXT_V1);
+    e->set_buffer(buffer);
+    std::free(buffer);
 
     return ep;
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 mir::EventUPtr mev::make_event(MirInputConfigurationAction action, MirInputDeviceId id, std::chrono::nanoseconds time)
 {
     auto e = new_event<MirInputConfigurationEvent>();
@@ -410,6 +357,7 @@ mir::EventUPtr mev::make_event(MirInputConfigurationAction action, MirInputDevic
 
     return make_uptr_event(e);
 }
+#pragma GCC diagnostic pop
 
 mir::EventUPtr mev::make_event(std::chrono::nanoseconds timestamp,
                                MirPointerButtons pointer_buttons,
@@ -424,8 +372,45 @@ mir::EventUPtr mev::make_event(std::chrono::nanoseconds timestamp,
     e->set_pointer_buttons(pointer_buttons);
     e->set_pointer_axis(mir_pointer_axis_x, x_axis_value);
     e->set_pointer_axis(mir_pointer_axis_y, y_axis_value);
-    for (auto& dev : device_states)
-        e->add_device(dev.id, std::move(dev.pressed_keys), dev.buttons);
+    e->set_device_states(device_states);
 
     return make_uptr_event(e);
 }
+
+mir::EventUPtr mev::clone_event(MirEvent const& event)
+{
+    return make_uptr_event(new MirEvent(event));
+}
+
+void mev::transform_positions(MirEvent& event, mir::geometry::Displacement const& movement)
+{
+    if (event.type() == mir_event_type_input)
+    {
+        auto const input_type = event.to_input()->input_type();
+        if (input_type == mir_input_event_type_pointer)
+        {
+            auto pev = event.to_input()->to_pointer();
+            pev->set_x(pev->x() - movement.dx.as_int());
+            pev->set_y(pev->y() - movement.dy.as_int());
+        }
+        else if (input_type == mir_input_event_type_touch)
+        {
+            auto tev = event.to_input()->to_touch();
+            for (unsigned i = 0; i < tev->pointer_count(); i++)
+            {
+                auto x = tev->x(i);
+                auto y = tev->y(i);
+                tev->set_x(i, x - movement.dx.as_int());
+                tev->set_y(i, y - movement.dy.as_int());
+            }
+        }
+    }
+}
+
+mir::EventUPtr mev::make_event(MirInputDeviceId device_id, std::chrono::nanoseconds timestamp,
+                               std::vector<uint8_t> const& cookie, MirInputEventModifiers modifiers,
+                               std::vector<mev::ContactState> const& contacts)
+{
+    auto e = new_event<MirTouchEvent>(device_id, timestamp, cookie, modifiers, contacts);
+    return make_uptr_event(e);
+}
