@@ -53,54 +53,29 @@ int valid_fd_or_system_error(int fd, char const* message)
 mircva::InputReceiver::InputReceiver(droidinput::sp<droidinput::InputChannel> const& input_channel,
                                      std::shared_ptr<mircv::XKBMapper> const& keymapper,
                                      std::function<void(MirEvent*)> const& event_handling_callback,
-                                     std::shared_ptr<mircv::InputReceiverReport> const& report,
-                                     AndroidClock clock)
-  : timer_fd{valid_fd_or_system_error(timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC),
-        "Failed to create IO timer")},
-    wake_fd{valid_fd_or_system_error(eventfd(0, EFD_CLOEXEC | EFD_NONBLOCK | EFD_SEMAPHORE),
+                                     std::shared_ptr<mircv::InputReceiverReport> const& report)
+  : wake_fd{valid_fd_or_system_error(eventfd(0, EFD_CLOEXEC | EFD_NONBLOCK),
         "Failed to create IO wakeup notifier")},
     input_channel(input_channel),
     handler{event_handling_callback},
     xkb_mapper(keymapper),
     report(report),
-    input_consumer(std::make_shared<droidinput::InputConsumer>(input_channel)),
-    android_clock(clock)
+    input_consumer(std::make_shared<droidinput::InputConsumer>(input_channel))
 {
-    /*
-     * 59Hz by default. This ensures the input rate never gets ahead of the
-     * typical display rate, which would be seen as visible lag.
-     */
-    event_rate_hz = 59;
-    auto env = getenv("MIR_CLIENT_INPUT_RATE");
-    if (env != NULL)
-        event_rate_hz = atoi(env);
-
-    dispatcher.add_watch(timer_fd, [this]()
-    {
-        consume_wake_notification(timer_fd);
-        process_and_maybe_send_event();
-    });
-
-    dispatcher.add_watch(wake_fd, [this]()
-    {
-        consume_wake_notification(wake_fd);
-        process_and_maybe_send_event();
-    });
-
+    dispatcher.add_watch(wake_fd,
+                         [this]() { woke(); });
     dispatcher.add_watch(mir::Fd{mir::IntOwnedFd{input_channel->getFd()}},
-                         [this]() { process_and_maybe_send_event(); });
+                         [this]() { woke(); });
 }
 
 mircva::InputReceiver::InputReceiver(int fd,
                                      std::shared_ptr<mircv::XKBMapper> const& keymapper,
                                      std::function<void(MirEvent*)> const& event_handling_callback,
-                                     std::shared_ptr<mircv::InputReceiverReport> const& report,
-                                     AndroidClock clock)
+                                     std::shared_ptr<mircv::InputReceiverReport> const& report)
     : InputReceiver(new droidinput::InputChannel(droidinput::String8(""), fd),
                     keymapper,
                     event_handling_callback,
-                    report,
-                    clock)
+                    report)
 {
 }
 
@@ -138,35 +113,28 @@ static void map_key_event(std::shared_ptr<mircv::XKBMapper> const& xkb_mapper, M
 
 }
 
-void mircva::InputReceiver::process_and_maybe_send_event()
+void mircva::InputReceiver::woke()
 {
+    uint64_t dummy;
+
+    /*
+     * We never care about the cause of the wakeup. Either real input or a
+     * wake(). Either way the only requirement is that we woke() after each
+     * wake() or real input channel event.
+     */
+    if (read(wake_fd, &dummy, sizeof(dummy)) != sizeof(dummy) &&
+        errno != EAGAIN &&
+        errno != EINTR)
+    {
+        BOOST_THROW_EXCEPTION((std::system_error{errno,
+                                                 std::system_category(),
+                                                 "Failed to consume notification"}));
+    }
+
     droidinput::InputEvent *android_event;
     uint32_t event_sequence_id;
 
-    /*
-     * Enable "Project Butter" input resampling in InputConsumer::consume():
-     *   consumeBatches = true, so as to ensure the "cooked" event rate that
-     *      clients experience is at least the minimum of event_rate_hz
-     *      and the raw device event rate.
-     *   frame_time = A regular interval. This provides a virtual frame
-     *      interval during which InputConsumer will collect raw events,
-     *      resample them and emit a "cooked" event back to us at roughly every
-     *      60th of a second. "cooked" events are both smoothed and
-     *      extrapolated/predicted into the future (for tool=finger) giving the
-     *      appearance of lower latency. Getting a real frame time from the
-     *      graphics logic (which is messy) does not appear to be necessary to
-     *      gain significant benefit.
-     */
-
-    auto frame_time = std::chrono::nanoseconds(-1);
-    if (event_rate_hz > 0)
-    {
-        std::chrono::nanoseconds const
-            now = android_clock(SYSTEM_TIME_MONOTONIC),
-            one_frame = std::chrono::nanoseconds(1000000000ULL / event_rate_hz);
-        frame_time = (now / one_frame) * one_frame;
-    }
-
+    auto const frame_time = std::chrono::nanoseconds(-1);
     auto result = input_consumer->consume(&event_factory,
                                           true,
                                           frame_time,
@@ -196,35 +164,6 @@ void mircva::InputReceiver::process_and_maybe_send_event()
         // So, we ensure we'll appear dispatchable by pushing an event to the wakeup pipe.
         wake();
     }
-    else if (input_consumer->hasPendingBatch())
-    {
-        /* TODO: Feed in vsync-ish events (or work out when consume() would
-         *       actually generate an event) rather than polling at 1000Hz
-         */
-        using namespace std::chrono;
-        using namespace std::literals::chrono_literals;
-        struct itimerspec const msec_delay = {
-            { 0, 0 },
-            { 0, duration_cast<nanoseconds>(1ms).count() }
-        };
-        if (timerfd_settime(timer_fd, 0, &msec_delay, NULL) < 0)
-        {
-            BOOST_THROW_EXCEPTION((std::system_error{errno,
-                                                     std::system_category(),
-                                                     "Failed to arm timer"}));
-        }
-    }
-}
-
-void mircva::InputReceiver::consume_wake_notification(mir::Fd const& fd)
-{
-    uint64_t dummy;
-    if (read(fd, &dummy, sizeof(dummy)) != sizeof(dummy))
-    {
-        BOOST_THROW_EXCEPTION((std::system_error{errno,
-                                                 std::system_category(),
-                                                 "Failed to consume notification"}));
-    }
 }
 
 void mircva::InputReceiver::wake()
