@@ -18,12 +18,15 @@
 
 #include "mir_toolkit/mir_presentation_chain.h"
 #include "mir_toolkit/mir_buffer.h"
+#include "mir_toolkit/mir_buffer_private.h"
 #include "presentation_chain.h"
 #include "mir_connection.h"
 #include "buffer.h"
+#include "mir/client_buffer.h"
 #include "mir/require.h"
 #include "mir/uncaught.h"
 #include "mir/require.h"
+#include "mir/client_buffer.h"
 #include <stdexcept>
 #include <boost/throw_exception.hpp>
 
@@ -34,23 +37,59 @@ void mir_connection_allocate_buffer(
     MirConnection* connection, 
     int width, int height,
     MirPixelFormat format,
-    MirBufferUsage usage,
-    mir_buffer_callback cb, void* context)
+    MirBufferCallback cb, void* context)
 try
 {
     mir::require(connection);
-    connection->allocate_buffer(mir::geometry::Size{width, height}, format, usage, cb, context);
+    connection->allocate_buffer(mir::geometry::Size{width, height}, format, cb, context);
 }
 catch (std::exception const& ex)
 {
     MIR_LOG_UNCAUGHT_EXCEPTION(ex);
 }
 
+MirBuffer* mir_connection_allocate_buffer_sync(
+    MirConnection* connection, 
+    int width, int height,
+    MirPixelFormat format)
+try
+{
+    mir::require(connection);
+
+    struct BufferInfo
+    {
+        MirBuffer* buffer = nullptr;
+        std::mutex mutex;
+        std::condition_variable cv;
+    } info;
+    connection->allocate_buffer(mir::geometry::Size{width, height}, format,
+        [](MirBuffer* buffer, void* c)
+        {
+            mir::require(buffer);
+            mir::require(c);
+            auto context = reinterpret_cast<BufferInfo*>(c);
+            std::unique_lock<decltype(context->mutex)> lk(context->mutex);
+            context->buffer = buffer;
+            context->cv.notify_all();
+        }, &info);
+
+    std::unique_lock<decltype(info.mutex)> lk(info.mutex);
+    info.cv.wait(lk, [&]{ return info.buffer; });
+    return info.buffer;
+}
+catch (std::exception const& ex)
+{
+    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+    return nullptr;
+}
+
+void ignore(MirBuffer*, void*){}
 void mir_buffer_release(MirBuffer* b) 
 try
 {
     mir::require(b);
     auto buffer = reinterpret_cast<mcl::MirBuffer*>(b);
+    buffer->set_callback(ignore, nullptr);
     auto connection = buffer->allocating_connection();
     connection->release_buffer(buffer);
 }
@@ -59,75 +98,37 @@ catch (std::exception const& ex)
     MIR_LOG_UNCAUGHT_EXCEPTION(ex);
 }
 
-MirNativeFence* mir_buffer_get_fence(MirBuffer* b)
+bool mir_buffer_map(MirBuffer* b, MirGraphicsRegion* region, MirBufferLayout* layout)
 try
 {
     mir::require(b);
+    mir::require(region);
+    mir::require(layout);
     auto buffer = reinterpret_cast<mcl::MirBuffer*>(b);
-    return buffer->get_fence();
+    *layout = mir_buffer_layout_linear;
+    *region = buffer->map_region();
+    return true;
 }
 catch (std::exception const& ex)
 {
     MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-    return nullptr;
+    *region = MirGraphicsRegion { 0, 0, 0, mir_pixel_format_invalid, nullptr };
+    *layout = mir_buffer_layout_unknown;
+    return false;
 }
 
-void mir_buffer_associate_fence(MirBuffer* b, MirNativeFence* fence, MirBufferAccess access)
+void mir_buffer_unmap(MirBuffer* b)
 try
 {
     mir::require(b);
     auto buffer = reinterpret_cast<mcl::MirBuffer*>(b);
-    buffer->set_fence(fence, access);
+    buffer->unmap_region();
 }
 catch (std::exception const& ex)
 {
     MIR_LOG_UNCAUGHT_EXCEPTION(ex);
 }
 
-int mir_buffer_wait_for_access(MirBuffer* b, MirBufferAccess access, int timeout)
-try
-{
-    mir::require(b);
-    auto buffer = reinterpret_cast<mcl::MirBuffer*>(b);
-    return buffer->wait_fence(access, std::chrono::nanoseconds(timeout));
-}
-catch (std::exception const& ex)
-{
-    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-    return -1;
-}
-
-MirNativeBuffer* mir_buffer_get_native_buffer(MirBuffer* b, MirBufferAccess access) 
-try
-{
-    mir::require(b);
-    auto buffer = reinterpret_cast<mcl::MirBuffer*>(b);
-    if (!buffer->wait_fence(access, std::chrono::nanoseconds(-1)))
-        BOOST_THROW_EXCEPTION(std::runtime_error("error accessing MirNativeBuffer"));
-    return buffer->as_mir_native_buffer();
-}
-catch (std::exception const& ex)
-{
-    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-    return nullptr;
-}
-
-MirGraphicsRegion mir_buffer_get_graphics_region(MirBuffer* b, MirBufferAccess access)
-try
-{
-    mir::require(b);
-    auto buffer = reinterpret_cast<mcl::MirBuffer*>(b);
-    if (!buffer->wait_fence(access, std::chrono::nanoseconds(-1)))
-        BOOST_THROW_EXCEPTION(std::runtime_error("error accessing MirNativeBuffer"));
-
-    return buffer->map_region();
-}
-catch (std::exception const& ex)
-{
-    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-    return MirGraphicsRegion { 0, 0, 0, mir_pixel_format_invalid, nullptr };
-}
-
 unsigned int mir_buffer_get_width(MirBuffer* b)
 try
 {
@@ -167,19 +168,6 @@ catch (std::exception const& ex)
     return mir_pixel_format_invalid;
 }
 
-MirBufferUsage mir_buffer_get_buffer_usage(MirBuffer* b)
-try
-{
-    mir::require(b);
-    auto buffer = reinterpret_cast<mcl::MirBuffer*>(b);
-    return buffer->buffer_usage();
-}
-catch (std::exception const& ex)
-{
-    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-    return mir_buffer_usage_hardware;
-}
-
 bool mir_buffer_is_valid(MirBuffer* b)
 try
 {
@@ -204,15 +192,33 @@ catch (std::exception const& ex)
     return "MirBuffer: unknown error";
 }
 
-void mir_buffer_set_callback(
-    MirBuffer* b, mir_buffer_callback available_callback, void* available_context)
+MirBufferPackage* mir_buffer_get_buffer_package(MirBuffer* b)
 try
 {
     mir::require(b);
     auto buffer = reinterpret_cast<mcl::Buffer*>(b);
-    buffer->set_callback(available_callback, available_context);
+    return buffer->client_buffer()->package();
 }
 catch (std::exception const& ex)
 {
     MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+    return nullptr;
+}
+
+bool mir_buffer_get_egl_image_parameters(
+    MirBuffer* b, EGLenum* type, EGLClientBuffer* client_buffer, EGLint** attr)
+try
+{
+    mir::require(b);
+    mir::require(type);
+    mir::require(client_buffer);
+    mir::require(attr);
+    auto buffer = reinterpret_cast<mcl::Buffer*>(b);
+    buffer->client_buffer()->egl_image_creation_parameters(type, client_buffer, attr);
+    return true;
+}
+catch (std::exception const& ex)
+{
+    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+    return false;
 }
