@@ -24,14 +24,16 @@
 #include "rpc/mir_display_server.h"
 #include "rpc/mir_display_server_debug.h"
 
+#include "mir_toolkit/extensions/window_coordinate_translation.h"
+#include "mir_toolkit/extensions/graphics_module.h"
 #include "mir/geometry/size.h"
 #include "mir/client_platform.h"
 #include "mir/frontend/surface_id.h"
 #include "mir/client_context.h"
 #include "mir_toolkit/mir_client_library.h"
-#include "mir_toolkit/client_types_nbs.h"
 #include "mir_surface.h"
 #include "display_configuration.h"
+#include "error_handler.h"
 
 #include <atomic>
 #include <memory>
@@ -59,12 +61,13 @@ namespace client
 {
 class ConnectionConfiguration;
 class ClientPlatformFactory;
-class ClientBufferStream;
 class ConnectionSurfaceMap;
 class DisplayConfiguration;
 class EventHandlerRegister;
 class AsyncBufferFactory;
 class MirBuffer;
+class BufferStream;
+class PresentationChain;
 
 namespace rpc
 {
@@ -105,12 +108,12 @@ public:
     MirConnection& operator=(MirConnection const &) = delete;
 
     MirWaitHandle* create_surface(
-        MirSurfaceSpec const& spec,
-        mir_surface_callback callback,
+        MirWindowSpec const& spec,
+        MirWindowCallback callback,
         void * context);
     MirWaitHandle* release_surface(
-        MirSurface *surface,
-        mir_surface_callback callback,
+        MirWindow *surface,
+        MirWindowCallback callback,
         void *context);
 
     MirPromptSession* create_prompt_session();
@@ -119,23 +122,23 @@ public:
 
     MirWaitHandle* connect(
         const char* app_name,
-        mir_connected_callback callback,
+        MirConnectedCallback callback,
         void * context);
 
     MirWaitHandle* disconnect();
 
     MirWaitHandle* platform_operation(
         MirPlatformMessage const* request,
-        mir_platform_operation_callback callback, void* context);
+        MirPlatformOperationCallback callback, void* context) override;
 
-    void register_lifecycle_event_callback(mir_lifecycle_event_callback callback, void* context);
+    void register_lifecycle_event_callback(MirLifecycleEventCallback callback, void* context);
 
-    void register_ping_event_callback(mir_ping_event_callback callback, void* context);
+    void register_ping_event_callback(MirPingEventCallback callback, void* context);
     void pong(int32_t serial);
 
-    void register_display_change_callback(mir_display_config_callback callback, void* context);
+    void register_display_change_callback(MirDisplayConfigCallback callback, void* context);
 
-    void register_error_callback(mir_error_callback callback, void* context);
+    void register_error_callback(MirErrorCallback callback, void* context);
 
     void populate(MirPlatformPackage& platform_package);
     void populate_graphics_module(MirModuleProperties& properties) override;
@@ -144,43 +147,53 @@ public:
     void available_surface_formats(MirPixelFormat* formats,
                                    unsigned int formats_size, unsigned int& valid_formats);
 
-    std::shared_ptr<mir::client::ClientBufferStream> make_consumer_stream(
+    std::shared_ptr<MirBufferStream> make_consumer_stream(
        mir::protobuf::BufferStream const& protobuf_bs);
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
     MirWaitHandle* create_client_buffer_stream(
         int width, int height,
         MirPixelFormat format,
         MirBufferUsage buffer_usage,
-        mir_buffer_stream_callback callback,
+        MirRenderSurface* render_surface,
+        MirBufferStreamCallback mbs_callback,
         void *context);
+    std::shared_ptr<mir::client::BufferStream> create_client_buffer_stream_with_id(
+        int width, int height,
+        MirRenderSurface* render_surface,
+        mir::protobuf::BufferStream const& a_protobuf_bs);
     MirWaitHandle* release_buffer_stream(
-        mir::client::ClientBufferStream*,
-        mir_buffer_stream_callback callback,
+        MirBufferStream*,
+        MirBufferStreamCallback callback,
         void *context);
 
-    void create_presentation_chain(
-        mir_presentation_chain_callback callback,
-        void *context);
-    void release_presentation_chain(MirPresentationChain* context);
+    std::shared_ptr<mir::client::PresentationChain> create_presentation_chain_with_id(
+        MirRenderSurface* render_surface,
+        mir::protobuf::BufferStream const& a_protobuf_bs);
 
-    void release_consumer_stream(mir::client::ClientBufferStream*);
+    void release_consumer_stream(MirBufferStream*);
 
     static bool is_valid(MirConnection *connection);
 
     EGLNativeDisplayType egl_native_display();
     MirPixelFormat       egl_pixel_format(EGLDisplay, EGLConfig) const;
 
-    void on_stream_created(int id, mir::client::ClientBufferStream* stream);
+    void on_stream_created(int id, MirBufferStream* stream);
 
     MirWaitHandle* configure_display(MirDisplayConfiguration* configuration);
     void done_display_configure();
 
+    void configure_session_display(mir::protobuf::DisplayConfiguration const& configuration);
+    void remove_session_display();
+
     MirWaitHandle* set_base_display_configuration(MirDisplayConfiguration const* configuration);
     void preview_base_display_configuration(
         mir::protobuf::DisplayConfiguration const& configuration,
         std::chrono::seconds timeout);
     void confirm_base_display_configuration(
         mir::protobuf::DisplayConfiguration const& configuration);
+    void cancel_base_display_configuration_preview();
     void done_set_base_display_configuration();
 
     std::shared_ptr<mir::client::rpc::MirBasicRpcChannel> rpc_channel() const
@@ -195,24 +208,42 @@ public:
         return input_devices;
     }
 
+    std::shared_ptr<mir::client::ConnectionSurfaceMap> const& connection_surface_map() const
+    {
+        return surface_map;
+    }
+
     void allocate_buffer(
-        mir::geometry::Size size, MirPixelFormat format, MirBufferUsage usage,
-        mir_buffer_callback callback, void* context);
-    void release_buffer(mir::client::MirBuffer* buffer);
+        mir::geometry::Size size, MirPixelFormat format,
+        MirBufferCallback callback, void* context) override;
+    void allocate_buffer(
+        mir::geometry::Size size, uint32_t native_format, uint32_t native_flags,
+        MirBufferCallback callback, void* context) override;
+    void release_buffer(mir::client::MirBuffer* buffer) override;
+
+    auto create_render_surface_with_content(
+        mir::geometry::Size logical_size,
+        MirRenderSurfaceCallback callback,
+        void* context) -> MirRenderSurface*;
+#pragma GCC diagnostic pop
+    void release_render_surface_with_content(
+        void* render_surface);
+
+    void* request_interface(char const* name, int version);
 
 private:
     //google cant have callbacks with more than 2 args
     struct SurfaceCreationRequest
     {
-        SurfaceCreationRequest(mir_surface_callback cb, void* context,  MirSurfaceSpec const& spec) :
+        SurfaceCreationRequest(MirWindowCallback cb, void* context, MirWindowSpec const& spec) :
             cb(cb), context(context), spec(spec),
-            response(std::make_shared<mir::protobuf::Surface>()),
-            wh(std::make_shared<MirWaitHandle>())
+              response(std::make_shared<mir::protobuf::Surface>()),
+              wh(std::make_shared<MirWaitHandle>())
         {
         }
-        mir_surface_callback cb;
+        MirWindowCallback cb;
         void* context;
-        MirSurfaceSpec const spec;
+        MirWindowSpec const spec;
         std::shared_ptr<mir::protobuf::Surface> response;
         std::shared_ptr<MirWaitHandle> wh;
     };
@@ -221,13 +252,24 @@ private:
 
     struct StreamCreationRequest
     {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
         StreamCreationRequest(
-            mir_buffer_stream_callback cb, void* context, mir::protobuf::BufferStreamParameters const& params) :
-            callback(cb), context(context), parameters(params), response(std::make_shared<mir::protobuf::BufferStream>()),
-            wh(std::make_shared<MirWaitHandle>())
+            MirRenderSurface* rs,
+            MirBufferStreamCallback mbs_cb,
+            void* context,
+            mir::protobuf::BufferStreamParameters const& params)
+            : rs(rs),
+              mbs_callback(mbs_cb),
+              context(context),
+              parameters(params),
+              response(std::make_shared<mir::protobuf::BufferStream>()),
+              wh(std::make_shared<MirWaitHandle>())
         {
         }
-        mir_buffer_stream_callback callback;
+        MirRenderSurface* rs;
+#pragma GCC diagnostic pop
+        MirBufferStreamCallback mbs_callback;
         void* context;
         mir::protobuf::BufferStreamParameters const parameters;
         std::shared_ptr<mir::protobuf::BufferStream> response;
@@ -237,21 +279,35 @@ private:
     void stream_created(StreamCreationRequest*);
     void stream_error(std::string const& error_msg, std::shared_ptr<StreamCreationRequest> const& request);
 
-    struct ChainCreationRequest
+    struct RenderSurfaceCreationRequest
     {
-        ChainCreationRequest(mir_presentation_chain_callback cb, void* context) :
-            callback(cb), context(context),
-            response(std::make_shared<mir::protobuf::BufferStream>())
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+        RenderSurfaceCreationRequest(
+            MirRenderSurfaceCallback cb,
+            void* context,
+            std::shared_ptr<void> native_window,
+            mir::geometry::Size size) :
+                callback(cb), context(context),
+                response(std::make_shared<mir::protobuf::BufferStream>()),
+                wh(std::make_shared<MirWaitHandle>()),
+                native_window(native_window),
+                logical_size(size)
         {
         }
 
-        mir_presentation_chain_callback callback;
+        MirRenderSurfaceCallback callback;
+#pragma GCC diagnostic pop
         void* context;
         std::shared_ptr<mir::protobuf::BufferStream> response;
+        std::shared_ptr<MirWaitHandle> const wh;
+        std::shared_ptr<void> native_window;
+        mir::geometry::Size logical_size;
     };
-    std::vector<std::shared_ptr<ChainCreationRequest>> context_requests;
-    void context_created(ChainCreationRequest*);
-    void chain_error(std::string const& error_msg, std::shared_ptr<ChainCreationRequest> const& request);
+
+    std::vector<std::shared_ptr<RenderSurfaceCreationRequest>> render_surface_requests;
+    void render_surface_created(RenderSurfaceCreationRequest*);
+    void render_surface_error(std::string const& error_msg, std::shared_ptr<RenderSurfaceCreationRequest> const& request);
 
     void populate_server_package(MirPlatformPackage& platform_package) override;
     // MUST be first data member so it is destroyed last.
@@ -295,7 +351,7 @@ private:
     MirWaitHandle set_base_display_configuration_wait_handle;
 
     std::mutex release_wait_handle_guard;
-    std::vector<MirWaitHandle*> release_wait_handles;
+    std::vector<std::unique_ptr<MirWaitHandle>> release_wait_handles;
 
     std::shared_ptr<mir::client::DisplayConfiguration> const display_configuration;
     std::shared_ptr<mir::input::InputDevices> const input_devices;
@@ -304,6 +360,7 @@ private:
 
     std::shared_ptr<mir::client::PingHandler> const ping_handler;
 
+    std::shared_ptr<mir::client::ErrorHandler> error_handler;
 
     std::shared_ptr<mir::client::EventHandlerRegister> const event_handler_register;
 
@@ -311,7 +368,6 @@ private:
 
     std::unique_ptr<mir::dispatch::ThreadedDispatcher> const eventloop;
     
-    mir::client::AtomicCallback<MirError const*> error_handler;
 
     struct SurfaceRelease;
     struct StreamRelease;
@@ -320,13 +376,15 @@ private:
 
     void set_error_message(std::string const& error);
     void done_disconnect();
-    void connected(mir_connected_callback callback, void * context);
+    void connected(MirConnectedCallback callback, void * context);
     void released(SurfaceRelease);
     void released(StreamRelease);
-    void done_platform_operation(mir_platform_operation_callback, void* context);
+    void done_platform_operation(MirPlatformOperationCallback, void* context);
     bool validate_user_display_config(MirDisplayConfiguration const* config);
 
     int const nbuffers;
+    mir::optional_value<MirExtensionWindowCoordinateTranslationV1> translation_ext;
+    mir::optional_value<MirExtensionGraphicsModuleV1> graphics_module_extension;
 };
 
 #endif /* MIR_CLIENT_MIR_CONNECTION_H_ */
