@@ -18,7 +18,6 @@
 
 #define MIR_LOG_COMPONENT "MirConnectionAPI"
 
-#include "mir_connection_api.h"
 #include "mir_toolkit/mir_connection.h"
 #include "mir/default_configuration.h"
 #include "mir/input/input_devices.h"
@@ -36,6 +35,7 @@
 
 namespace mcl = mir::client;
 namespace mp = mir::protobuf;
+namespace mi = mir::input;
 
 namespace
 {
@@ -45,118 +45,65 @@ void assign_result(void* result, void** context)
     if (context)
         *context = result;
 }
-
-class DefaultMirConnectionAPI : public mcl::MirConnectionAPI
-{
-public:
-    MirWaitHandle* connect(
-        mcl::ConfigurationFactory configuration,
-        char const* socket_file,
-        char const* name,
-        mir_connected_callback callback,
-        void* context) override
-    {
-        try
-        {
-            std::string sock;
-            if (socket_file)
-                sock = socket_file;
-            else
-            {
-                auto socket_env = getenv("MIR_SOCKET");
-                if (socket_env)
-                    sock = socket_env;
-                else
-                    sock = mir::default_server_socket;
-            }
-
-            auto const conf = configuration(sock);
-
-            auto connection = std::make_unique<MirConnection>(*conf);
-            auto const result = connection->connect(name, callback, context);
-            connection.release();
-            return result;
-        }
-        catch (std::exception const& x)
-        {
-            MirConnection* error_connection = new MirConnection(x.what());
-            mcl::ErrorConnections::instance().insert(error_connection);
-            callback(error_connection, context);
-            return nullptr;
-        }
-    }
-
-    void release(MirConnection* connection) override
-    {
-        if (!mcl::ErrorConnections::instance().contains(connection))
-        {
-            try
-            {
-                auto wait_handle = connection->disconnect();
-                wait_handle->wait_for_all();
-            }
-            catch (std::exception const& ex)
-            {
-                // We're implementing a C API so no exceptions are to be
-                // propagated. And that's OK because if disconnect() fails,
-                // we don't care why. We're finished with the connection anyway.
-
-                MIR_LOG_UNCAUGHT_EXCEPTION(ex);
-            }
-        }
-        else
-        {
-            mcl::ErrorConnections::instance().remove(connection);
-        }
-
-        delete connection;
-    }
-
-    mcl::ConfigurationFactory configuration_factory() override
-    {
-        return [](std::string const& socket) {
-            return std::unique_ptr<mcl::ConnectionConfiguration>{
-                new mcl::DefaultConnectionConfiguration{socket}
-            };
-        };
-    }
-};
-
-DefaultMirConnectionAPI default_api;
 }
 
-mcl::MirConnectionAPI* mir_connection_api_impl{&default_api};
-
 MirWaitHandle* mir_connect(
     char const* socket_file,
     char const* name,
-    mir_connected_callback callback,
+    MirConnectedCallback callback,
     void* context)
+try
 {
     try
     {
-        return mir_connection_api_impl->connect(mir_connection_api_impl->configuration_factory(),
-                                                socket_file,
-                                                name,
-                                                callback,
-                                                context);
+        std::string sock;
+        if (socket_file)
+            sock = socket_file;
+        else
+        {
+            auto socket_env = getenv("MIR_SOCKET");
+            if (socket_env && socket_env[0])
+                sock = socket_env;
+            else
+                sock = mir::default_server_socket;
+        }
+
+        mcl::DefaultConnectionConfiguration conf(sock);
+
+        auto connection = std::make_unique<MirConnection>(conf);
+        auto const result = connection->connect(name, callback, context);
+        connection.release();
+        return result;
     }
-    catch (std::exception const& ex)
+    catch (std::exception const& x)
     {
-        MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+        MirConnection* error_connection = new MirConnection(x.what());
+        mcl::ErrorConnections::instance().insert(error_connection);
+        callback(error_connection, context);
         return nullptr;
     }
 }
+catch (std::exception const& ex)
+{
+    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+    return nullptr;
+}
 
 MirConnection* mir_connect_sync(
     char const* server,
     char const* app_name)
 {
     MirConnection* conn = nullptr;
-    mir_wait_for(mir_connect(server, app_name,
-                             reinterpret_cast<mir_connected_callback>
-                                             (assign_result),
-                             &conn));
+    auto wh = mir_connect(server, app_name,
+                          reinterpret_cast<MirConnectedCallback>
+                          (assign_result),
+                          &conn);
+
+    if (wh != nullptr)
+    {
+        wh->wait_for_all();
+    }
+
     return conn;
 }
 
@@ -171,16 +118,26 @@ char const* mir_connection_get_error_message(MirConnection* connection)
 }
 
 void mir_connection_release(MirConnection* connection)
+try
 {
-    try
+    std::unique_ptr<MirConnection> scoped_deleter{connection};
+    if (!mcl::ErrorConnections::instance().contains(connection))
     {
-        return mir_connection_api_impl->release(connection);
+        auto wait_handle = connection->disconnect();
+        wait_handle->wait_for_all();
     }
-    catch (std::exception const& ex)
+    else
     {
-        MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+        mcl::ErrorConnections::instance().remove(connection);
     }
 }
+catch (std::exception const& ex)
+{
+    // We're implementing a C API so no exceptions are to be
+    // propagated. And that's OK because if disconnect() fails,
+    // we don't care why. We're finished with the connection anyway.
+    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+}
 
 void mir_connection_get_platform(
     MirConnection* connection,
@@ -204,7 +161,7 @@ catch (std::exception const& ex)
 
 void mir_connection_set_lifecycle_event_callback(
     MirConnection* connection,
-    mir_lifecycle_event_callback callback,
+    MirLifecycleEventCallback callback,
     void* context)
 {
     if (!mcl::ErrorConnections::instance().contains(connection))
@@ -213,7 +170,7 @@ void mir_connection_set_lifecycle_event_callback(
 
 void mir_connection_set_ping_event_callback(
     MirConnection* connection,
-    mir_ping_event_callback callback,
+    MirPingEventCallback callback,
     void* context)
 {
     if (!mcl::ErrorConnections::instance().contains(connection))
@@ -239,18 +196,17 @@ MirDisplayConfig* mir_connection_create_display_configuration(
 {
     mir::require(mir_connection_is_valid(connection));
 
-    return reinterpret_cast<MirDisplayConfig*>(connection->snapshot_display_configuration().release());
+    return new MirDisplayConfig(*connection->snapshot_display_configuration());
 }
 
-void mir_display_config_release(MirDisplayConfig* user_config)
+void mir_display_config_release(MirDisplayConfig* config)
 {
-    auto config = reinterpret_cast<mir::protobuf::DisplayConfiguration*>(user_config);
     delete config;
 }
 
 void mir_connection_set_display_config_change_callback(
     MirConnection* connection,
-    mir_display_config_callback callback,
+    MirDisplayConfigCallback callback,
     void* context)
 {
     if (connection)
@@ -294,16 +250,22 @@ MirWaitHandle* mir_connection_set_base_display_config(
 
 MirInputConfig* mir_connection_create_input_config(
     MirConnection* connection)
+try
 {
     mir::require(mir_connection_is_valid(connection));
 
-    auto devices = connection->the_input_devices();
-    return reinterpret_cast<MirInputConfig*>(new std::vector<mir::input::DeviceData>(devices->copy_devices()));
+    auto wrapper = connection->the_input_devices();
+    return new MirInputConfig{wrapper->devices()};
+}
+catch (std::exception const& ex)
+{
+    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+    return nullptr;
 }
 
 void mir_connection_set_input_config_change_callback(
     MirConnection* connection,
-    mir_input_config_callback callback,
+    MirInputConfigCallback callback,
     void* context)
 {
     if (!connection)
@@ -314,8 +276,12 @@ void mir_connection_set_input_config_change_callback(
 
 void mir_input_config_destroy(MirInputConfig const* config)
 {
-    auto device_vector = reinterpret_cast<std::vector<mir::input::DeviceData> const*>(config);
-    delete device_vector;
+    mir_input_config_release(config);
+}
+
+void mir_input_config_release(MirInputConfig const* config)
+{
+    delete config;
 }
 
 void mir_connection_preview_base_display_configuration(
@@ -327,9 +293,7 @@ void mir_connection_preview_base_display_configuration(
 
     try
     {
-        connection->preview_base_display_configuration(
-            *reinterpret_cast<mp::DisplayConfiguration const*>(config),
-            std::chrono::seconds{timeout_seconds});
+        connection->preview_base_display_configuration(*config, std::chrono::seconds{timeout_seconds});
     }
     catch (std::exception const& ex)
     {
@@ -345,8 +309,22 @@ void mir_connection_confirm_base_display_configuration(
 
     try
     {
-        connection->confirm_base_display_configuration(
-            *reinterpret_cast<mp::DisplayConfiguration const*>(config));
+        connection->confirm_base_display_configuration(*config);
+    }
+    catch (std::exception const& ex)
+    {
+        MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+    }
+}
+
+void mir_connection_cancel_base_display_configuration_preview(
+    MirConnection* connection)
+{
+    mir::require(mir_connection_is_valid(connection));
+
+    try
+    {
+        connection->cancel_base_display_configuration_preview();
     }
     catch (std::exception const& ex)
     {
@@ -379,7 +357,7 @@ void mir_connection_get_available_surface_formats(
 MirWaitHandle* mir_connection_platform_operation(
     MirConnection* connection,
     MirPlatformMessage const* request,
-    mir_platform_operation_callback callback, void* context)
+    MirPlatformOperationCallback callback, void* context)
 {
     try
     {
@@ -394,7 +372,7 @@ MirWaitHandle* mir_connection_platform_operation(
 
 void mir_connection_set_error_callback(
     MirConnection* connection,
-    mir_error_callback callback,
+    MirErrorCallback callback,
     void* context)
 {
     mir::require(mir_connection_is_valid(connection));
@@ -409,3 +387,22 @@ void mir_connection_set_error_callback(
     }
 }
 
+void mir_connection_apply_session_display_config(MirConnection* connection, MirDisplayConfig const* display_config)
+try
+{
+    connection->configure_session_display(*display_config);
+}
+catch (std::exception const& ex)
+{
+    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+}
+
+void mir_connection_remove_session_display_config(MirConnection* connection)
+try
+{
+    connection->remove_session_display();
+}
+catch (std::exception const& ex)
+{
+    MIR_LOG_UNCAUGHT_EXCEPTION(ex);
+}
