@@ -42,8 +42,7 @@ namespace rpc
 {
 class DisplayServer;
 }
-class ClientBufferStreamFactory;
-class ClientBufferStream;
+class MirBufferStreamFactory;
 }
 }
 
@@ -73,29 +72,29 @@ public:
     MirScreencast(
         MirScreencastSpec const& spec,
         mir::client::rpc::DisplayServer& server,
-        mir_screencast_callback callback, void* context);
+        MirScreencastCallback callback, void* context);
 
     MirWaitHandle* creation_wait_handle();
     bool valid();
     char const* get_error_message();
 
     MirWaitHandle* release(
-        mir_screencast_callback callback, void* context);
+        MirScreencastCallback callback, void* context);
 
     EGLNativeWindowType egl_native_window();
 
-    mir::client::ClientBufferStream* get_buffer_stream();
+    MirBufferStream* get_buffer_stream();
 
 private:
     void screencast_created(
-        mir_screencast_callback callback, void* context);
+        MirScreencastCallback callback, void* context);
     void released(
-        mir_screencast_callback callback, void* context);
+        MirScreencastCallback callback, void* context);
 
     std::mutex mutable mutex;
     mir::client::rpc::DisplayServer* const server{nullptr};
     MirConnection* const connection{nullptr};
-    std::shared_ptr<mir::client::ClientBufferStream> buffer_stream;
+    std::shared_ptr<MirBufferStream> buffer_stream;
 
     std::unique_ptr<mir::protobuf::Screencast> const protobuf_screencast;
     std::unique_ptr<mir::protobuf::Void> const protobuf_void;
