@@ -1,5 +1,5 @@
 /*
- * Copyright © 2012, 2015 Canonical Ltd.
+ * Copyright © 2012-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License version 3,
@@ -19,18 +19,20 @@
 #include "mir_surface.h"
 #include "mir_connection.h"
 #include "cursor_configuration.h"
-#include "client_buffer_stream.h"
 #include "make_protobuf_object.h"
 #include "mir_protobuf.pb.h"
+#include "connection_surface_map.h"
 
 #include "mir_toolkit/mir_client_library.h"
 #include "mir/frontend/client_constants.h"
 #include "mir/client_buffer.h"
+#include "mir/mir_buffer_stream.h"
 #include "mir/dispatch/threaded_dispatcher.h"
 #include "mir/input/input_platform.h"
 #include "mir/input/xkb_mapper.h"
 #include "mir/cookie/cookie.h"
 #include "mir_cookie.h"
+#include "mir/time/posix_timestamp.h"
 
 #include <cassert>
 #include <unistd.h>
@@ -46,10 +48,25 @@ namespace mp = mir::protobuf;
 namespace gp = google::protobuf;
 namespace md = mir::dispatch;
 
+using mir::client::FrameClock;
+
 namespace
 {
 std::mutex handle_mutex;
-std::unordered_set<MirSurface*> valid_surfaces;
+std::unordered_set<MirWindow*> valid_surfaces;
+}
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+
+MirPersistentId::MirPersistentId(std::string const& string_id)
+    : string_id{string_id}
+{
+}
+
+std::string const& MirPersistentId::as_string()
+{
+    return string_id;
 }
 
 MirSurfaceSpec::MirSurfaceSpec(
@@ -62,33 +79,23 @@ MirSurfaceSpec::MirSurfaceSpec(
 {
 }
 
-MirSurfaceSpec::MirSurfaceSpec(MirConnection* connection, MirSurfaceParameters const& params)
+MirSurfaceSpec::MirSurfaceSpec(MirConnection* connection, MirWindowParameters const& params)
     : connection{connection},
       width{params.width},
       height{params.height},
       pixel_format{params.pixel_format},
       buffer_usage{params.buffer_usage}
 {
-    type = mir_surface_type_normal;
+    type = mir_window_type_normal;
     if (params.output_id != mir_display_output_id_invalid)
     {
         output_id = params.output_id;
-        state = mir_surface_state_fullscreen;
+        state = mir_window_state_fullscreen;
     }
 }
 
 MirSurfaceSpec::MirSurfaceSpec() = default;
 
-MirPersistentId::MirPersistentId(std::string const& string_id)
-    : string_id{string_id}
-{
-}
-
-std::string const&MirPersistentId::as_string()
-{
-    return string_id;
-}
-
 MirSurface::MirSurface(
     std::string const& error,
     MirConnection* conn,
@@ -96,6 +103,7 @@ MirSurface::MirSurface(
     std::shared_ptr<MirWaitHandle> const& handle) :
     surface{mcl::make_protobuf_object<mir::protobuf::Surface>()},
     connection_(conn),
+    frame_clock(std::make_shared<FrameClock>()),
     creation_handle(handle)
 {
     surface->set_error(error);
@@ -103,15 +111,17 @@ MirSurface::MirSurface(
 
     std::lock_guard<decltype(handle_mutex)> lock(handle_mutex);
     valid_surfaces.insert(this);
+
+    configure_frame_clock();
 }
 
 MirSurface::MirSurface(
     MirConnection *allocating_connection,
     mclr::DisplayServer& the_server,
     mclr::DisplayServerDebug* debug,
-    std::shared_ptr<mcl::ClientBufferStream> const& buffer_stream,
+    std::shared_ptr<MirBufferStream> const& buffer_stream,
     std::shared_ptr<mircv::InputPlatform> const& input_platform,
-    MirSurfaceSpec const& spec,
+    MirWindowSpec const& spec,
     mir::protobuf::Surface const& surface_proto,
     std::shared_ptr<MirWaitHandle> const& handle)
     : server{&the_server},
@@ -126,12 +136,31 @@ MirSurface::MirSurface(
       input_platform(input_platform),
       keymapper(std::make_shared<mircv::XKBMapper>()),
       configure_result{mcl::make_protobuf_object<mir::protobuf::SurfaceSetting>()},
+      frame_clock(std::make_shared<FrameClock>()),
       creation_handle(handle),
       size({surface_proto.width(), surface_proto.height()}),
       format(static_cast<MirPixelFormat>(surface_proto.pixel_format())),
       usage(static_cast<MirBufferUsage>(surface_proto.buffer_usage())),
       output_id(spec.output_id.is_set() ? spec.output_id.value() : static_cast<uint32_t>(mir_display_output_id_invalid))
 {
+    if (default_stream)
+        streams.insert(default_stream);
+
+    if (spec.streams.is_set())
+    {
+        auto& map = connection_->connection_surface_map();
+        auto const& spec_streams = spec.streams.value();
+        for (auto& ci : spec_streams)
+        {
+            mir::frontend::BufferStreamId id(ci.stream_id);
+            if (auto bs = map->stream(id))
+                streams.insert(bs);
+        }
+    }
+
+    for (auto& s : streams)
+        s->adopted_by(this);
+
     for(int i = 0; i < surface_proto.attributes_size(); i++)
     {
         auto const& attrib = surface_proto.attributes(i);
@@ -155,10 +184,23 @@ MirSurface::MirSurface(
 
     std::lock_guard<decltype(handle_mutex)> lock(handle_mutex);
     valid_surfaces.insert(this);
+
+    configure_frame_clock();
 }
 
 MirSurface::~MirSurface()
 {
+    StreamSet old_streams;
+
+    {
+        std::lock_guard<decltype(mutex)> lock(mutex);
+        old_streams = std::move(streams);
+    }
+
+    // Do callbacks without holding the lock
+    for (auto const& s : old_streams)
+        s->unadopted_by(this);
+
     {
         std::lock_guard<decltype(handle_mutex)> lock(handle_mutex);
         valid_surfaces.erase(this);
@@ -172,7 +214,19 @@ MirSurface::~MirSurface()
         close(surface->fd(i));
 }
 
-MirSurfaceParameters MirSurface::get_parameters() const
+void MirSurface::configure_frame_clock()
+{
+    /*
+     * TODO: Implement frame_clock->set_resync_callback(...) when IPC to get
+     *       timestamps from the server exists.
+     *       Until we do, client-side vsync will perform suboptimally (it is
+     *       randomly up to one frame out of phase with the real display).
+     *       However even while it's suboptimal it's dramatically lower latency
+     *       than the old approach and still totally eliminates nesting lag.
+     */
+}
+
+MirWindowParameters MirSurface::get_parameters() const
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
 
@@ -207,7 +261,7 @@ bool MirSurface::is_valid(MirSurface* query)
     return false;
 }
 
-void MirSurface::acquired_persistent_id(mir_surface_id_callback callback, void* context)
+void MirSurface::acquired_persistent_id(MirWindowIdCallback callback, void* context)
 {
     if (!persistent_id->has_error())
     {
@@ -220,7 +274,7 @@ void MirSurface::acquired_persistent_id(mir_surface_id_callback callback, void*
     persistent_id_wait_handle.result_received();
 }
 
-MirWaitHandle* MirSurface::request_persistent_id(mir_surface_id_callback callback, void* context)
+MirWaitHandle* MirSurface::request_persistent_id(MirWindowIdCallback callback, void* context)
 {
     std::lock_guard<decltype(mutex)> lock{mutex};
 
@@ -279,13 +333,13 @@ MirWaitHandle* MirSurface::configure_cursor(MirCursorConfiguration const* cursor
     return &configure_cursor_wait_handle;
 }
 
-MirWaitHandle* MirSurface::configure(MirSurfaceAttrib at, int value)
+MirWaitHandle* MirSurface::configure(MirWindowAttrib at, int value)
 {
     // TODO: This is obviously strange. It should be
     // possible to eliminate it in the second phase of buffer
     // stream where the existing MirSurface swap interval functions
     // may be deprecated in terms of mir_buffer_stream_ alternatives
-    if ((at == mir_surface_attrib_swapinterval) && default_stream)
+    if ((at == mir_window_attrib_swapinterval) && default_stream)
     {
         default_stream->set_swap_interval(value);
         return &configure_wait_handle;
@@ -360,11 +414,11 @@ void MirSurface::on_configured()
 
         switch (a)
         {
-        case mir_surface_attrib_type:
-        case mir_surface_attrib_state:
-        case mir_surface_attrib_focus:
-        case mir_surface_attrib_dpi:
-        case mir_surface_attrib_preferred_orientation:
+        case mir_window_attrib_type:
+        case mir_window_attrib_state:
+        case mir_window_attrib_focus:
+        case mir_window_attrib_dpi:
+        case mir_window_attrib_preferred_orientation:
             if (configure_result->has_ivalue())
                 attrib_cache[a] = configure_result->ivalue();
             else
@@ -385,11 +439,11 @@ void MirSurface::on_cursor_configured()
 }
 
 
-int MirSurface::attrib(MirSurfaceAttrib at) const
+int MirSurface::attrib(MirWindowAttrib at) const
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
 
-    if (at == mir_surface_attrib_swapinterval)
+    if (at == mir_window_attrib_swapinterval)
     {
         if (default_stream)
             return default_stream->swap_interval();
@@ -400,7 +454,7 @@ int MirSurface::attrib(MirSurfaceAttrib at) const
     return attrib_cache[at];
 }
 
-void MirSurface::set_event_handler(mir_surface_event_callback callback,
+void MirSurface::set_event_handler(MirWindowEventCallback callback,
                                    void* context)
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
@@ -430,12 +484,12 @@ void MirSurface::handle_event(MirEvent const& e)
 
     switch (mir_event_get_type(&e))
     {
-    case mir_event_type_surface:
+    case mir_event_type_window:
     {
-        auto sev = mir_event_get_surface_event(&e);
-        auto a = mir_surface_event_get_attribute(sev);
-        if (a < mir_surface_attribs)
-            attrib_cache[a] = mir_surface_event_get_attribute_value(sev);
+        auto sev = mir_event_get_window_event(&e);
+        auto a = mir_window_event_get_attribute(sev);
+        if (a < mir_window_attribs)
+            attrib_cache[a] = mir_window_event_get_attribute_value(sev);
         break;
     }
     case mir_event_type_orientation:
@@ -458,6 +512,36 @@ void MirSurface::handle_event(MirEvent const& e)
             default_stream->set_size(size);
         break;
     }
+    case mir_event_type_window_output:
+    {
+        /*
+         * We set the frame clock according to the (maximum) refresh rate of
+         * the display. Note that we don't do the obvious thing and measure
+         * previous vsyncs mainly because that's a very unreliable predictor
+         * of the desired maximum refresh rate in the case of GSync/FreeSync
+         * and panel self-refresh. You can't assume the previous frame
+         * represents the display running at full native speed and so should
+         * not measure it. Instead we conveniently have
+         * mir_window_output_event_get_refresh_rate that tells us the full
+         * native speed of the most relevant output...
+         */
+        auto soevent = mir_event_get_surface_output_event(&e);
+        auto rate = mir_surface_output_event_get_refresh_rate(soevent);
+        if (rate > 10.0)  // should be >0, but 10 to workaround LP: #1639725
+        {
+            std::chrono::nanoseconds const ns(
+                static_cast<long>(1000000000L / rate));
+            frame_clock->set_period(ns);
+        }
+        /* else: The graphics driver has not provided valid timing so we will
+         *       default to swap interval 0 behaviour.
+         *       Or would people prefer some different fallback?
+         *         - Re-enable server-side vsync?
+         *         - Rate limit to 60Hz?
+         *         - Just fix LP: #1639725?
+         */
+        break;
+    }
     default:
         break;
     };
@@ -470,7 +554,7 @@ void MirSurface::handle_event(MirEvent const& e)
     }
 }
 
-void MirSurface::request_and_wait_for_configure(MirSurfaceAttrib a, int value)
+void MirSurface::request_and_wait_for_configure(MirWindowAttrib a, int value)
 {
     configure(a, value)->wait_for_all();
 }
@@ -484,7 +568,7 @@ MirOrientation MirSurface::get_orientation() const
 
 MirWaitHandle* MirSurface::set_preferred_orientation(MirOrientationMode mode)
 {
-    return configure(mir_surface_attrib_preferred_orientation, mode);
+    return configure(mir_window_attrib_preferred_orientation, mode);
 }
 
 void MirSurface::raise_surface(MirCookie const* cookie)
@@ -496,7 +580,7 @@ void MirSurface::raise_surface(MirCookie const* cookie)
 
     auto const event_cookie = raise_request.mutable_cookie();
 
-    event_cookie->set_cookie(cookie->blob().data(), cookie->size());
+    event_cookie->set_cookie(cookie->cookie().data(), cookie->size());
 
     server->raise_surface(
         &raise_request,
@@ -504,7 +588,7 @@ void MirSurface::raise_surface(MirCookie const* cookie)
         google::protobuf::NewCallback(google::protobuf::DoNothing));
 }
 
-mir::client::ClientBufferStream* MirSurface::get_buffer_stream()
+MirBufferStream* MirSurface::get_buffer_stream()
 {
     std::lock_guard<decltype(mutex)> lock(mutex);
     
@@ -523,7 +607,7 @@ void MirSurface::on_modified()
     modify_wait_handle.result_received();
 }
 
-MirWaitHandle* MirSurface::modify(MirSurfaceSpec const& spec)
+MirWaitHandle* MirSurface::modify(MirWindowSpec const& spec)
 {
     mp::SurfaceModifications mods;
 
@@ -550,6 +634,11 @@ MirWaitHandle* MirSurface::modify(MirSurfaceSpec const& spec)
     // parent_id is a special case (below)
     // aux_rect is a special case (below)
     COPY_IF_SET(edge_attachment);
+    COPY_IF_SET(aux_rect_placement_gravity);
+    COPY_IF_SET(surface_placement_gravity);
+    COPY_IF_SET(placement_hints);
+    COPY_IF_SET(aux_rect_placement_offset_x);
+    COPY_IF_SET(aux_rect_placement_offset_y);
     COPY_IF_SET(min_width);
     COPY_IF_SET(min_height);
     COPY_IF_SET(max_width);
@@ -558,6 +647,7 @@ MirWaitHandle* MirSurface::modify(MirSurfaceSpec const& spec)
     COPY_IF_SET(height_inc);
     COPY_IF_SET(shell_chrome);
     COPY_IF_SET(confine_pointer);
+    COPY_IF_SET(cursor_name);
     // min_aspect is a special case (below)
     // max_aspect is a special case (below)
     #undef COPY_IF_SET
@@ -603,7 +693,30 @@ MirWaitHandle* MirSurface::modify(MirSurfaceSpec const& spec)
 
     if (spec.streams.is_set())
     {
-        default_stream = nullptr;
+        std::shared_ptr<mir::client::ConnectionSurfaceMap> map;
+        StreamSet old_streams, new_streams;
+
+        /*
+         * Note that we are updating our local copy of 'streams' before the
+         * server has updated its copy. That's mainly because we have never yet
+         * implemented any kind of client-side error handling for when
+         * modify_surface fails and is rejected. There's a prototype started
+         * along those lines in lp:~vanvugt/mir/adoption, but it does not seem
+         * to be important to block on...
+         *   The worst case is still perfectly acceptable: That is if the server
+         * rejects the surface spec our local copy of streams is wrong. That's
+         * a harmless consequence which just means the stream is now
+         * synchronized to the output the client intended even though the server
+         * is refusing to display it.
+         */
+        {
+            std::lock_guard<decltype(mutex)> lock(mutex);
+            default_stream = nullptr;
+            old_streams = std::move(streams);
+            streams.clear();  // Leave the container valid before we unlock
+            map = connection_->connection_surface_map();
+        }
+
         for(auto const& stream : spec.streams.value())
         {
             auto const new_stream = surface_specification->add_stream();
@@ -615,6 +728,27 @@ MirWaitHandle* MirSurface::modify(MirSurfaceSpec const& spec)
                 new_stream->set_width(stream.size.value().width.as_int());
                 new_stream->set_height(stream.size.value().height.as_int());
             }
+
+            mir::frontend::BufferStreamId id(stream.stream_id);
+            if (auto bs = map->stream(id))
+                new_streams.insert(bs);
+            /* else: we could implement stream ID validation here, which we've
+             * never had before.
+             * The only problem with that plan is that we have some tests
+             * that rely on the ability to pass in invalid stream IDs so those
+             * need fixing.
+             */
+        }
+
+        // Worth optimizing and avoiding the intersection of both sets?....
+        for (auto const& old_stream : old_streams)
+            old_stream->unadopted_by(this);
+        for (auto const& new_stream : new_streams)
+            new_stream->adopted_by(this);
+        // ... probably not, since we can std::move(new_streams) this way...
+        {
+            std::unique_lock<decltype(mutex)> lock(mutex);
+            streams = std::move(new_streams);
         }
     }
 
@@ -630,6 +764,21 @@ MirWaitHandle* MirSurface::modify(MirSurfaceSpec const& spec)
         }
     }
 
+    if (spec.rendersurface_cursor.is_set())
+    {
+        auto const rs_cursor = spec.rendersurface_cursor.value();
+        surface_specification->mutable_cursor_id()->set_value(rs_cursor.id.as_value());
+        surface_specification->set_hotspot_x(rs_cursor.hotspot.x.as_int());
+        surface_specification->set_hotspot_y(rs_cursor.hotspot.y.as_int());
+    }
+
+    if (spec.event_handler.is_set())
+    {
+        set_event_handler(
+            spec.event_handler.value().callback,
+            spec.event_handler.value().context);
+    }
+
     modify_wait_handle.expect_result();
     server->modify_surface(&mods, modify_result.get(),
               google::protobuf::NewCallback(this, &MirSurface::on_modified));
@@ -641,3 +790,10 @@ MirConnection* MirSurface::connection() const
 {
     return connection_;
 }
+
+std::shared_ptr<FrameClock> MirSurface::get_frame_clock() const
+{
+    return frame_clock;
+}
+
+#pragma GCC diagnostic pop
