@@ -1,5 +1,5 @@
 /*
- * Copyright © 2012, 2015 Canonical Ltd.
+ * Copyright © 2012-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License version 3,
@@ -18,9 +18,10 @@
 #ifndef MIR_CLIENT_MIR_SURFACE_H_
 #define MIR_CLIENT_MIR_SURFACE_H_
 
-#include "client_buffer_depository.h"
-#include "client_buffer_stream.h"
+#include "cursor_configuration.h"
+#include "mir/mir_buffer_stream.h"
 #include "mir_wait_handle.h"
+#include "frame_clock.h"
 #include "rpc/mir_display_server.h"
 #include "rpc/mir_display_server_debug.h"
 
@@ -32,6 +33,7 @@
 #include "mir_toolkit/common.h"
 #include "mir_toolkit/mir_client_library.h"
 #include "mir/graphics/native_buffer.h"
+#include "mir/time/posix_timestamp.h"
 
 #include <memory>
 #include <functional>
@@ -69,8 +71,7 @@ class DisplayServerDebug;
 }
 
 class ClientBuffer;
-class ClientBufferStream;
-class ClientBufferStreamFactory;
+class MirBufferStreamFactory;
 
 struct MemoryRegion;
 }
@@ -83,11 +84,25 @@ struct ContentInfo
     mir::optional_value<mir::geometry::Size> size;
 };
 
+struct MirPersistentId
+{
+public:
+    MirPersistentId(std::string const& string_id);
+
+    std::string const& as_string();
+
+private:
+    std::string const string_id;
+};
+
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+
 struct MirSurfaceSpec
 {
     MirSurfaceSpec();
     MirSurfaceSpec(MirConnection* connection, int width, int height, MirPixelFormat format);
-    MirSurfaceSpec(MirConnection* connection, MirSurfaceParameters const& params);
+    MirSurfaceSpec(MirConnection* connection, MirWindowParameters const& params);
 
     struct AspectRatio { unsigned width; unsigned height; };
 
@@ -103,14 +118,19 @@ struct MirSurfaceSpec
     mir::optional_value<std::string> surface_name;
     mir::optional_value<uint32_t> output_id;
 
-    mir::optional_value<MirSurfaceType> type;
-    mir::optional_value<MirSurfaceState> state;
+    mir::optional_value<MirWindowType> type;
+    mir::optional_value<MirWindowState> state;
     mir::optional_value<MirOrientationMode> pref_orientation;
 
-    mir::optional_value<MirSurface*> parent;
+    mir::optional_value<MirWindow*> parent;
     std::shared_ptr<MirPersistentId> parent_id;
     mir::optional_value<MirRectangle> aux_rect;
     mir::optional_value<MirEdgeAttachment> edge_attachment;
+    mir::optional_value<MirPlacementHints> placement_hints;
+    mir::optional_value<MirPlacementGravity> surface_placement_gravity;
+    mir::optional_value<MirPlacementGravity> aux_rect_placement_gravity;
+    mir::optional_value<int> aux_rect_placement_offset_x;
+    mir::optional_value<int> aux_rect_placement_offset_y;
 
     mir::optional_value<int> min_width;
     mir::optional_value<int> min_height;
@@ -126,22 +146,19 @@ struct MirSurfaceSpec
 
     struct EventHandler
     {
-        mir_surface_event_callback callback;
+        MirWindowEventCallback callback;
         void* context;
     };
     mir::optional_value<EventHandler> event_handler;
     mir::optional_value<MirShellChrome> shell_chrome;
-};
-
-struct MirPersistentId
-{
-public:
-    MirPersistentId(std::string const& string_id);
-
-    std::string const& as_string();
-
-private:
-    std::string const string_id;
+    
+    mir::optional_value<std::string> cursor_name;
+    struct RenderSurfaceCursor
+    {
+        mir::frontend::BufferStreamId id;
+        mir::geometry::Point hotspot;
+    };
+    mir::optional_value<RenderSurfaceCursor> rendersurface_cursor;
 };
 
 struct MirSurface
@@ -160,18 +177,18 @@ public:
         MirConnection *allocating_connection,
         mir::client::rpc::DisplayServer& server,
         mir::client::rpc::DisplayServerDebug* debug,
-        std::shared_ptr<mir::client::ClientBufferStream> const& buffer_stream,
+        std::shared_ptr<MirBufferStream> const& buffer_stream,
         std::shared_ptr<mir::input::receiver::InputPlatform> const& input_platform,
-        MirSurfaceSpec const& spec, mir::protobuf::Surface const& surface_proto,
+        MirWindowSpec const& spec, mir::protobuf::Surface const& surface_proto,
         std::shared_ptr<MirWaitHandle> const& handle);
 
     ~MirSurface();
 
-    MirSurfaceParameters get_parameters() const;
+    MirWindowParameters get_parameters() const;
     char const * get_error_message();
     int id() const;
 
-    MirWaitHandle* configure(MirSurfaceAttrib a, int value);
+    MirWaitHandle* configure(MirWindowAttrib a, int value);
 
     // TODO: Some sort of extension mechanism so that this can be moved
     //       out into a separate class in the libmirclient-debug DSO.
@@ -179,7 +196,7 @@ public:
                                          int* screen_x, int* screen_y);
     
     // Non-blocking
-    int attrib(MirSurfaceAttrib a) const;
+    int attrib(MirWindowAttrib a) const;
 
     MirOrientation get_orientation() const;
     MirWaitHandle* set_preferred_orientation(MirOrientationMode mode);
@@ -188,26 +205,30 @@ public:
 
     MirWaitHandle* configure_cursor(MirCursorConfiguration const* cursor);
 
-    void set_event_handler(mir_surface_event_callback callback,
+    void set_event_handler(MirWindowEventCallback callback,
                            void* context);
     void handle_event(MirEvent const& e);
 
-    void request_and_wait_for_configure(MirSurfaceAttrib a, int value);
+    void request_and_wait_for_configure(MirWindowAttrib a, int value);
 
-    mir::client::ClientBufferStream* get_buffer_stream();
+    MirBufferStream* get_buffer_stream();
 
-    MirWaitHandle* modify(MirSurfaceSpec const& changes);
+    MirWaitHandle* modify(MirWindowSpec const& changes);
 
     static bool is_valid(MirSurface* query);
 
-    MirWaitHandle* request_persistent_id(mir_surface_id_callback callback, void* context);
+    MirWaitHandle* request_persistent_id(MirWindowIdCallback callback, void* context);
     MirConnection* connection() const;
+
+    std::shared_ptr<mir::client::FrameClock> get_frame_clock() const;
+
 private:
     std::mutex mutable mutex; // Protects all members of *this
 
+    void configure_frame_clock();
     void on_configured();
     void on_cursor_configured();
-    void acquired_persistent_id(mir_surface_id_callback callback, void* context);
+    void acquired_persistent_id(MirWindowIdCallback callback, void* context);
     MirPixelFormat convert_ipc_pf_to_geometry(google::protobuf::int32 pf) const;
 
     mir::client::rpc::DisplayServer* const server{nullptr};
@@ -229,16 +250,20 @@ private:
     MirWaitHandle persistent_id_wait_handle;
 
     //Deprecated functions can cause MirSurfaces to be created with a default stream
-    std::shared_ptr<mir::client::ClientBufferStream> default_stream;
+    std::shared_ptr<MirBufferStream> default_stream;
+    typedef std::unordered_set<std::shared_ptr<MirBufferStream>> StreamSet;
+    StreamSet streams;
     std::shared_ptr<mir::input::receiver::InputPlatform> const input_platform;
     std::shared_ptr<mir::input::receiver::XKBMapper> const keymapper;
 
     std::unique_ptr<mir::protobuf::SurfaceSetting> const configure_result;
 
     // Cache of latest SurfaceSettings returned from the server
-    int attrib_cache[mir_surface_attribs];
+    int attrib_cache[mir_window_attribs];
     MirOrientation orientation = mir_orientation_normal;
 
+    std::shared_ptr<mir::client::FrameClock> const frame_clock;
+
     std::function<void(MirEvent const*)> handle_event_callback;
     std::shared_ptr<mir::dispatch::ThreadedDispatcher> input_thread;
 
@@ -252,4 +277,6 @@ private:
 
 };
 
+#pragma GCC diagnostic pop
+
 #endif /* MIR_CLIENT_PRIVATE_MIR_WAIT_HANDLE_H_ */
