@@ -19,7 +19,8 @@ public:
     ProbingClientPlatformFactory(
         std::shared_ptr<mir::SharedLibraryProberReport> const& rep,
         StringList const& force_libs,
-        StringList const& lib_paths);
+        StringList const& lib_paths,
+        std::shared_ptr<mir::logging::Logger> const& logger);
 
     std::shared_ptr<ClientPlatform> create_client_platform(ClientContext *context) override;
 
@@ -27,6 +28,7 @@ private:
     std::shared_ptr<mir::SharedLibraryProberReport> const shared_library_prober_report;
     StringList const platform_overrides;
     StringList const platform_paths;
+    std::shared_ptr<mir::logging::Logger> const logger;
 };
 
 }
