@@ -1,5 +1,5 @@
 /*
- * Copyright © 2012 Canonical Ltd.
+ * Copyright © 2012-2016 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License version 3,
@@ -29,11 +29,13 @@
 #include "../event_sink.h"
 #include "../make_protobuf_object.h"
 #include "../protobuf_to_native_buffer.h"
+#include "../mir_error.h"
 #include "mir/input/input_devices.h"
 #include "mir/variable_length_array.h"
 #include "mir/events/event_builders.h"
 #include "mir/events/event_private.h"
 #include "mir/events/serialization.h"
+#include "mir/events/surface_placement_event.h"
 
 #include "mir_protobuf.pb.h"  // For Buffer frig
 #include "mir_protobuf_wire.pb.h"
@@ -52,11 +54,6 @@ namespace mclr = mir::client::rpc;
 namespace md = mir::dispatch;
 namespace mp = mir::protobuf;
 
-namespace
-{
-std::chrono::milliseconds const timeout(200);
-}
-
 mclr::MirProtobufRpcChannel::MirProtobufRpcChannel(
     std::unique_ptr<mclr::StreamTransport> transport,
     std::shared_ptr<mcl::SurfaceMap> const& surface_map,
@@ -66,6 +63,7 @@ mclr::MirProtobufRpcChannel::MirProtobufRpcChannel(
     std::shared_ptr<RpcReport> const& rpc_report,
     std::shared_ptr<LifecycleControl> const& lifecycle_control,
     std::shared_ptr<PingHandler> const& ping_handler,
+    std::shared_ptr<ErrorHandler> const& error_handler,
     std::shared_ptr<EventSink> const& event_sink) :
     rpc_report(rpc_report),
     pending_calls(rpc_report),
@@ -75,6 +73,7 @@ mclr::MirProtobufRpcChannel::MirProtobufRpcChannel(
     input_devices(input_devices),
     lifecycle_control(lifecycle_control),
     ping_handler{ping_handler},
+    error_handler{error_handler},
     event_sink(event_sink),
     disconnected(false),
     transport{std::move(transport)},
@@ -106,7 +105,7 @@ void mclr::MirProtobufRpcChannel::notify_disconnected()
     if (auto map = surface_map.lock()) 
     {
         map->with_all_streams_do(
-            [](mcl::ClientBufferStream* receiver) {
+            [](MirBufferStream* receiver) {
                 if (receiver) receiver->buffer_unavailable();
             });
     }
@@ -198,6 +197,20 @@ void mclr::MirProtobufRpcChannel::call_method(
     google::protobuf::MessageLite* response,
     google::protobuf::Closure* complete)
 {
+    if (discard)
+    {
+       /*
+        * Until recently we had no explicit plan for what to do in this case.
+        * Callbacks would race with destruction of the MirConnection and either
+        * succeed, deadlock, crash or corrupt memory. However the one apparent
+        * intent in the old plan was that we close all closures so that the
+        * user doesn't leak any memory. So do that...
+        */
+       if (complete)
+           complete->Run();
+       return;
+    }
+
     // Only send message when details saved for handling response
     std::vector<mir::Fd> fds;
     if (parameters->GetTypeName() == "mir.protobuf.BufferRequest")
@@ -229,6 +242,16 @@ void mclr::MirProtobufRpcChannel::call_method(
     send_message(invocation, invocation, fds);
 }
 
+void mclr::MirProtobufRpcChannel::discard_future_calls()
+{
+    discard = true;
+}
+
+void mclr::MirProtobufRpcChannel::wait_for_outstanding_calls()
+{
+    pending_calls.wait_till_complete();
+}
+
 void mclr::MirProtobufRpcChannel::send_message(
     mir::protobuf::wire::Invocation const& body,
     mir::protobuf::wire::Invocation const& invocation,
@@ -270,16 +293,9 @@ void mclr::MirProtobufRpcChannel::process_event_sequence(std::string const& even
         display_configuration->update_configuration(seq.display_configuration());
     }
 
-    if (seq.input_devices_size())
+    if (seq.has_input_configuration())
     {
-        std::vector<mir::input::DeviceData> devices;
-
-        devices.reserve(seq.input_devices_size());
-
-        for (auto const& dev : seq.input_devices())
-            devices.emplace_back(dev.id(), dev.capabilities(), dev.name(), dev.unique_id());
-
-        input_devices->update_devices(std::move(devices));
+        input_devices->update_devices(seq.input_configuration());
     }
 
     if (seq.has_lifecycle_event())
@@ -292,6 +308,15 @@ void mclr::MirProtobufRpcChannel::process_event_sequence(std::string const& even
         (*ping_handler)(seq.ping_event().serial());
     }
 
+    if (seq.has_structured_error())
+    {
+        auto const error = MirError{
+            static_cast<MirErrorDomain>(seq.structured_error().domain()),
+            seq.structured_error().code()
+        };
+        (*error_handler)(&error);
+    }
+
     if (seq.has_buffer_request())
     {
         std::array<char, 1> dummy;
@@ -311,10 +336,9 @@ void mclr::MirProtobufRpcChannel::process_event_sequence(std::string const& even
             {
                 if (seq.buffer_request().has_id())
                 {
-                    map->with_stream_do(mf::BufferStreamId(seq.buffer_request().id().value()),
-                    [&] (mcl::ClientBufferStream* receiver) {
+                    mf::BufferStreamId stream_id(seq.buffer_request().id().value());
+                    if (auto receiver = map->stream(stream_id))
                         receiver->buffer_available(seq.buffer_request().buffer());
-                    });
                 }
                 
                 else if (seq.buffer_request().has_operation())
@@ -371,41 +395,41 @@ void mclr::MirProtobufRpcChannel::process_event_sequence(std::string const& even
                 {
                     rpc_report->event_parsing_succeeded(*e);
 
-                    auto const send_e = [&e](MirSurface* surface)
-                        { surface->handle_event(*e); };
+                    int surface_id = 0;
+                    bool is_surface_event = true;
 
                     switch (e->type())
                     {
-                    case mir_event_type_surface:
-                        if (auto map = surface_map.lock())
-                            map->with_surface_do(mf::SurfaceId(e->to_surface()->id()), send_e);
+                    case mir_event_type_window:
+                        surface_id = e->to_surface()->id();
                         break;
-
                     case mir_event_type_resize:
-                        if (auto map = surface_map.lock())
-                            map->with_surface_do(mf::SurfaceId(e->to_resize()->surface_id()), send_e);
+                        surface_id = e->to_resize()->surface_id();
                         break;
-
                     case mir_event_type_orientation:
-                        if (auto map = surface_map.lock())
-                            map->with_surface_do(mf::SurfaceId(e->to_orientation()->surface_id()), send_e);
+                        surface_id = e->to_orientation()->surface_id();
                         break;
-
-                    case mir_event_type_close_surface:
-                        if (auto map = surface_map.lock())
-                            map->with_surface_do(mf::SurfaceId(e->to_close_surface()->surface_id()), send_e);
+                    case mir_event_type_close_window:
+                        surface_id = e->to_close_window()->surface_id();
                         break;
                     case mir_event_type_keymap:
-                        if (auto map = surface_map.lock())
-                            map->with_surface_do(mf::SurfaceId(e->to_keymap()->surface_id()), send_e);
+                        surface_id = e->to_keymap()->surface_id();
                         break;
-                    case mir_event_type_surface_output:
-                        if (auto map = surface_map.lock())
-                            map->with_surface_do(mf::SurfaceId(e->to_surface_output()->surface_id()), send_e);
+                    case mir_event_type_window_output:
+                        surface_id = e->to_window_output()->surface_id();
+                        break;
+                    case mir_event_type_window_placement:
+                        surface_id = e->to_window_placement()->id();
                         break;
                     default:
+                        is_surface_event = false;
                         event_sink->handle_event(*e);
                     }
+
+                    if (is_surface_event)
+                        if (auto map = surface_map.lock())
+                            if (auto surf = map->surface(mf::SurfaceId(surface_id)))
+                                surf->handle_event(*e);
                 }
             }
             catch(...)
