@@ -31,7 +31,8 @@
 #include "mir/events/input_event.h"
 #include "mir/events/keyboard_event.h"
 #include "mir/events/keymap_event.h"
-#include "mir/events/motion_event.h"
+#include "mir/events/touch_event.h"
+#include "mir/events/pointer_event.h"
 #include "mir/events/orientation_event.h"
 #include "mir/events/prompt_session_event.h"
 #include "mir/events/resize_event.h"
