@@ -24,8 +24,11 @@
 #include "mir/input/key_mapper.h"
 #include "mir/optional_value.h"
 
+#include <xkbcommon/xkbcommon.h>
+#include <xkbcommon/xkbcommon-compose.h>
 #include <mutex>
 #include <unordered_map>
+#include <unordered_set>
 
 namespace mir
 {
@@ -40,7 +43,8 @@ XKBKeymapPtr make_unique_keymap(xkb_context* context, Keymap const& keymap);
 XKBKeymapPtr make_unique_keymap(xkb_context* context, char const* buffer, size_t size);
 
 using XKBStatePtr = std::unique_ptr<xkb_state, void(*)(xkb_state*)>;
-XKBStatePtr make_unique_state(xkb_keymap* keymap);
+using XKBComposeTablePtr = std::unique_ptr<xkb_compose_table, void(*)(xkb_compose_table*)>;
+using XKBComposeStatePtr = std::unique_ptr<xkb_compose_state, void(*)(xkb_compose_state*)>;
 
 namespace receiver
 {
@@ -71,25 +75,37 @@ private:
 
     std::mutex mutable guard;
 
+    struct ComposeState
+    {
+        ComposeState(XKBComposeTablePtr const& table);
+        void update_and_map(MirEvent& event);
+        xkb_keysym_t update_state(xkb_keysym_t mapped_key, MirKeyboardAction action);
+        XKBComposeStatePtr state;
+        std::unordered_set<xkb_keysym_t> consumed_keys;
+        mir::optional_value<std::tuple<xkb_keysym_t,xkb_keysym_t>> last_composed_key;
+    };
+
     struct XkbMappingState
     {
         explicit XkbMappingState(std::shared_ptr<xkb_keymap> const& keymap);
         void set_key_state(std::vector<uint32_t> const& key_state);
-        bool update_and_map(MirEvent& event);
-        xkb_keysym_t update_state(uint32_t scan_code, MirKeyboardAction direction);
+        bool update_and_map(MirEvent& event, ComposeState* compose_state);
+        xkb_keysym_t update_state(uint32_t scan_code, MirKeyboardAction direction, ComposeState* compose_state);
         std::shared_ptr<xkb_keymap> const keymap;
         XKBStatePtr state;
         MirInputEventModifiers modifier_state{0};
-        // TODO optional compose key state..
     };
 
     XkbMappingState* get_keymapping_state(MirInputDeviceId id);
+    ComposeState* get_compose_state(MirInputDeviceId id);
 
     XKBContextPtr context;
     std::shared_ptr<xkb_keymap> default_keymap;
+    XKBComposeTablePtr compose_table;
 
     mir::optional_value<MirInputEventModifiers> modifier_state;
-    std::unordered_map<MirInputDeviceId, XkbMappingState> device_mapping;
+    std::unordered_map<MirInputDeviceId, std::unique_ptr<XkbMappingState>> device_mapping;
+    std::unordered_map<MirInputDeviceId, std::unique_ptr<ComposeState>> device_composing;
 };
 }
 }
