@@ -19,6 +19,9 @@
 #ifndef MIR_GRAPHICS_NESTED_CONTEXT_H_
 #define MIR_GRAPHICS_NESTED_CONTEXT_H_
 
+#include "mir/optional_value.h"
+#include "mir/fd.h"
+#include <memory>
 #include <vector>
 
 struct gbm_device;
@@ -29,12 +32,38 @@ namespace graphics
 {
 struct PlatformOperationMessage;
 
+class MesaAuthExtension
+{
+public:
+    virtual ~MesaAuthExtension() = default;
+    virtual mir::Fd auth_fd() = 0;
+    virtual int auth_magic(unsigned int) = 0;
+protected:
+    MesaAuthExtension() = default;
+    MesaAuthExtension(MesaAuthExtension const&) = delete;
+    MesaAuthExtension& operator=(MesaAuthExtension const&) = delete;
+};
+
+class SetGbmExtension
+{
+public:
+    virtual ~SetGbmExtension() = default;
+    virtual void set_gbm_device(gbm_device*) = 0;
+protected:
+    SetGbmExtension() = default;
+    SetGbmExtension(SetGbmExtension const&) = delete;
+    SetGbmExtension& operator=(SetGbmExtension const&) = delete;
+};
+
 class NestedContext
 {
 public:
     virtual ~NestedContext() = default;
 
-    virtual std::vector<int> platform_fd_items() = 0;
+    //unique_ptr would be nice, but don't want to break mircore
+    virtual mir::optional_value<std::shared_ptr<MesaAuthExtension>> auth_extension() = 0;
+    virtual mir::optional_value<std::shared_ptr<SetGbmExtension>> set_gbm_extension() = 0;
+
     virtual PlatformOperationMessage platform_operation(
         unsigned int op, PlatformOperationMessage const& request) = 0;
 
