@@ -33,11 +33,8 @@ class DisplayChanger
 public:
     virtual ~DisplayChanger() = default;
 
-    enum SystemStateHandling : bool { RetainSystemState, PauseResumeSystem };
-
     virtual void configure_for_hardware_change(
-        std::shared_ptr<graphics::DisplayConfiguration> const& conf,
-        SystemStateHandling pause_resume_system) = 0;
+        std::shared_ptr<graphics::DisplayConfiguration> const& conf) = 0;
 
     virtual void pause_display_config_processing() = 0;
     virtual void resume_display_config_processing() = 0;
