@@ -40,15 +40,16 @@ char const* const mo::legacy_input_report_opt     = "legacy-input-report";
 char const* const mo::connector_report_opt        = "connector-report";
 char const* const mo::scene_report_opt            = "scene-report";
 char const* const mo::input_report_opt            = "input-report";
+char const* const mo::seat_report_opt            = "seat-report";
 char const* const mo::shared_library_prober_report_opt = "shared-library-prober-report";
 char const* const mo::shell_report_opt            = "shell-report";
 char const* const mo::host_socket_opt             = "host-socket";
+char const* const mo::nested_passthrough_opt      = "nested-passthrough";
 char const* const mo::frontend_threads_opt        = "ipc-thread-pool";
 char const* const mo::name_opt                    = "name";
 char const* const mo::touchspots_opt              = "enable-touchspots";
 char const* const mo::fatal_except_opt            = "on-fatal-error-except";
 char const* const mo::debug_opt                   = "debug";
-char const* const mo::nbuffers_opt                = "nbuffers";
 char const* const mo::composite_delay_opt         = "composite-delay";
 char const* const mo::enable_key_repeat_opt       = "enable-key-repeat";
 
@@ -161,6 +162,8 @@ mo::DefaultConfiguration::DefaultConfiguration(
             "How to handle to Input report. [{log,lttng,off}]")
         (legacy_input_report_opt, po::value<std::string>()->default_value(off_opt_value),
             "How to handle the Legacy Input report. [{log,off}]")
+        (seat_report_opt, po::value<std::string>()->default_value(off_opt_value),
+            "How to handle to Seat report. [{log,off}]")
         (session_mediator_report_opt, po::value<std::string>()->default_value(off_opt_value),
             "How to handle the SessionMediator report. [{log,lttng,off}]")
         (msg_processor_report_opt, po::value<std::string>()->default_value(off_opt_value),
@@ -171,15 +174,16 @@ mo::DefaultConfiguration::DefaultConfiguration(
             "How to handle the SharedLibraryProber report. [{log,lttng,off}]")
         (shell_report_opt, po::value<std::string>()->default_value(off_opt_value),
          "How to handle the Shell report. [{log,off}]")
-        (nbuffers_opt, po::value<int>()->default_value(0),
-            "Number of buffers per surface.")
-        (composite_delay_opt, po::value<int>()->default_value(-1),
+        (composite_delay_opt, po::value<int>()->default_value(0),
             "Compositor frame delay in milliseconds (how long to wait for new "
             "frames from clients before compositing). Higher values result in "
             "lower latency but risk causing frame skipping. "
             "Default: A negative value means decide automatically.")
         (name_opt, po::value<std::string>(),
             "When nested, the name Mir uses when registering with the host.")
+        (nested_passthrough_opt, po::value<bool>()->default_value(true),
+            "When nested, attempt to pass a client's graphics content directly to the host"
+            " to avoid a composition pass")
         (touchspots_opt,
             "Display visualization of touchspots (e.g. for screencasting).")
         (enable_key_repeat_opt, po::value<bool>()->default_value(true),
