@@ -20,6 +20,9 @@
 #define MIR_GRAPHICS_ANDROID_DISPLAY_H_
 
 #include "mir/graphics/display.h"
+#include "mir/graphics/frame.h"
+#include "mir/graphics/atomic_frame.h"
+#include "mir/renderer/gl/context_source.h"
 #include "gl_context.h"
 #include "display_group.h"
 #include "hwc_configuration.h"
@@ -29,6 +32,7 @@
 #include <memory>
 #include <mutex>
 #include <array>
+#include <unordered_map>
 
 namespace mir
 {
@@ -48,7 +52,9 @@ class DisplayChangePipe;
 class DisplayDevice;
 class NativeWindowReport;
 
-class Display : public graphics::Display
+class Display : public graphics::Display,
+                public graphics::NativeDisplay,
+                public renderer::gl::ContextSource
 {
 public:
     explicit Display(
@@ -63,6 +69,7 @@ public:
     void for_each_display_sync_group(std::function<void(graphics::DisplaySyncGroup&)> const& f) override;
 
     std::unique_ptr<graphics::DisplayConfiguration> configuration() const override;
+    bool apply_if_configuration_preserves_display_buffers(graphics::DisplayConfiguration const& conf) override;
     void configure(graphics::DisplayConfiguration const&) override;
 
     void register_configuration_change_handler(
@@ -78,12 +85,17 @@ public:
     void resume() override;
 
     std::shared_ptr<Cursor> create_hardware_cursor(std::shared_ptr<CursorImage> const& initial_image) override;
-    std::unique_ptr<graphics::GLContext> create_gl_context() override;
     std::unique_ptr<VirtualOutput> create_virtual_output(int width, int height) override;
 
+    NativeDisplay* native_display() override;
+
+    std::unique_ptr<renderer::gl::Context> create_gl_context() override;
+
+    Frame last_frame_on(unsigned output_id) const override;
+
 private:
     void on_hotplug();
-    void on_vsync(DisplayName) const;
+    void on_vsync(DisplayName, graphics::Frame::Timestamp);
 
     geometry::Point const origin{0,0};
     std::shared_ptr<DisplayReport> const display_report;
@@ -102,6 +114,12 @@ private:
     OverlayOptimization const overlay_option;
 
     void update_configuration(std::lock_guard<decltype(configuration_mutex)> const&) const;
+    void configure_locked(
+        graphics::DisplayConfiguration const& new_configuration,
+        std::lock_guard<decltype(configuration_mutex)> const&);
+
+    std::mutex mutable vsync_mutex;
+    std::unordered_map<unsigned,AtomicFrame> last_frame;
 };
 
 }
