@@ -32,15 +32,16 @@ namespace mesa
 class NativeSurface : public MirMesaEGLNativeSurface
 {
 public:
-    explicit NativeSurface(EGLNativeSurface&);
+    explicit NativeSurface(EGLNativeSurface*);
 
     int advance_buffer(MirBufferPackage* buffer_package);
-    int get_parameters(MirSurfaceParameters* surface_parameters);
+    int get_parameters(MirWindowParameters* surface_parameters);
     int set_swapinterval(int interval);
+    void use_native_surface(EGLNativeSurface* native_surface);
 
 private:
     bool starting;
-    EGLNativeSurface& surface;
+    EGLNativeSurface* surface;
 };
 
 }
