@@ -36,6 +36,7 @@ public:
     DisplayConfiguration(MirPixelFormat pf,
                          mir::geometry::Size const pixels,
                          mir::geometry::Size const size_mm,
+                         float const scale,
                          MirOrientation orientation);
     DisplayConfiguration(DisplayConfiguration const&);
 
@@ -46,6 +47,8 @@ public:
     void for_each_output(std::function<void(UserDisplayConfigurationOutput&)> f) override;
     std::unique_ptr<graphics::DisplayConfiguration> clone() const override;
 
+    static DisplayConfigurationOutputId const the_output_id;
+
 private:
     DisplayConfigurationOutput configuration;
     DisplayConfigurationCard card;
