@@ -165,6 +165,8 @@ void mix::XInputDevice::pointer_press(std::chrono::nanoseconds event_time, int b
             event_time,
             mir_pointer_action_button_down,
             button_state,
+            pointer_pos.x.as_int(),
+            pointer_pos.y.as_int(),
             scroll.dx.as_int(),
             scroll.dy.as_int(),
             movement.dx.as_int(),
@@ -184,6 +186,8 @@ void mix::XInputDevice::pointer_release(std::chrono::nanoseconds event_time, int
             event_time,
             mir_pointer_action_button_up,
             button_state,
+            pointer_pos.x.as_int(),
+            pointer_pos.y.as_int(),
             scroll.dx.as_int(),
             scroll.dy.as_int(),
             movement.dx.as_int(),
@@ -202,6 +206,8 @@ void mix::XInputDevice::pointer_motion(std::chrono::nanoseconds event_time, mir:
             event_time,
             mir_pointer_action_motion,
             button_state,
+            pointer_pos.x.as_int(),
+            pointer_pos.y.as_int(),
             scroll.dx.as_int(),
             scroll.dy.as_int(),
             movement.dx.as_int(),
