@@ -16,7 +16,7 @@
  * Authored by: Kevin DuBois <kevin.dubois@canonical.com>
  */
 
-#include "mir/graphics/graphic_buffer_allocator.h"
+#include "mir/graphics/buffer.h"
 #include "mir/frontend/buffer_sink.h"
 #include "buffer_map.h"
 #include <boost/throw_exception.hpp>
@@ -38,26 +38,24 @@ enum class BufferMap::Owner
 }
 }
 
-mc::BufferMap::BufferMap(
-    std::shared_ptr<mf::BufferSink> const& sink,
-    std::shared_ptr<mg::GraphicBufferAllocator> const& allocator) :
-    sink(sink),
-    allocator(allocator)
+mc::BufferMap::BufferMap(std::shared_ptr<mf::BufferSink> const& sink) :
+    sink(sink)
 {
 }
 
-mg::BufferID mc::BufferMap::add_buffer(mg::BufferProperties const& properties)
+mg::BufferID mc::BufferMap::add_buffer(std::shared_ptr<mg::Buffer> const& buffer)
 {
     try
     {
         std::unique_lock<decltype(mutex)> lk(mutex);
-        auto buffer = allocator->alloc_buffer(properties);
         buffers[buffer->id()] = {buffer, Owner::client};
-        sink->add_buffer(*buffer);
+        if (auto s = sink.lock())
+            s->add_buffer(*buffer);
         return buffer->id();
     } catch (std::exception& e)
     {
-        sink->error_buffer(properties, e.what());
+        if (auto s = sink.lock())
+            s->error_buffer(buffer->size(), buffer->pixel_format(), e.what());
         throw;
     }
 }
@@ -66,7 +64,8 @@ void mc::BufferMap::remove_buffer(mg::BufferID id)
 {
     std::unique_lock<decltype(mutex)> lk(mutex);
     auto it = checked_buffers_find(id, lk);
-    sink->remove_buffer(*it->second.buffer);
+    if (auto s = sink.lock())
+        s->remove_buffer(*it->second.buffer);
     buffers.erase(it); 
 }
 
@@ -79,7 +78,8 @@ void mc::BufferMap::send_buffer(mg::BufferID id)
         auto buffer = it->second.buffer;
         it->second.owner = Owner::client;
         lk.unlock();
-        sink->update_buffer(*buffer);
+        if (auto s = sink.lock())
+            s->update_buffer(*buffer);
     }
 }
 
