@@ -19,15 +19,14 @@
  */
 
 #include "buffer_stream_factory.h"
-#include "buffer_stream_surfaces.h"
 #include "mir/graphics/buffer_properties.h"
-#include "buffer_queue.h"
 #include "stream.h"
 #include "buffer_map.h"
 #include "mir/graphics/buffer.h"
 #include "mir/graphics/buffer_id.h"
 #include "mir/graphics/graphic_buffer_allocator.h"
 #include "mir/graphics/display.h"
+#include "mir/compositor/frame_dropping_policy.h"
 
 #include <cassert>
 #include <memory>
@@ -38,48 +37,31 @@ namespace ms = mir::scene;
 namespace mf = mir::frontend;
 
 mc::BufferStreamFactory::BufferStreamFactory(
-    std::shared_ptr<mg::GraphicBufferAllocator> const& gralloc,
-    std::shared_ptr<mc::FrameDroppingPolicyFactory> const& policy_factory,
-    unsigned int nbuffers) :
-    gralloc(gralloc),
-    policy_factory{policy_factory},
-    nbuffers(nbuffers)
+    std::shared_ptr<mc::FrameDroppingPolicyFactory> const& policy_factory) :
+    policy_factory{policy_factory}
 {
-    assert(gralloc);
     assert(policy_factory);
-    if (nbuffers == 1)
-        throw std::logic_error("nbuffers cannot be 1");
 }
 
-
 std::shared_ptr<mc::BufferStream> mc::BufferStreamFactory::create_buffer_stream(
     mf::BufferStreamId id, std::shared_ptr<mf::ClientBuffers> const& buffers,
     mg::BufferProperties const& buffer_properties)
 {
-    return create_buffer_stream(id, buffers, nbuffers, buffer_properties);
+    return create_buffer_stream(id, buffers, 0, buffer_properties);
 }
 
 std::shared_ptr<mc::BufferStream> mc::BufferStreamFactory::create_buffer_stream(
     mf::BufferStreamId, std::shared_ptr<mf::ClientBuffers> const& buffers,
-    int nbuffers, mg::BufferProperties const& buffer_properties)
+    int, mg::BufferProperties const& buffer_properties)
 {
-    if (nbuffers == 0)
-    {
-        return std::make_shared<mc::Stream>(
-            *policy_factory,
-            buffers,
-            buffer_properties.size, buffer_properties.format);
-    }
-    else
-    {
-        auto switching_bundle = std::make_shared<mc::BufferQueue>(
-            nbuffers, gralloc, buffer_properties, *policy_factory);
-        return std::make_shared<mc::BufferStreamSurfaces>(switching_bundle);
-    }
+    return std::make_shared<mc::Stream>(
+        *policy_factory,
+        buffers,
+        buffer_properties.size, buffer_properties.format);
 }
 
 std::shared_ptr<mf::ClientBuffers> mc::BufferStreamFactory::create_buffer_map(
         std::shared_ptr<mf::BufferSink> const& sink)
 {
-    return std::make_shared<mc::BufferMap>(sink, gralloc);
+    return std::make_shared<mc::BufferMap>(sink);
 }
