@@ -25,6 +25,7 @@
 #include "mir/graphics/buffer.h"
 #include "mir/compositor/frame_dropping_policy_factory.h"
 #include "mir/compositor/frame_dropping_policy.h"
+#include <boost/throw_exception.hpp>
 
 namespace mc = mir::compositor;
 namespace geom = mir::geometry;
@@ -73,6 +74,12 @@ mc::Stream::Stream(
 {
 }
 
+mc::Stream::~Stream()
+{
+    while(schedule->num_scheduled())
+        buffers->send_buffer(schedule->next_buffer()->id());
+}
+
 unsigned int mc::Stream::client_owned_buffer_count(std::lock_guard<decltype(mutex)> const&) const
 {
     auto server_count = schedule->num_scheduled();
@@ -81,21 +88,20 @@ unsigned int mc::Stream::client_owned_buffer_count(std::lock_guard<decltype(mute
     return associated_buffers.size() - server_count;
 }
 
-void mc::Stream::swap_buffers(mg::Buffer* buffer, std::function<void(mg::Buffer* new_buffer)> fn)
+void mc::Stream::submit_buffer(std::shared_ptr<mg::Buffer> const& buffer)
 {
-    if (buffer)
+    if (!buffer)
+        BOOST_THROW_EXCEPTION(std::invalid_argument("cannot submit null buffer"));
+
     {
-        {
-            std::lock_guard<decltype(mutex)> lk(mutex); 
-            first_frame_posted = true;
-            buffers->receive_buffer(buffer->id());
-            schedule->schedule((*buffers)[buffer->id()]);
-            if (!associated_buffers.empty() && (client_owned_buffer_count(lk) == 0))
-                drop_policy->swap_now_blocking();
-        }
-        observers.frame_posted(1, buffer->size());
+        std::lock_guard<decltype(mutex)> lk(mutex); 
+        first_frame_posted = true;
+        buffers->receive_buffer(buffer->id());
+        schedule->schedule((*buffers)[buffer->id()]);
+        if (!associated_buffers.empty() && (client_owned_buffer_count(lk) == 0))
+            drop_policy->swap_now_blocking();
     }
-    fn(nullptr); //legacy support
+    observers.frame_posted(1, buffer->size());
 }
 
 void mc::Stream::with_most_recent_buffer_do(std::function<void(mg::Buffer&)> const& fn)
@@ -158,6 +164,11 @@ void mc::Stream::allow_framedropping(bool dropping)
     }
 }
 
+bool mc::Stream::framedropping() const
+{
+    return schedule_mode == ScheduleMode::Dropping;
+}
+
 void mc::Stream::transition_schedule(
     std::shared_ptr<mc::Schedule>&& new_schedule, std::lock_guard<std::mutex> const&)
 {
