@@ -49,6 +49,8 @@ public:
     void configure(
         std::shared_ptr<frontend::Session> const&,
         std::shared_ptr<graphics::DisplayConfiguration> const&) override;
+    void remove_session_configuration(
+        std::shared_ptr<frontend::Session> const&) override;
     void set_base_configuration(
         std::shared_ptr<graphics::DisplayConfiguration> const&) override;
     void preview_base_configuration(
@@ -58,7 +60,8 @@ public:
     void confirm_base_configuration(
         std::shared_ptr<Session> const&,
         std::shared_ptr<graphics::DisplayConfiguration> const&) override;
-
+    void cancel_base_configuration_preview(
+        std::shared_ptr<Session> const& session) override;
 
 private:
     std::shared_ptr<frontend::DisplayChanger> const changer;
