@@ -30,15 +30,16 @@ namespace graphics
 {
 namespace nested
 {
-
+class HostSurfaceSpec;
 class HostSurface
 {
 public:
     virtual ~HostSurface() = default;
 
     virtual EGLNativeWindowType egl_native_window() = 0;
-    virtual void set_event_handler(mir_surface_event_callback cb,
+    virtual void set_event_handler(MirWindowEventCallback cb,
                                    void* context) = 0;
+    virtual void apply_spec(HostSurfaceSpec&) = 0;
 protected:
     HostSurface() = default;
     HostSurface(HostSurface const&) = delete;
