@@ -21,7 +21,7 @@
 
 #include "server_tracepoint_provider.h"
 
-#include "mir/frontend/session_mediator_report.h"
+#include "mir/frontend/session_mediator_observer.h"
 
 namespace mir
 {
@@ -30,12 +30,11 @@ namespace report
 namespace lttng
 {
 // Interface for monitoring application activity
-class SessionMediatorReport : public frontend::SessionMediatorReport
+class SessionMediatorReport : public frontend::SessionMediatorObserver
 {
 public:
     void session_connect_called(std::string const& app_name) override;
     void session_create_surface_called(std::string const& app_name) override;
-    void session_exchange_buffer_called(std::string const& app_name) override;
     void session_submit_buffer_called(std::string const& app_name) override;
     void session_allocate_buffers_called(std::string const& app_name) override;
     void session_release_buffers_called(std::string const& app_name) override;
