@@ -21,7 +21,7 @@
 #ifndef MIR_REPORT_NULL_SESSION_MEDIATOR_REPORT_H_
 #define MIR_REPORT_NULL_SESSION_MEDIATOR_REPORT_H_
 
-#include "mir/frontend/session_mediator_report.h"
+#include "mir/frontend/session_mediator_observer.h"
 
 #include <string>
 
@@ -33,14 +33,12 @@ namespace null
 {
 
 // Do-nothing implementation to satisfy dependencies
-class SessionMediatorReport : public frontend::SessionMediatorReport
+class SessionMediatorReport : public frontend::SessionMediatorObserver
 {
     void session_connect_called(std::string const& app_name) override;
 
     void session_create_surface_called(std::string const& app_name) override;
 
-    void session_exchange_buffer_called(std::string const& app_name) override;
-
     void session_submit_buffer_called(std::string const& app_name) override;
 
     void session_allocate_buffers_called(std::string const& app_name) override;
