@@ -39,6 +39,7 @@ namespace graphics
 {
 class DisplayConfiguration;
 class GraphicBufferAllocator;
+class BufferAttribute;
 }
 namespace shell { class SurfaceStack; }
 namespace scene
@@ -49,7 +50,7 @@ class SnapshotStrategy;
 class BufferStreamFactory;
 class SurfaceFactory;
 
-class ApplicationSession : public Session
+class ApplicationSession : public Session, public frontend::SessionExtensions
 {
 public:
     ApplicationSession(
@@ -86,7 +87,8 @@ public:
     void show() override;
 
     void send_display_config(graphics::DisplayConfiguration const& info) override;
-    void send_input_device_change(std::vector<std::shared_ptr<input::Device>> const& devices) override;
+    void send_error(ClientVisibleError const& error) override;
+    void send_input_config(MirInputConfig const& devices) override;
 
     void set_lifecycle_state(MirLifecycleState state) override;
 
@@ -102,6 +104,8 @@ public:
     void destroy_surface(std::weak_ptr<Surface> const& surface) override;
 
     graphics::BufferID create_buffer(graphics::BufferProperties const& properties) override;
+    graphics::BufferID create_buffer(geometry::Size, MirPixelFormat) override;
+    graphics::BufferID create_buffer(geometry::Size, uint32_t native_format, uint32_t native_flags) override;
     void destroy_buffer(graphics::BufferID) override;
     std::shared_ptr<graphics::Buffer> get_buffer(graphics::BufferID) override;
 protected:
@@ -118,6 +122,7 @@ private:
     std::shared_ptr<SessionListener> const session_listener;
     std::shared_ptr<frontend::EventSink> const event_sink;
     std::shared_ptr<frontend::ClientBuffers> const buffers;
+    std::shared_ptr<graphics::GraphicBufferAllocator> const gralloc;
 
     frontend::SurfaceId next_id();
 
