@@ -65,7 +65,6 @@ public:
         std::string const& name,
         geometry::Rectangle rect,
         MirPointerConfinementState state,
-        bool nonrectangular,
         std::list<scene::StreamInfo> const& streams,
         std::shared_ptr<input::InputChannel> const& input_channel,
         std::shared_ptr<input::InputSender> const& sender,
@@ -77,7 +76,6 @@ public:
         geometry::Rectangle rect,
         std::weak_ptr<Surface> const& parent,
         MirPointerConfinementState state,
-        bool nonrectangular,
         std::list<scene::StreamInfo> const& streams,
         std::shared_ptr<input::InputChannel> const& input_channel,
         std::shared_ptr<input::InputSender> const& sender,
@@ -105,8 +103,6 @@ public:
 
     void set_input_region(std::vector<geometry::Rectangle> const& input_rectangles) override;
 
-    std::shared_ptr<compositor::BufferStream> buffer_stream() const;
-
     void resize(geometry::Size const& size) override;
     geometry::Point top_left() const override;
     geometry::Rectangle input_bounds() const override;
@@ -121,10 +117,10 @@ public:
     graphics::RenderableList generate_renderables(compositor::CompositorID id) const override;
     int buffers_ready_for_compositor(void const* compositor_id) const override;
 
-    MirSurfaceType type() const override;
-    MirSurfaceState state() const override;
-    int configure(MirSurfaceAttrib attrib, int value) override;
-    int query(MirSurfaceAttrib attrib) const override;
+    MirWindowType type() const override;
+    MirWindowState state() const override;
+    int configure(MirWindowAttrib attrib, int value) override;
+    int query(MirWindowAttrib attrib) const override;
     void hide() override;
     void show() override;
     
@@ -152,15 +148,16 @@ public:
 
     void set_confine_pointer_state(MirPointerConfinementState state) override;
     MirPointerConfinementState confine_pointer_state() const override;
+    void placed_relative(geometry::Rectangle const& placement) override;
 
 private:
     bool visible(std::unique_lock<std::mutex>&) const;
-    MirSurfaceType set_type(MirSurfaceType t);  // Use configure() to make public changes
-    MirSurfaceState set_state(MirSurfaceState s);
+    MirWindowType set_type(MirWindowType t);  // Use configure() to make public changes
+    MirWindowState set_state(MirWindowState s);
     int set_dpi(int);
-    MirSurfaceVisibility set_visibility(MirSurfaceVisibility v);
+    MirWindowVisibility set_visibility(MirWindowVisibility v);
     int set_swap_interval(int);
-    MirSurfaceFocusState set_focus_state(MirSurfaceFocusState f);
+    MirWindowFocusState set_focus_state(MirWindowFocusState f);
     MirOrientationMode set_preferred_orientation(MirOrientationMode mode);
 
     SurfaceObservers observers;
@@ -171,7 +168,6 @@ private:
     float surface_alpha;
     bool hidden;
     input::InputReceptionMode input_mode;
-    const bool nonrectangular;
     std::vector<geometry::Rectangle> custom_input_rectangles;
     std::shared_ptr<compositor::BufferStream> const surface_buffer_stream;
     std::shared_ptr<input::InputChannel> const server_input_channel;
@@ -182,12 +178,12 @@ private:
 
     std::list<StreamInfo> layers;
     // Surface attributes:
-    MirSurfaceType type_ = mir_surface_type_normal;
-    MirSurfaceState state_ = mir_surface_state_restored;
+    MirWindowType type_ = mir_window_type_normal;
+    MirWindowState state_ = mir_window_state_restored;
     int swapinterval_ = 1;
-    MirSurfaceFocusState focus_ = mir_surface_unfocused;
+    MirWindowFocusState focus_ = mir_window_focus_state_unfocused;
     int dpi_ = 0;
-    MirSurfaceVisibility visibility_ = mir_surface_visibility_occluded;
+    MirWindowVisibility visibility_ = mir_window_visibility_occluded;
     MirOrientationMode pref_orientation_mode = mir_orientation_mode_any;
     MirPointerConfinementState confine_pointer_state_ = mir_pointer_unconfined;
 
