@@ -28,7 +28,8 @@ namespace scene
 class DefaultCoordinateTranslator : public CoordinateTranslator
 {
 public:
-    geometry::Point surface_to_screen(std::shared_ptr<frontend::Surface> surface, int32_t x, int32_t y);
+    geometry::Point surface_to_screen(std::shared_ptr<frontend::Surface> surface, int32_t x, int32_t y) override;
+    bool translation_supported() const override;
 };
 
 }
