@@ -17,10 +17,11 @@
  */
 
 #include "unsupported_coordinate_translator.h"
-#include "mir/frontend/unsupported_feature_exception.h"
 
 #include <boost/exception/all.hpp>
 
+#include <stdexcept>
+
 namespace mg = mir::geometry;
 namespace mf = mir::frontend;
 namespace ms = mir::scene;
@@ -29,5 +30,10 @@ mg::Point ms::UnsupportedCoordinateTranslator::surface_to_screen(
         std::shared_ptr<mf::Surface> /*surface*/,
         int32_t /*x*/, int32_t /*y*/)
 {
-    BOOST_THROW_EXCEPTION(mf::unsupported_feature());
+    BOOST_THROW_EXCEPTION(std::runtime_error{"Unsupported feature requested"});
+}
+
+bool ms::UnsupportedCoordinateTranslator::translation_supported() const
+{
+    return false;
 }
