@@ -36,9 +36,9 @@
 #include "mir/cookie/authority.h"
 
 // TODO these are used to frig a stub renderer when running headless
-#include "mir/compositor/renderer.h"
+#include "mir/renderer/renderer.h"
 #include "mir/graphics/renderable.h"
-#include "mir/compositor/renderer_factory.h"
+#include "mir/renderer/renderer_factory.h"
 
 #include <iostream>
 
@@ -88,7 +88,6 @@ struct TemporaryCompositeEventFilter : public mi::CompositeEventFilter
     MACRO(compositor)\
     MACRO(cursor_images)\
     MACRO(display_buffer_compositor_factory)\
-    MACRO(display_configuration_report)\
     MACRO(gl_config)\
     MACRO(host_lifecycle_event_listener)\
     MACRO(input_dispatcher)\
@@ -98,7 +97,6 @@ struct TemporaryCompositeEventFilter : public mi::CompositeEventFilter
     MACRO(server_status_listener)\
     MACRO(session_authorizer)\
     MACRO(session_listener)\
-    MACRO(session_mediator_report)\
     MACRO(shell)\
     MACRO(application_not_responding_detector)\
     MACRO(cookie_authority)\
@@ -131,7 +129,10 @@ struct TemporaryCompositeEventFilter : public mi::CompositeEventFilter
     MACRO(the_touch_visualizer)\
     MACRO(the_input_device_hub)\
     MACRO(the_application_not_responding_detector)\
-    MACRO(the_persistent_surface_store)
+    MACRO(the_persistent_surface_store)\
+    MACRO(the_display_configuration_observer_registrar)\
+    MACRO(the_seat_observer_registrar)\
+    MACRO(the_session_mediator_observer_registrar)
 
 #define MIR_SERVER_BUILDER(name)\
     std::function<std::result_of<decltype(&mir::DefaultServerConfiguration::the_##name)(mir::DefaultServerConfiguration*)>::type()> name##_builder;
@@ -147,6 +148,7 @@ struct mir::Server::Self
     std::string config_file;
     std::shared_ptr<ServerConfiguration> server_config;
 
+    std::function<void()> pre_init_callback{[]{}};
     std::function<void()> init_callback{[]{}};
     std::function<void()> stop_callback{[]{}};
     int argc{0};
@@ -201,52 +203,6 @@ auto wrap_##name(decltype(Self::name##_wrapper)::result_type const& wrapped)\
     return mir::DefaultServerConfiguration::wrap_##name(wrapped);\
 }
 
-// TODO these are used to frig a stub renderer when running headless
-namespace
-{
-class StubGLRenderer : public mir::compositor::Renderer
-{
-public:
-    StubGLRenderer(mir::graphics::DisplayBuffer& display_buffer)
-        : render_target{
-            dynamic_cast<mir::renderer::gl::RenderTarget*>(
-                display_buffer.native_display_buffer())}
-    {
-    }
-
-    void set_viewport(mir::geometry::Rectangle const&) override {}
-    void set_output_transform(MirOrientation, MirMirrorMode) override {}
-
-    void render(mir::graphics::RenderableList const& renderables) const override
-    {
-        // Invoke GL renderer specific functions if the DisplayBuffer supports them
-        if (render_target)
-            render_target->make_current();
-
-        for (auto const& r : renderables)
-            r->buffer(); // We need to consume a buffer to unblock client tests
-
-        // Invoke GL renderer specific functions if the DisplayBuffer supports them
-        if (render_target)
-            render_target->swap_buffers();
-    }
-
-    void suspend() override {}
-
-    mir::renderer::gl::RenderTarget* const render_target;
-};
-
-class StubGLRendererFactory : public mir::compositor::RendererFactory
-{
-public:
-    auto create_renderer_for(mir::graphics::DisplayBuffer& display_buffer)
-    -> std::unique_ptr<mir::compositor::Renderer>
-    {
-        return std::make_unique<StubGLRenderer>(display_buffer);
-    }
-};
-}
-
 struct mir::Server::ServerConfiguration : mir::DefaultServerConfiguration
 {
     ServerConfiguration(
@@ -257,20 +213,6 @@ struct mir::Server::ServerConfiguration : mir::DefaultServerConfiguration
     {
     }
 
-    // TODO this is an ugly frig to avoid exposing the render factory to end users and tests running headless
-    auto the_renderer_factory() -> std::shared_ptr<compositor::RendererFactory> override
-    {
-        auto const& options = the_options();
-        if (options->is_set(options::platform_graphics_lib))
-        {
-            auto const graphics_lib = options->get<std::string>(options::platform_graphics_lib);
-
-            if (graphics_lib.find("graphics-dummy.so") != std::string::npos)
-                return std::make_shared<StubGLRendererFactory>();
-        }
-        return mir::DefaultServerConfiguration::the_renderer_factory();
-    }
-
     std::function<void()> the_stop_callback() override
     {
         return self->stop_callback;
@@ -346,6 +288,19 @@ void mir::Server::set_command_line(int argc, char const* argv[])
     self->argv = argv;
 }
 
+void mir::Server::add_pre_init_callback(std::function<void()> const& pre_init_callback)
+{
+    auto const& existing = self->pre_init_callback;
+
+    auto const updated = [=]
+        {
+            existing();
+            pre_init_callback();
+        };
+
+    self->pre_init_callback = updated;
+}
+
 void mir::Server::add_init_callback(std::function<void()> const& init_callback)
 {
     auto const& existing = self->init_callback;
@@ -446,6 +401,8 @@ void mir::Server::run()
         if (self->emergency_cleanup_handler)
             emergency_cleanup->add(self->emergency_cleanup_handler);
 
+        self->pre_init_callback();
+
         run_mir(
             *self->server_config,
             [&](DisplayServer&)
