@@ -24,6 +24,7 @@
 #include "mir/shell/window_manager.h"
 #include "mir/scene/prompt_session.h"
 #include "mir/scene/prompt_session_manager.h"
+#include "mir/scene/null_surface_observer.h"
 #include "mir/scene/session_coordinator.h"
 #include "mir/scene/session.h"
 #include "mir/scene/surface.h"
@@ -33,6 +34,37 @@ namespace mf = mir::frontend;
 namespace ms = mir::scene;
 namespace mi = mir::input;
 namespace msh = mir::shell;
+namespace geom = mir::geometry;
+
+namespace
+{
+
+struct UpdateConfinementOnSurfaceChanges : ms::NullSurfaceObserver
+{
+    UpdateConfinementOnSurfaceChanges(msh::AbstractShell* shell) :
+        shell(shell)
+    {
+    }
+
+    void resized_to(geom::Size const& /*size*/) override
+    {
+        update_confinement_region();
+    }
+
+    void moved_to(geom::Point const& /*top_left*/) override
+    {
+        update_confinement_region();
+    }
+
+private:
+    void update_confinement_region()
+    {
+        shell->update_focused_surface_confined_region();
+    }
+
+    msh::AbstractShell* shell;
+};
+}
 
 msh::AbstractShell::AbstractShell(
     std::shared_ptr<InputTargeter> const& input_targeter,
@@ -48,7 +80,8 @@ msh::AbstractShell::AbstractShell(
     prompt_session_manager(prompt_session_manager),
     window_manager(wm_builder(this)),
     seat(seat),
-    report(report)
+    report(report),
+    focus_surface_observer(std::make_shared<UpdateConfinementOnSurfaceChanges>(this))
 {
 }
 
@@ -56,6 +89,16 @@ msh::AbstractShell::~AbstractShell() noexcept
 {
 }
 
+void msh::AbstractShell::update_focused_surface_confined_region()
+{
+    auto const current_focus = focus_surface.lock();
+
+    if (current_focus && current_focus->confine_pointer_state() == mir_pointer_confined_to_window)
+    {
+        seat->set_confinement_regions({current_focus->input_bounds()});
+    }
+}
+
 std::shared_ptr<ms::Session> msh::AbstractShell::open_session(
     pid_t client_pid,
     std::string const& name,
@@ -118,9 +161,29 @@ void msh::AbstractShell::modify_surface(std::shared_ptr<scene::Session> const& s
         window_manager->modify_surface(session, surface, wm_relevant_mods);
     }
 
-    if (modifications.confine_pointer.is_set())
+    if (modifications.cursor_image.is_set())
     {
-        if (surface->confine_pointer_state() == mir_pointer_confined_to_surface)
+        surface->set_cursor_image(modifications.cursor_image.value());
+    }
+
+    if (modifications.stream_cursor.is_set())
+    {
+        auto stream_id = modifications.stream_cursor.value().stream_id;
+        if (stream_id != mir::frontend::BufferStreamId{-1})
+        {
+            auto hotspot = modifications.stream_cursor.value().hotspot;
+            auto stream = session->get_buffer_stream(modifications.stream_cursor.value().stream_id);
+            surface->set_cursor_stream(stream, hotspot);
+        }
+        else
+        {
+            surface->set_cursor_image({});
+        }
+    }
+
+    if (modifications.confine_pointer.is_set() && focused_surface() == surface)
+    {
+        if (surface->confine_pointer_state() == mir_pointer_confined_to_window)
         {
             seat->set_confinement_regions({surface->input_bounds()});
         }
@@ -166,7 +229,7 @@ void msh::AbstractShell::stop_prompt_session(
 int msh::AbstractShell::set_surface_attribute(
     std::shared_ptr<ms::Session> const& session,
     std::shared_ptr<ms::Surface> const& surface,
-    MirSurfaceAttrib attrib,
+    MirWindowAttrib attrib,
     int value)
 {
     report->update_surface(*session, *surface, attrib, value);
@@ -175,7 +238,7 @@ int msh::AbstractShell::set_surface_attribute(
 
 int msh::AbstractShell::get_surface_attribute(
     std::shared_ptr<ms::Surface> const& surface,
-    MirSurfaceAttrib attrib)
+    MirWindowAttrib attrib)
 {
     return surface->query(attrib);
 }
@@ -193,12 +256,14 @@ void msh::AbstractShell::focus_next_session()
     std::unique_lock<std::mutex> lock(focus_mutex);
     auto const focused_session = focus_session.lock();
     auto successor = session_coordinator->successor_of(focused_session);
+    auto const sentinel = successor;
 
     while (successor != nullptr &&
-           successor != focused_session &&
            successor->default_surface() == nullptr)
     {
         successor = session_coordinator->successor_of(successor);
+        if (successor == sentinel)
+            break;
     }
 
     auto const surface = successor ? successor->default_surface() : nullptr;
@@ -243,11 +308,14 @@ void msh::AbstractShell::set_focus_to_locked(
         seat->reset_confinement_regions();
 
         if (current_focus)
-            current_focus->configure(mir_surface_attrib_focus, mir_surface_unfocused);
+        {
+            current_focus->configure(mir_window_attrib_focus, mir_window_focus_state_unfocused);
+            current_focus->remove_observer(focus_surface_observer);
+        }
 
         if (surface)
         {
-            if (surface->confine_pointer_state() == mir_pointer_confined_to_surface)
+            if (surface->confine_pointer_state() == mir_pointer_confined_to_window)
             {
                 seat->set_confinement_regions({surface->input_bounds()});
             }
@@ -255,7 +323,8 @@ void msh::AbstractShell::set_focus_to_locked(
             // Ensure the surface has really taken the focus before notifying it that it is focused
             input_targeter->set_focus(surface);
             surface->consume(seat->create_device_state().get());
-            surface->configure(mir_surface_attrib_focus, mir_surface_focused);
+            surface->configure(mir_window_attrib_focus, mir_window_focus_state_focused);
+            surface->add_observer(focus_surface_observer);
         }
         else
         {
@@ -311,6 +380,10 @@ bool msh::AbstractShell::handle(MirEvent const& event)
 
     case mir_input_event_type_pointer:
         return window_manager->handle_pointer_event(mir_input_event_get_pointer_event(input_event));
+    
+    case mir_input_event_types:
+        abort();
+        return false;
     }
 
     return false;
@@ -327,3 +400,4 @@ void msh::AbstractShell::raise(SurfaceSet const& surfaces)
     surface_stack->raise(surfaces);
     report->surfaces_raised(surfaces);
 }
+
