@@ -16,13 +16,12 @@
  * Author: Daniel van Vugt <daniel.van.vugt@canonical.com>
  */
 
-#define _POSIX_C_SOURCE 200112L  // for setenv() from stdlib.h
 #include "eglapp.h"
 #include <assert.h>
 #include <stdio.h>
 #include <math.h>
 #include <GLES2/gl2.h>
-#include <mir_toolkit/mir_surface.h>
+#include <mir_toolkit/mir_window.h>
 #include <pthread.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -181,7 +180,7 @@ static void get_all_touch_points(const MirInputEvent *ievent, TouchState *touch)
     }
 }
 
-static void on_event(MirSurface *surface, const MirEvent *event, void *context)
+static void on_event(MirWindow *surface, const MirEvent *event, void *context)
 {
     (void)surface;
     State *state = (State*)context;
@@ -199,7 +198,7 @@ static void on_event(MirSurface *surface, const MirEvent *event, void *context)
     case mir_event_type_resize:
         state->resized = true;
         break;
-    case mir_event_type_close_surface:
+    case mir_event_type_close_window:
         state->running = false;
         break;
     default:
@@ -265,13 +264,8 @@ int main(int argc, char *argv[])
     sigaddset(&sigs, SIGHUP);
     pthread_sigmask(SIG_BLOCK, &sigs, NULL);
 
-    // Disable Mir's input resampling. We do our own here, in a way that
-    // has even lower latency than Mir's default algorithm.
-    // TODO: Make a proper client API function for this:
-    setenv("MIR_CLIENT_INPUT_RATE", "0", 0);
-
     static unsigned int width = 0, height = 0;
-    if (!mir_eglapp_init(argc, argv, &width, &height))
+    if (!mir_eglapp_init(argc, argv, &width, &height, NULL))
         return 1;
 
     GLuint vshader = load_shader(vshadersrc, GL_VERTEX_SHADER);
@@ -351,8 +345,8 @@ int main(int argc, char *argv[])
         return 3;
     }
 
-    MirSurface *surface = mir_eglapp_native_surface();
-    mir_surface_set_event_handler(surface, on_event, &state);
+    MirWindow* window = mir_eglapp_native_window();
+    mir_window_set_event_handler(window, on_event, &state);
 
     while (true)
     {
@@ -421,8 +415,8 @@ int main(int argc, char *argv[])
         mir_eglapp_swap_buffers();
     }
 
-    mir_surface_set_event_handler(surface, NULL, NULL);
-    mir_eglapp_shutdown();
+    mir_window_set_event_handler(window, NULL, NULL);
+    mir_eglapp_cleanup();
 
     pthread_join(shutdown_handler_thread, NULL);
     return 0;
