@@ -44,43 +44,10 @@ namespace mg = mir::geometry;
 
 namespace
 {
-struct SurfaceSync
-{
-    void surface_created(MirSurface * new_surface)
-    {
-        std::unique_lock<std::mutex> lock(guard);
-        surface = new_surface;
-        signal.notify_all();
-    }
-
-    void surface_released(MirSurface * /*released_surface*/)
-    {
-        std::unique_lock<std::mutex> lock(guard);
-        surface = nullptr;
-        signal.notify_all();
-    }
-
-    void wait_for_surface_create()
-    {
-        std::unique_lock<std::mutex> lock(guard);
-        signal.wait(lock, [&]{ return !!surface; });
-    }
-
-    void wait_for_surface_release()
-    {
-        std::unique_lock<std::mutex> lock(guard);
-        signal.wait(lock, [&]{ return !surface; });
-    }
-
-    std::mutex guard;
-    std::condition_variable signal;
-    MirSurface * surface{nullptr};
-};
-
 struct ClientSurfaces : mtf::ConnectedClientHeadlessServer
 {
     static const int max_surface_count = 5;
-    SurfaceSync ssync[max_surface_count];
+    MirWindow* window[max_surface_count] = {nullptr};
 
     void SetUp() override
     {
@@ -93,64 +60,38 @@ struct ClientSurfaces : mtf::ConnectedClientHeadlessServer
 
     testing::NiceMock<mtd::MockWindowManager> window_manager;
 };
-
-extern "C" void create_surface_callback(MirSurface* surface, void * context)
-{
-    SurfaceSync* config = reinterpret_cast<SurfaceSync*>(context);
-    config->surface_created(surface);
-}
-
-extern "C" void release_surface_callback(MirSurface* surface, void * context)
-{
-    SurfaceSync* config = reinterpret_cast<SurfaceSync*>(context);
-    config->surface_released(surface);
-}
-
-void wait_for_surface_create(SurfaceSync* context)
-{
-    context->wait_for_surface_create();
-}
-
-void wait_for_surface_release(SurfaceSync* context)
-{
-    context->wait_for_surface_release();
-}
 }
 
 TEST_F(ClientSurfaces, are_created_with_correct_size)
 {
     int width_1 = 640, height_1 = 480, width_2 = 1600, height_2 = 1200;
     
-    auto spec = mir_connection_create_spec_for_normal_surface(connection, width_1, height_1, mir_pixel_format_abgr_8888);
-    mir_surface_create(spec, create_surface_callback, ssync);
-    wait_for_surface_create(ssync);
+    auto spec = mir_create_normal_window_spec(connection, width_1, height_1);
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
+    window[0] = mir_create_window_sync(spec);
 
-    mir_surface_spec_set_width(spec, width_2);
-    mir_surface_spec_set_height(spec, height_2);
+    mir_window_spec_set_width(spec, width_2);
+    mir_window_spec_set_height(spec, height_2);
 
-    mir_surface_create(spec, create_surface_callback, ssync+1);
-    wait_for_surface_create(ssync+1);
+    window[1] = mir_create_window_sync(spec);
     
-    mir_surface_spec_release(spec);
+    mir_window_spec_release(spec);
 
-    MirSurfaceParameters response_params;
-    mir_surface_get_parameters(ssync->surface, &response_params);
+    MirWindowParameters response_params;
+    mir_window_get_parameters(window[0], &response_params);
     EXPECT_EQ(640, response_params.width);
     EXPECT_EQ(480, response_params.height);
     EXPECT_EQ(mir_pixel_format_abgr_8888, response_params.pixel_format);
     EXPECT_EQ(mir_buffer_usage_hardware, response_params.buffer_usage);
 
-    mir_surface_get_parameters(ssync[1].surface, &response_params);
+    mir_window_get_parameters(window[1], &response_params);
     EXPECT_EQ(1600, response_params.width);
     EXPECT_EQ(1200, response_params.height);
     EXPECT_EQ(mir_pixel_format_abgr_8888, response_params.pixel_format);
     EXPECT_EQ(mir_buffer_usage_hardware, response_params.buffer_usage);
 
-    mir_surface_release(ssync[1].surface, release_surface_callback, ssync+1);
-    wait_for_surface_release(ssync+1);
-
-    mir_surface_release(ssync->surface, release_surface_callback, ssync);
-    wait_for_surface_release(ssync);
+    mir_window_release_sync(window[1]);
+    mir_window_release_sync(window[0]);
 }
 
 TEST_F(ClientSurfaces, have_distinct_ids)
@@ -158,64 +99,60 @@ TEST_F(ClientSurfaces, have_distinct_ids)
     auto surface_1 = mtf::make_any_surface(connection);
     auto surface_2 = mtf::make_any_surface(connection);
     
-    EXPECT_NE(mir_debug_surface_id(surface_1),
-        mir_debug_surface_id(surface_2));
+    EXPECT_NE(mir_debug_window_id(surface_1),
+        mir_debug_window_id(surface_2));
 
-    mir_surface_release_sync(surface_1);
-    mir_surface_release_sync(surface_2);
+    mir_window_release_sync(surface_1);
+    mir_window_release_sync(surface_2);
 }
 
 TEST_F(ClientSurfaces, creates_need_not_be_serialized)
 {
     for (int i = 0; i != max_surface_count; ++i)
     {
-        auto spec = mir_connection_create_spec_for_normal_surface(connection, 1, 1, mir_pixel_format_abgr_8888);
-        mir_surface_create(spec, create_surface_callback, ssync+i);
-        mir_surface_spec_release(spec);
+        auto spec = mir_create_normal_window_spec(connection, 1, 1);
+        mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
+        window[i] = mir_create_window_sync(spec);
+        mir_window_spec_release(spec);
     }
 
-    for (int i = 0; i != max_surface_count; ++i)
-        wait_for_surface_create(ssync+i);
-
     for (int i = 0; i != max_surface_count; ++i)
     {
         for (int j = 0; j != max_surface_count; ++j)
         {
             if (i == j)
                 EXPECT_EQ(
-                    mir_debug_surface_id(ssync[i].surface),
-                    mir_debug_surface_id(ssync[j].surface));
+                    mir_debug_window_id(window[i]),
+                    mir_debug_window_id(window[j]));
             else
                 EXPECT_NE(
-                    mir_debug_surface_id(ssync[i].surface),
-                    mir_debug_surface_id(ssync[j].surface));
+                    mir_debug_window_id(window[i]),
+                    mir_debug_window_id(window[j]));
         }
     }
 
     for (int i = 0; i != max_surface_count; ++i)
-        mir_surface_release(ssync[i].surface, release_surface_callback, ssync+i);
-
-    for (int i = 0; i != max_surface_count; ++i)
-        wait_for_surface_release(ssync+i);
+        mir_window_release_sync(window[i]);
 }
 
 struct WithOrientation : ClientSurfaces, ::testing::WithParamInterface<MirOrientationMode> {};
 
 TEST_P(WithOrientation, have_requested_preferred_orientation)
 {
-    auto spec = mir_connection_create_spec_for_normal_surface(connection, 1, 1, mir_pixel_format_abgr_8888);
+    auto spec = mir_create_normal_window_spec(connection, 1, 1);
     ASSERT_THAT(spec, NotNull());
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
 
     MirOrientationMode mode{GetParam()};
-    mir_surface_spec_set_preferred_orientation(spec, mode);
+    mir_window_spec_set_preferred_orientation(spec, mode);
 
-    auto surface = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    auto window = mir_create_window_sync(spec);
+    mir_window_spec_release(spec);
 
-    ASSERT_THAT(surface, IsValid());
-    EXPECT_EQ(mir_surface_get_preferred_orientation(surface), mode);
+    ASSERT_THAT(window, IsValid());
+    EXPECT_EQ(mir_window_get_preferred_orientation(window), mode);
 
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(window);
 }
 
 INSTANTIATE_TEST_CASE_P(ClientSurfaces,
@@ -230,92 +167,94 @@ TEST_F(ClientSurfaces, can_be_menus)
     auto parent = mtf::make_any_surface(connection);
     MirRectangle attachment_rect{100, 200, 100, 100};
 
-    auto spec = mir_connection_create_spec_for_menu(connection, 640, 480,
-        mir_pixel_format_abgr_8888, parent, &attachment_rect, mir_edge_attachment_vertical);
+    auto spec = mir_create_menu_window_spec(connection, 640, 480,
+        parent, &attachment_rect, mir_edge_attachment_vertical);
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
     ASSERT_THAT(spec, NotNull());
 
-    auto menu = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    auto menu = mir_create_window_sync(spec);
+    mir_window_spec_release(spec);
 
     ASSERT_THAT(menu, IsValid());
-    EXPECT_EQ(mir_surface_get_type(menu), mir_surface_type_menu);
+    EXPECT_EQ(mir_window_get_type(menu), mir_window_type_menu);
 
-    mir_surface_release_sync(parent);
-    mir_surface_release_sync(menu);
+    mir_window_release_sync(parent);
+    mir_window_release_sync(menu);
 }
 
-TEST_F(ClientSurfaces, can_be_tooltips)
+TEST_F(ClientSurfaces, can_be_tips)
 {
     auto parent = mtf::make_any_surface(connection);
-    MirRectangle zone_rect{100, 200, 100, 100};
+    MirRectangle rect{100, 200, 100, 100};
 
-    auto spec = mir_connection_create_spec_for_tooltip(connection, 640, 480,
-        mir_pixel_format_abgr_8888, parent, &zone_rect);
+    auto spec = mir_create_tip_window_spec(connection, 640, 480,
+        parent, &rect, mir_edge_attachment_any);
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
     ASSERT_THAT(spec, NotNull());
 
-    auto tooltip = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    auto tooltip = mir_create_window_sync(spec);
+    mir_window_spec_release(spec);
 
     ASSERT_THAT(tooltip, IsValid());
-    EXPECT_EQ(mir_surface_get_type(tooltip), mir_surface_type_tip);
+    EXPECT_EQ(mir_window_get_type(tooltip), mir_window_type_tip);
 
-    mir_surface_release_sync(parent);
-    mir_surface_release_sync(tooltip);
+    mir_window_release_sync(parent);
+    mir_window_release_sync(tooltip);
 }
 
 TEST_F(ClientSurfaces, can_be_dialogs)
 {
-    auto spec = mir_connection_create_spec_for_dialog(connection, 640, 480,
-        mir_pixel_format_abgr_8888);
+    auto spec = mir_create_dialog_window_spec(connection, 640, 480);
     ASSERT_THAT(spec, NotNull());
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
 
-    auto dialog = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    auto dialog = mir_create_window_sync(spec);
+    mir_window_spec_release(spec);
 
     ASSERT_THAT(dialog, IsValid());
-    EXPECT_EQ(mir_surface_get_type(dialog), mir_surface_type_dialog);
+    EXPECT_EQ(mir_window_get_type(dialog), mir_window_type_dialog);
 
-    mir_surface_release_sync(dialog);
+    mir_window_release_sync(dialog);
 }
 
 TEST_F(ClientSurfaces, can_be_modal_dialogs)
 {
     auto parent = mtf::make_any_surface(connection);
-    auto spec = mir_connection_create_spec_for_modal_dialog(connection, 640, 480,
-        mir_pixel_format_abgr_8888, parent);
+    auto spec = mir_create_modal_dialog_window_spec(connection, 640, 480, parent);
     ASSERT_THAT(spec, NotNull());
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
 
-    auto dialog = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    auto dialog = mir_create_window_sync(spec);
+    mir_window_spec_release(spec);
 
     ASSERT_THAT(dialog, IsValid());
-    EXPECT_EQ(mir_surface_get_type(dialog), mir_surface_type_dialog);
+    EXPECT_EQ(mir_window_get_type(dialog), mir_window_type_dialog);
 
-    mir_surface_release_sync(parent);
-    mir_surface_release_sync(dialog);
+    mir_window_release_sync(parent);
+    mir_window_release_sync(dialog);
 }
 
 TEST_F(ClientSurfaces, can_be_input_methods)
 {
-    auto spec = mir_connection_create_spec_for_input_method(connection, 640, 480,
-        mir_pixel_format_abgr_8888);
+    auto spec = mir_create_input_method_window_spec(connection, 640, 480);
     ASSERT_THAT(spec, NotNull());
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
 
-    auto im = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    auto im = mir_create_window_sync(spec);
+    mir_window_spec_release(spec);
 
-    EXPECT_EQ(mir_surface_get_type(im), mir_surface_type_inputmethod);
+    EXPECT_EQ(mir_window_get_type(im), mir_window_type_inputmethod);
 
-    mir_surface_release_sync(im);
+    mir_window_release_sync(im);
 }
 
 TEST_F(ClientSurfaces, can_be_renamed)
 {
-    auto spec = mir_connection_create_spec_for_normal_surface(
-                   connection, 123, 456, mir_pixel_format_abgr_8888);
+    auto spec = mir_create_normal_window_spec(connection, 123, 456);
     ASSERT_THAT(spec, NotNull());
-    auto surf = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
+    auto window = mir_create_window_sync(spec);
+    mir_window_spec_release(spec);
 
     /*
      * Generally no windowing system ever censors window names. They are
@@ -327,21 +266,23 @@ TEST_F(ClientSurfaces, can_be_renamed)
      *
      * At least verify the rename completes without blocking...
      */
-    spec = mir_connection_create_spec_for_changes(connection);
+    spec = mir_create_window_spec(connection);
     ASSERT_THAT(spec, NotNull());
-    mir_surface_spec_set_name(spec, "New Name");
-    mir_surface_apply_spec(surf, spec);
+    mir_window_spec_set_name(spec, "New Name");
+    mir_window_apply_spec(window, spec);
 
-    mir_surface_spec_set_name(spec, "");
-    mir_surface_apply_spec(surf, spec);
+    mir_window_spec_set_name(spec, "");
+    mir_window_apply_spec(window, spec);
 
-    mir_surface_spec_set_name(spec, "Alice");
-    mir_surface_apply_spec(surf, spec);
-    mir_surface_spec_release(spec);
+    mir_window_spec_set_name(spec, "Alice");
+    mir_window_apply_spec(window, spec);
+    mir_window_spec_release(spec);
 
-    mir_surface_release_sync(surf);
+    mir_window_release_sync(window);
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 TEST_F(ClientSurfaces, input_methods_get_corret_parent_coordinates)
 {
     using namespace testing;
@@ -370,26 +311,27 @@ TEST_F(ClientSurfaces, input_methods_get_corret_parent_coordinates)
         return builder(session, params);
     }));
 
-    auto surface = mtf::make_any_surface(connection);
+    auto window = mtf::make_any_surface(connection);
 
-    auto parent_id = mir_surface_request_persistent_id_sync(surface);
+    auto parent_id = mir_window_request_persistent_id_sync(window);
 
     auto im_connection = mir_connect_sync(new_connection().c_str(), "Mock IM connection");
     ASSERT_THAT(im_connection, IsValid());
 
-    auto spec = mir_connection_create_spec_for_input_method(im_connection, 100, 20,
-        mir_pixel_format_abgr_8888);
+    auto spec = mir_create_input_method_window_spec(im_connection, 100, 20);
+    mir_window_spec_set_pixel_format(spec, mir_pixel_format_abgr_8888);
     ASSERT_THAT(spec, NotNull());
 
-    mir_surface_spec_attach_to_foreign_parent(spec, parent_id, &client_rect, edge_attachment);
+    mir_window_spec_attach_to_foreign_parent(spec, parent_id, &client_rect, edge_attachment);
 
     mir_persistent_id_release(parent_id);
 
-    auto im = mir_surface_create_sync(spec);
-    mir_surface_spec_release(spec);
+    auto im = mir_create_window_sync(spec);
+    mir_window_spec_release(spec);
 
-    mir_surface_release_sync(im);
-    mir_surface_release_sync(surface);
+    mir_window_release_sync(im);
+    mir_window_release_sync(window);
 
     mir_connection_release(im_connection);
 }
+#pragma GCC diagnostic pop
