@@ -39,6 +39,8 @@ namespace mtd = mir::test::doubles;
 using namespace mir::geometry;
 using namespace testing;
 
+using namespace std::literals::chrono_literals;
+
 namespace
 {
 class MockSurfaceObserver : public ms::NullSurfaceObserver
@@ -70,8 +72,8 @@ struct SurfaceModifications : mtf::ConnectedClientWithASurface
         auto const scene_surface = shell_surface.lock();
         scene_surface->add_observer(mt::fake_shared(surface_observer));
 
-        // Swap buffers to ensure surface is visible for event based tests
-        mir_buffer_stream_swap_buffers_sync(mir_surface_get_buffer_stream(surface));
+        // Swap buffers to ensure window is visible for event based tests
+        mir_buffer_stream_swap_buffers_sync(mir_window_get_buffer_stream(window));
     }
 
     void generate_alt_click_at(Point const& click_position)
@@ -121,9 +123,9 @@ struct SurfaceModifications : mtf::ConnectedClientWithASurface
         EXPECT_CALL(surface_observer, renamed(StrEq(new_title))).
             WillOnce(InvokeWithoutArgs([&]{ server_ready.raise(); }));
 
-        apply_changes([&](MirSurfaceSpec* spec)
+        apply_changes([&](MirWindowSpec* spec)
             {
-                mir_surface_spec_set_name(spec, new_title);
+                mir_window_spec_set_name(spec, new_title);
             });
 
         server_ready.wait();
@@ -132,12 +134,12 @@ struct SurfaceModifications : mtf::ConnectedClientWithASurface
     template<typename Specifier>
     void apply_changes(Specifier const& specifier) const
     {
-        auto const spec = mir_connection_create_spec_for_changes(connection);
+        auto const spec = mir_create_window_spec(connection);
 
         specifier(spec);
 
-        mir_surface_apply_spec(surface, spec);
-        mir_surface_spec_release(spec);
+        mir_window_apply_spec(window, spec);
+        mir_window_spec_release(spec);
     }
 
     MirInputDeviceId const device_id = MirInputDeviceId(7);
@@ -159,19 +161,19 @@ MATCHER_P(HeightEq, value, "")
 
 struct StatePair
 {
-    MirSurfaceState from;
-    MirSurfaceState to;
+    MirWindowState from;
+    MirWindowState to;
 
     friend std::ostream& operator<<(std::ostream& out, StatePair const& types)
         { return out << "from:" << types.from << ", to:" << types.to; }
 };
 
-bool is_visible(MirSurfaceState state)
+bool is_visible(MirWindowState state)
 {
     switch (state)
     {
-    case mir_surface_state_hidden:
-    case mir_surface_state_minimized:
+    case mir_window_state_hidden:
+    case mir_window_state_minimized:
         return false;
     default:
         break;
@@ -189,9 +191,9 @@ TEST_F(SurfaceModifications, surface_spec_name_is_notified)
 
     EXPECT_CALL(surface_observer, renamed(StrEq(new_title)));
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_name(spec, new_title);
+            mir_window_spec_set_name(spec, new_title);
         });
 }
 
@@ -202,10 +204,10 @@ TEST_F(SurfaceModifications, surface_spec_resize_is_notified)
 
     EXPECT_CALL(surface_observer, resized_to(Size{new_width, new_height}));
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_width(spec, new_width);
-            mir_surface_spec_set_height(spec, new_height);
+            mir_window_spec_set_width(spec, new_width);
+            mir_window_spec_set_height(spec, new_height);
         });
 }
 
@@ -215,9 +217,9 @@ TEST_F(SurfaceModifications, surface_spec_change_width_is_notified)
 
     EXPECT_CALL(surface_observer, resized_to(WidthEq(new_width)));
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_width(spec, new_width);
+            mir_window_spec_set_width(spec, new_width);
         });
 }
 
@@ -227,9 +229,9 @@ TEST_F(SurfaceModifications, surface_spec_change_height_is_notified)
 
     EXPECT_CALL(surface_observer, resized_to(HeightEq(new_height)));
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_height(spec, new_height);
+            mir_window_spec_set_height(spec, new_height);
         });
 }
 
@@ -237,9 +239,9 @@ TEST_F(SurfaceModifications, surface_spec_min_width_is_respected)
 {
     auto const min_width = 17;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_min_width(spec, min_width);
+            mir_window_spec_set_min_width(spec, min_width);
         });
 
     ensure_server_has_processed_setup();
@@ -257,9 +259,9 @@ TEST_F(SurfaceModifications, surface_spec_min_height_is_respected)
 {
     auto const min_height = 19;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_min_height(spec, min_height);
+            mir_window_spec_set_min_height(spec, min_height);
         });
 
     ensure_server_has_processed_setup();
@@ -277,9 +279,9 @@ TEST_F(SurfaceModifications, surface_spec_max_width_is_respected)
 {
     auto const max_width = 23;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_max_width(spec, max_width);
+            mir_window_spec_set_max_width(spec, max_width);
         });
 
     ensure_server_has_processed_setup();
@@ -297,9 +299,9 @@ TEST_F(SurfaceModifications, surface_spec_max_height_is_respected)
 {
     auto const max_height = 29;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_max_height(spec, max_height);
+            mir_window_spec_set_max_height(spec, max_height);
         });
 
     ensure_server_has_processed_setup();
@@ -317,9 +319,9 @@ TEST_F(SurfaceModifications, surface_spec_width_inc_is_respected)
 {
     auto const width_inc = 13;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_width_increment(spec, width_inc);
+            mir_window_spec_set_width_increment(spec, width_inc);
         });
 
     ensure_server_has_processed_setup();
@@ -341,10 +343,10 @@ TEST_F(SurfaceModifications, surface_spec_with_min_width_and_width_inc_is_respec
     auto const width_inc = 13;
     auto const min_width = 7;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_width_increment(spec, width_inc);
-            mir_surface_spec_set_min_width(spec, min_width);
+            mir_window_spec_set_width_increment(spec, width_inc);
+            mir_window_spec_set_min_width(spec, min_width);
         });
 
     ensure_server_has_processed_setup();
@@ -365,9 +367,9 @@ TEST_F(SurfaceModifications, surface_spec_height_inc_is_respected)
 {
     auto const height_inc = 13;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_height_increment(spec, height_inc);
+            mir_window_spec_set_height_increment(spec, height_inc);
         });
 
     ensure_server_has_processed_setup();
@@ -389,10 +391,10 @@ TEST_F(SurfaceModifications, surface_spec_with_min_height_and_height_inc_is_resp
     auto const height_inc = 13;
     auto const min_height = 7;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_height_increment(spec, height_inc);
-            mir_surface_spec_set_min_height(spec, min_height);
+            mir_window_spec_set_height_increment(spec, height_inc);
+            mir_window_spec_set_min_height(spec, min_height);
         });
 
     ensure_server_has_processed_setup();
@@ -414,9 +416,9 @@ TEST_F(SurfaceModifications, surface_spec_with_min_aspect_ratio_is_respected)
     auto const aspect_width = 11;
     auto const aspect_height = 7;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_min_aspect_ratio(spec, aspect_width, aspect_height);
+            mir_window_spec_set_min_aspect_ratio(spec, aspect_width, aspect_height);
         });
 
     ensure_server_has_processed_setup();
@@ -439,9 +441,9 @@ TEST_F(SurfaceModifications, surface_spec_with_max_aspect_ratio_is_respected)
     auto const aspect_width = 7;
     auto const aspect_height = 11;
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_max_aspect_ratio(spec, aspect_width, aspect_height);
+            mir_window_spec_set_max_aspect_ratio(spec, aspect_width, aspect_height);
         });
 
     ensure_server_has_processed_setup();
@@ -473,22 +475,22 @@ TEST_F(SurfaceModifications, surface_spec_with_fixed_aspect_ratio_and_size_range
     Size actual;
     EXPECT_CALL(surface_observer, resized_to(_)).Times(AnyNumber()).WillRepeatedly(SaveArg<0>(&actual));
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
           {
-              mir_surface_spec_set_min_aspect_ratio(spec, aspect_width, aspect_height);
-              mir_surface_spec_set_max_aspect_ratio(spec, aspect_width, aspect_height);
+              mir_window_spec_set_min_aspect_ratio(spec, aspect_width, aspect_height);
+              mir_window_spec_set_max_aspect_ratio(spec, aspect_width, aspect_height);
 
-              mir_surface_spec_set_min_height(spec, min_height);
-              mir_surface_spec_set_min_width(spec, min_width);
+              mir_window_spec_set_min_height(spec, min_height);
+              mir_window_spec_set_min_width(spec, min_width);
 
-              mir_surface_spec_set_max_height(spec, max_height);
-              mir_surface_spec_set_max_width(spec, max_width);
+              mir_window_spec_set_max_height(spec, max_height);
+              mir_window_spec_set_max_width(spec, max_width);
 
-              mir_surface_spec_set_width_increment(spec, width_inc);
-              mir_surface_spec_set_height_increment(spec, height_inc);
+              mir_window_spec_set_width_increment(spec, width_inc);
+              mir_window_spec_set_height_increment(spec, height_inc);
 
-              mir_surface_spec_set_height(spec, min_height);
-              mir_surface_spec_set_width(spec, min_width);
+              mir_window_spec_set_height(spec, min_height);
+              mir_window_spec_set_width(spec, min_width);
           });
 
     ensure_server_has_processed_setup();
@@ -521,13 +523,13 @@ TEST_F(SurfaceModifications, surface_spec_with_fixed_aspect_ratio_and_size_range
 
 TEST_F(SurfaceModifications, surface_spec_state_affects_surface_visibility)
 {
-    auto const new_state = mir_surface_state_hidden;
+    auto const new_state = mir_window_state_hidden;
 
     EXPECT_CALL(surface_observer, hidden_set_to(true));
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_state(spec, new_state);
+            mir_window_spec_set_state(spec, new_state);
         });
 }
 
@@ -539,29 +541,29 @@ TEST_P(SurfaceSpecStateCase, set_state_affects_surface_visibility)
     EXPECT_CALL(surface_observer, hidden_set_to(is_visible(initial_state)));
     EXPECT_CALL(surface_observer, hidden_set_to(is_visible(new_state)));
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
         {
-            mir_surface_spec_set_state(spec, initial_state);
+            mir_window_spec_set_state(spec, initial_state);
         });
 
-    apply_changes([&](MirSurfaceSpec* spec)
+    apply_changes([&](MirWindowSpec* spec)
        {
-           mir_surface_spec_set_state(spec, new_state);
+           mir_window_spec_set_state(spec, new_state);
        });
 }
 
 INSTANTIATE_TEST_CASE_P(SurfaceModifications, SurfaceSpecStateCase,
     Values(
-        StatePair{mir_surface_state_hidden, mir_surface_state_restored},
-        StatePair{mir_surface_state_hidden, mir_surface_state_maximized},
-        StatePair{mir_surface_state_hidden, mir_surface_state_vertmaximized},
-        StatePair{mir_surface_state_hidden, mir_surface_state_horizmaximized},
-        StatePair{mir_surface_state_hidden, mir_surface_state_fullscreen},
-        StatePair{mir_surface_state_minimized, mir_surface_state_restored},
-        StatePair{mir_surface_state_minimized, mir_surface_state_maximized},
-        StatePair{mir_surface_state_minimized, mir_surface_state_vertmaximized},
-        StatePair{mir_surface_state_minimized, mir_surface_state_horizmaximized},
-        StatePair{mir_surface_state_minimized, mir_surface_state_fullscreen}
+        StatePair{mir_window_state_hidden, mir_window_state_restored},
+        StatePair{mir_window_state_hidden, mir_window_state_maximized},
+        StatePair{mir_window_state_hidden, mir_window_state_vertmaximized},
+        StatePair{mir_window_state_hidden, mir_window_state_horizmaximized},
+        StatePair{mir_window_state_hidden, mir_window_state_fullscreen},
+        StatePair{mir_window_state_minimized, mir_window_state_restored},
+        StatePair{mir_window_state_minimized, mir_window_state_maximized},
+        StatePair{mir_window_state_minimized, mir_window_state_vertmaximized},
+        StatePair{mir_window_state_minimized, mir_window_state_horizmaximized},
+        StatePair{mir_window_state_minimized, mir_window_state_fullscreen}
     ));
 
 TEST_P(SurfaceStateCase, set_state_affects_surface_visibility)
@@ -569,23 +571,31 @@ TEST_P(SurfaceStateCase, set_state_affects_surface_visibility)
     auto const initial_state = GetParam().from;
     auto const new_state = GetParam().to;
 
-    EXPECT_CALL(surface_observer, hidden_set_to(is_visible(initial_state)));
-    EXPECT_CALL(surface_observer, hidden_set_to(is_visible(new_state)));
+    mt::Signal received_initial_state;
+    mt::Signal received_new_state;
 
-    mir_wait_for(mir_surface_set_state(surface, initial_state));
-    mir_wait_for(mir_surface_set_state(surface, new_state));
+    EXPECT_CALL(surface_observer, hidden_set_to(is_visible(initial_state)))
+        .WillOnce(InvokeWithoutArgs([&]{ received_initial_state.raise(); }));
+    EXPECT_CALL(surface_observer, hidden_set_to(is_visible(new_state)))
+        .WillOnce(InvokeWithoutArgs([&]{ received_new_state.raise(); }));
+
+    mir_window_set_state(window, initial_state);
+    mir_window_set_state(window, new_state);
+
+    received_initial_state.wait_for(400ms);
+    received_new_state.wait_for(400ms);
 }
 
 INSTANTIATE_TEST_CASE_P(SurfaceModifications, SurfaceStateCase,
     Values(
-        StatePair{mir_surface_state_hidden, mir_surface_state_restored},
-        StatePair{mir_surface_state_hidden, mir_surface_state_maximized},
-        StatePair{mir_surface_state_hidden, mir_surface_state_vertmaximized},
-        StatePair{mir_surface_state_hidden, mir_surface_state_horizmaximized},
-        StatePair{mir_surface_state_hidden, mir_surface_state_fullscreen},
-        StatePair{mir_surface_state_minimized, mir_surface_state_restored},
-        StatePair{mir_surface_state_minimized, mir_surface_state_maximized},
-        StatePair{mir_surface_state_minimized, mir_surface_state_vertmaximized},
-        StatePair{mir_surface_state_minimized, mir_surface_state_horizmaximized},
-        StatePair{mir_surface_state_minimized, mir_surface_state_fullscreen}
+        StatePair{mir_window_state_hidden, mir_window_state_restored},
+        StatePair{mir_window_state_hidden, mir_window_state_maximized},
+        StatePair{mir_window_state_hidden, mir_window_state_vertmaximized},
+        StatePair{mir_window_state_hidden, mir_window_state_horizmaximized},
+        StatePair{mir_window_state_hidden, mir_window_state_fullscreen},
+        StatePair{mir_window_state_minimized, mir_window_state_restored},
+        StatePair{mir_window_state_minimized, mir_window_state_maximized},
+        StatePair{mir_window_state_minimized, mir_window_state_vertmaximized},
+        StatePair{mir_window_state_minimized, mir_window_state_horizmaximized},
+        StatePair{mir_window_state_minimized, mir_window_state_fullscreen}
     ));
