@@ -39,11 +39,12 @@ namespace mtf = mir_test_framework;
 namespace mt = mir::test;
 namespace mi = mir::input;
 namespace mis = mir::input::synthesis;
+using namespace testing;
 
 namespace
 {
 std::chrono::seconds const max_wait{4};
-void cookie_capturing_callback(MirSurface* surface, MirEvent const* ev, void* ctx);
+void cookie_capturing_callback(MirWindow* surface, MirEvent const* ev, void* ctx);
 }
 
 struct RaiseSurfaces : mtf::ConnectedClientHeadlessServer
@@ -52,40 +53,44 @@ struct RaiseSurfaces : mtf::ConnectedClientHeadlessServer
     {
         ConnectedClientHeadlessServer::SetUp();
 
-       surface1 = mtf::make_any_surface(connection);
-        mir_surface_set_event_handler(surface1, &cookie_capturing_callback, this);
-        mir_buffer_stream_swap_buffers_sync(
-            mir_surface_get_buffer_stream(surface1));
+        surface1 = mtf::make_any_surface(connection);
+        mir_window_set_event_handler(surface1, &cookie_capturing_callback, this);
 
         surface2 = mtf::make_any_surface(connection);
-        mir_surface_set_event_handler(surface2, &cookie_capturing_callback, this);
-        mir_buffer_stream_swap_buffers_sync(
-            mir_surface_get_buffer_stream(surface2));
 
         // Need fullscreen for the cursor events
-        auto const spec = mir_connection_create_spec_for_changes(connection);
-        mir_surface_spec_set_fullscreen_on_output(spec, 1);
-        mir_surface_apply_spec(surface1, spec);
-        mir_surface_apply_spec(surface2, spec);
-        mir_surface_spec_release(spec);
-        
+        auto const spec = mir_create_window_spec(connection);
+        mir_window_spec_set_fullscreen_on_output(spec, 1);
+        mir_window_apply_spec(surface1, spec);
+
+        mir_window_spec_set_event_handler(spec, &cookie_capturing_callback, this);
+        mir_window_apply_spec(surface2, spec);
+        mir_window_spec_release(spec);
+
+        mir_buffer_stream_swap_buffers_sync(
+            mir_window_get_buffer_stream(surface1));
+        mir_buffer_stream_swap_buffers_sync(
+            mir_window_get_buffer_stream(surface2));
+
         bool surface_fullscreen = mt::spin_wait_for_condition_or_timeout(
             [this]
             {
-                return mir_surface_get_state(surface1) == mir_surface_state_fullscreen &&
-                       mir_surface_get_state(surface2) == mir_surface_state_fullscreen;
+                return mir_window_get_state(surface1) == mir_window_state_fullscreen &&
+                       mir_window_get_state(surface2) == mir_window_state_fullscreen;
             },
-            std::chrono::seconds{max_wait});
+            max_wait);
 
         ready_to_accept_events.wait_for(max_wait);
         if (!ready_to_accept_events.raised())
-            BOOST_THROW_EXCEPTION(std::runtime_error("Timeout waiting for surface to become focused and exposed"));
+            BOOST_THROW_EXCEPTION(std::runtime_error("Timeout waiting for window to become focused and exposed"));
 
-        EXPECT_TRUE(surface_fullscreen);
+        ASSERT_TRUE(surface_fullscreen);
+        ASSERT_TRUE(mt::spin_wait_for_condition_or_timeout([this]
+            { return mir_window_get_focus_state(surface2) == mir_window_focus_state_focused; }, max_wait));
     }
 
-    MirSurface* surface1;
-    MirSurface* surface2;
+    MirWindow* surface1;
+    MirWindow* surface2;
 
     mir::test::Signal ready_to_accept_events;
     std::vector<std::vector<uint8_t>> out_cookies;
@@ -105,27 +110,27 @@ struct RaiseSurfaces : mtf::ConnectedClientHeadlessServer
 namespace
 {
 
-void cookie_capturing_callback(MirSurface* /*surface*/, MirEvent const* ev, void* ctx)
+void cookie_capturing_callback(MirWindow* /*surface*/, MirEvent const* ev, void* ctx)
 {
     auto const event_type = mir_event_get_type(ev);
     auto raise_surfaces = static_cast<RaiseSurfaces*>(ctx);
 
-    if (event_type == mir_event_type_surface)
+    if (event_type == mir_event_type_window)
     {
-        auto event = mir_event_get_surface_event(ev);
-        auto const attrib = mir_surface_event_get_attribute(event);
-        auto const value = mir_surface_event_get_attribute_value(event);
+        auto event = mir_event_get_window_event(ev);
+        auto const attrib = mir_window_event_get_attribute(event);
+        auto const value = mir_window_event_get_attribute_value(event);
 
         std::lock_guard<std::mutex> lk(raise_surfaces->mutex);
 
-        if (attrib == mir_surface_attrib_visibility &&
-            value == mir_surface_visibility_exposed)
+        if (attrib == mir_window_attrib_visibility &&
+            value == mir_window_visibility_exposed)
         {
             raise_surfaces->exposed = true;
         }
 
-        if (attrib == mir_surface_attrib_focus &&
-            value == mir_surface_focused)
+        if (attrib == mir_window_attrib_focus &&
+            value == mir_window_focus_state_focused)
         {
             raise_surfaces->focused = true;
         }
@@ -163,21 +168,21 @@ bool wait_for_n_events(size_t n, RaiseSurfaces const* raise_surfaces)
             std::lock_guard<std::mutex> lk(raise_surfaces->mutex);
             return raise_surfaces->event_count >= n;
         },
-        std::chrono::seconds{max_wait});
+        max_wait);
 
    EXPECT_TRUE(all_events);
    return all_events;
 }
 
-bool attempt_focus(MirSurface* surface, MirCookie const* cookie)
+bool attempt_focus(MirWindow* surface, MirCookie const* cookie)
 {
-    mir_surface_raise(surface, cookie);
+    mir_window_raise(surface, cookie);
     bool surface_becomes_focused = mt::spin_wait_for_condition_or_timeout(
         [&surface]
         {
-            return mir_surface_get_focus(surface) == mir_surface_focused;
+            return mir_window_get_focus_state(surface) == mir_window_focus_state_focused;
         },
-        std::chrono::seconds{max_wait});
+        max_wait);
  
     return surface_becomes_focused;
 }
@@ -193,10 +198,9 @@ TEST_F(RaiseSurfaces, key_event_with_cookie)
     {
         std::lock_guard<std::mutex> lk(mutex);
         ASSERT_FALSE(out_cookies.empty());
-        EXPECT_EQ(mir_surface_get_focus(surface2), mir_surface_focused);
 
         MirCookie const* cookie = mir_cookie_from_buffer(out_cookies.back().data(), out_cookies.back().size());
-        attempt_focus(surface2, cookie);
+        EXPECT_TRUE(attempt_focus(surface2, cookie));
 
         mir_cookie_release(cookie);
     }
@@ -212,16 +216,16 @@ TEST_F(RaiseSurfaces, older_timestamp_does_not_focus)
     {
         std::lock_guard<std::mutex> lk(mutex);
         ASSERT_FALSE(out_cookies.empty());
-        EXPECT_EQ(mir_surface_get_focus(surface2), mir_surface_focused);
+        EXPECT_EQ(mir_window_get_focus_state(surface2), mir_window_focus_state_focused);
 
         MirCookie const* cookie = mir_cookie_from_buffer(out_cookies.front().data(), out_cookies.front().size());
-        mir_surface_raise(surface1, cookie);
+        mir_window_raise(surface1, cookie);
 
         mir_cookie_release(cookie);
 
         // Need to wait for this call to actually go through
         std::this_thread::sleep_for(std::chrono::milliseconds{1000});
-        EXPECT_EQ(mir_surface_get_focus(surface2), mir_surface_focused);
+        EXPECT_EQ(mir_window_get_focus_state(surface2), mir_window_focus_state_focused);
     }
 }
 
@@ -229,7 +233,7 @@ TEST_F(RaiseSurfaces, motion_events_dont_prevent_raise)
 {
     fake_keyboard->emit_event(mis::a_key_down_event().of_scancode(KEY_M));
     fake_keyboard->emit_event(mis::a_key_up_event().of_scancode(KEY_M));
-    int events = 2;
+    size_t events = 2;
     if (wait_for_n_events(events, this))
     {
         fake_pointer->emit_event(mis::a_pointer_event().with_movement(1, 1));
@@ -239,7 +243,7 @@ TEST_F(RaiseSurfaces, motion_events_dont_prevent_raise)
         {
             std::lock_guard<std::mutex> lk(mutex);
             ASSERT_FALSE(out_cookies.empty());
-            EXPECT_EQ(mir_surface_get_focus(surface2), mir_surface_focused);
+            EXPECT_EQ(mir_window_get_focus_state(surface2), mir_window_focus_state_focused);
 
             // We still have 2 cookie, but have gotten more then 2 events
             EXPECT_EQ(out_cookies.size(), events);
