@@ -27,7 +27,7 @@ namespace mir
 {
 namespace geometry
 {
-class Rectangle;
+struct Rectangle;
 }
 namespace test
 {
@@ -37,10 +37,10 @@ struct MockBufferRegistrar : public client::android::BufferRegistrar
 {
     ~MockBufferRegistrar() noexcept {}
     MOCK_CONST_METHOD2(register_buffer,
-        std::shared_ptr<graphics::NativeBuffer>(MirBufferPackage const&,
+        std::shared_ptr<graphics::android::NativeBuffer>(MirBufferPackage const&,
         MirPixelFormat));
     MOCK_METHOD2(secure_for_cpu, std::shared_ptr<char>(
-        std::shared_ptr<graphics::NativeBuffer> const&,
+        std::shared_ptr<graphics::android::NativeBuffer> const&,
         geometry::Rectangle const));
 };
 }
