@@ -44,11 +44,9 @@ struct MockBufferStream : public compositor::BufferStream
         ON_CALL(*this, buffers_ready_for_compositor(::testing::_))
             .WillByDefault(testing::Invoke(this, &MockBufferStream::buffers_ready));
         ON_CALL(*this, with_most_recent_buffer_do(testing::_))
-            .WillByDefault(testing::InvokeArgument<0>(*std::make_shared<StubBuffer>()));
+            .WillByDefault(testing::InvokeArgument<0>(testing::ByRef(*buffer)));
         ON_CALL(*this, acquire_client_buffer(testing::_))
             .WillByDefault(testing::InvokeArgument<0>(nullptr));
-        ON_CALL(*this, swap_buffers(testing::_, testing::_))
-            .WillByDefault(testing::InvokeArgument<1>(nullptr));
         ON_CALL(*this, has_submitted_buffer())
             .WillByDefault(testing::Return(true));
         ON_CALL(*this, pixel_format())
@@ -56,6 +54,7 @@ struct MockBufferStream : public compositor::BufferStream
         ON_CALL(*this, stream_size())
             .WillByDefault(testing::Return(geometry::Size{0,0}));
     }
+    std::shared_ptr<StubBuffer> buffer { std::make_shared<StubBuffer>() };
     MOCK_METHOD1(acquire_client_buffer, void(std::function<void(graphics::Buffer* buffer)>));
     MOCK_METHOD1(release_client_buffer, void(graphics::Buffer*));
     MOCK_METHOD1(lock_compositor_buffer,
@@ -68,13 +67,14 @@ struct MockBufferStream : public compositor::BufferStream
     MOCK_METHOD1(resize, void(geometry::Size const&));
     MOCK_METHOD0(force_client_completion, void());
     MOCK_METHOD1(allow_framedropping, void(bool));
+    MOCK_CONST_METHOD0(framedropping, bool());
 
     MOCK_METHOD0(drop_outstanding_requests, void());
     MOCK_CONST_METHOD1(buffers_ready_for_compositor, int(void const*));
     MOCK_METHOD0(drop_old_buffers, void());
     MOCK_METHOD0(drop_client_requests, void());
 
-    MOCK_METHOD2(swap_buffers, void(graphics::Buffer*, std::function<void(graphics::Buffer*)>));
+    MOCK_METHOD1(submit_buffer, void(std::shared_ptr<graphics::Buffer> const&));
     MOCK_METHOD1(with_most_recent_buffer_do, void(std::function<void(graphics::Buffer&)> const&));
     MOCK_CONST_METHOD0(pixel_format, MirPixelFormat());
     MOCK_CONST_METHOD0(has_submitted_buffer, bool());
