@@ -43,7 +43,7 @@ public:
             .WillByDefault(Return(this));
     }
     MOCK_CONST_METHOD0(view_area, geometry::Rectangle());
-    MOCK_METHOD1(post_renderables_if_optimizable, bool(graphics::RenderableList const&));
+    MOCK_METHOD1(overlay, bool(graphics::RenderableList const&));
     MOCK_CONST_METHOD0(orientation, MirOrientation());
     MOCK_CONST_METHOD0(mirror_mode, MirMirrorMode());
     MOCK_METHOD0(native_display_buffer, graphics::NativeDisplayBuffer*());
