@@ -33,8 +33,11 @@ namespace doubles
 struct MockFence : public graphics::android::Fence
 {
     MOCK_METHOD0(wait, void());
+    MOCK_METHOD1(wait_for, bool(std::chrono::milliseconds));
+    MOCK_METHOD0(reset_fence, void());
     MOCK_METHOD1(merge_with, void(graphics::android::NativeFence&));
     MOCK_CONST_METHOD0(copy_native_handle, graphics::android::NativeFence());
+    MOCK_CONST_METHOD0(native_handle, graphics::android::NativeFence());
 };
 
 }
