@@ -60,17 +60,19 @@ public:
     MOCK_METHOD1(pause_processing_for,void (void const*));
     MOCK_METHOD1(resume_processing_for,void (void const*));
 
+    MOCK_METHOD1(spawn, void(std::function<void()>&));
+
     MOCK_METHOD1(create_alarm, std::unique_ptr<time::Alarm>(std::function<void()> const& callback));
     MOCK_METHOD1(create_alarm, std::unique_ptr<time::Alarm>(LockableCallback* callback));
 
-    std::unique_ptr<time::Alarm> create_alarm(std::unique_ptr<LockableCallback> callback)
+    std::unique_ptr<time::Alarm> create_alarm(std::unique_ptr<LockableCallback> callback) override
     {
         return create_alarm(callback.get());
     }
 
     void register_signal_handler(
         std::initializer_list<int> signals,
-        mir::UniqueModulePtr<std::function<void(int)>> handler)
+        mir::UniqueModulePtr<std::function<void(int)>> handler) override
     {
         register_signal_handler_module_ptr(signals, *handler);
     }
@@ -78,10 +80,16 @@ public:
     void register_fd_handler(
          std::initializer_list<int> fds,
          void const* owner,
-         mir::UniqueModulePtr<std::function<void(int)>> handler)
+         mir::UniqueModulePtr<std::function<void(int)>> handler) override
     {
         register_fd_handler_module_ptr(fds, owner, *handler);
     }
+
+    void spawn(std::function<void()>&& work) override
+    {
+        std::function<void()> work_copy{std::move(work)};
+        spawn(work_copy);
+    }
 };
 
 }
