@@ -39,9 +39,27 @@ public:
     void configure(std::shared_ptr<frontend::Session> const&, std::shared_ptr<graphics::DisplayConfiguration> const&) override
     {
     }
+    void remove_session_configuration(std::shared_ptr<frontend::Session> const&) override
+    {
+    }
     void set_base_configuration(std::shared_ptr<graphics::DisplayConfiguration> const&) override
     {
     }
+    void preview_base_configuration(
+        std::weak_ptr<frontend::Session> const&,
+        std::shared_ptr<graphics::DisplayConfiguration> const&,
+        std::chrono::seconds) override
+    {
+    }
+    void confirm_base_configuration(
+        std::shared_ptr<frontend::Session> const&,
+        std::shared_ptr<graphics::DisplayConfiguration> const&) override
+    {
+    }
+    void cancel_base_configuration_preview(
+        std::shared_ptr<frontend::Session> const&)
+    {
+    }
 };
 }
 }
