@@ -39,13 +39,14 @@ struct NullEventSink : public frontend::EventSink
     void handle_event(MirEvent const&) override {}
     void handle_lifecycle_event(MirLifecycleState) override {}
     void handle_display_config_change(graphics::DisplayConfiguration const&) override {}
+    void handle_error(ClientVisibleError const&) override {}
     void send_ping(int32_t) override {}
     void send_buffer(frontend::BufferStreamId, graphics::Buffer&, graphics::BufferIpcMsgType) override {}
-    void handle_input_device_change(std::vector<std::shared_ptr<mir::input::Device>> const&) override {}
+    void handle_input_config_change(MirInputConfig const&) override {}
     void add_buffer(graphics::Buffer&) override {}
     void remove_buffer(graphics::Buffer&) override {}
     void update_buffer(graphics::Buffer&) override {}
-    void error_buffer(graphics::BufferProperties const&, std::string const&) override {}
+    void error_buffer(geometry::Size, MirPixelFormat, std::string const&) override {}
 };
 
 std::unique_ptr<frontend::EventSink> null_sink_factory(std::shared_ptr<frontend::MessageSender> const&);
