@@ -22,6 +22,7 @@
 #include <mir/compositor/buffer_stream.h>
 #include <mir/test/doubles/stub_buffer.h>
 #include "mir/test/current_thread_name.h"
+#include "mir_test_framework/stub_platform_native_buffer.h"
 
 namespace mir
 {
@@ -35,7 +36,8 @@ class StubBufferStream : public compositor::BufferStream
 public:
     StubBufferStream()
     {
-        stub_compositor_buffer = std::make_shared<StubBuffer>();
+        stub_compositor_buffer = std::make_shared<StubBuffer>(
+            std::make_shared<mir_test_framework::NativeBuffer>(graphics::BufferProperties{}));
     }
 
 
@@ -61,29 +63,30 @@ public:
     void allow_framedropping(bool) override
     {
     }
-
+    bool framedropping() const override
+    {
+        return false;
+    }
     int buffers_ready_for_compositor(void const*) const override { return nready; }
 
     void drop_old_buffers() override {}
-    void swap_buffers(graphics::Buffer* b, std::function<void(graphics::Buffer*)> complete) override
+    void submit_buffer(std::shared_ptr<graphics::Buffer> const& b) override
     {
         if (b) ++nready;
-        complete(&stub_client_buffer);
     }
-    void with_most_recent_buffer_do(std::function<void(graphics::Buffer&)> const& fn)
+    void with_most_recent_buffer_do(std::function<void(graphics::Buffer&)> const& fn) override
     {
         thread_name = current_thread_name();
         fn(*stub_compositor_buffer);
     }
-    MirPixelFormat pixel_format() const { return mir_pixel_format_abgr_8888; }
-    void add_observer(std::shared_ptr<scene::SurfaceObserver> const&) {}
-    void remove_observer(std::weak_ptr<scene::SurfaceObserver> const&) {}
-    bool has_submitted_buffer() const { return true; }
-    void associate_buffer(graphics::BufferID) {}
-    void disassociate_buffer(graphics::BufferID) {}
-    void set_scale(float) {}
+    MirPixelFormat pixel_format() const override { return mir_pixel_format_abgr_8888; }
+    void add_observer(std::shared_ptr<scene::SurfaceObserver> const&) override {}
+    void remove_observer(std::weak_ptr<scene::SurfaceObserver> const&) override {}
+    bool has_submitted_buffer() const override { return true; }
+    void associate_buffer(graphics::BufferID) override {}
+    void disassociate_buffer(graphics::BufferID) override {}
+    void set_scale(float) override {}
 
-    StubBuffer stub_client_buffer;
     std::shared_ptr<graphics::Buffer> stub_compositor_buffer;
     int nready = 0;
     std::string thread_name;
