@@ -46,7 +46,7 @@ struct StubDisplayServer : public mir::frontend::detail::DisplayServer
         mir::protobuf::SurfaceModifications const* /*request*/,
         mir::protobuf::Void* /*response*/,
         google::protobuf::Closure* /*done*/) {}
-    void next_buffer(
+    void swap_buffers(
         mir::protobuf::SurfaceId const* /*request*/,
         mir::protobuf::Buffer* /*response*/,
         google::protobuf::Closure* /*done*/) {}
@@ -66,6 +66,10 @@ struct StubDisplayServer : public mir::frontend::detail::DisplayServer
         mir::protobuf::DisplayConfiguration const* /*request*/,
         mir::protobuf::DisplayConfiguration* /*response*/,
         google::protobuf::Closure* /*done*/) {}
+    void remove_session_configuration(
+        mir::protobuf::Void const* /*request*/,
+        mir::protobuf::Void* /*response*/,
+        google::protobuf::Closure* /*done*/) {}
     void set_base_display_configuration(
         mir::protobuf::DisplayConfiguration const* /*request*/,
         mir::protobuf::Void* /*response*/,
@@ -78,6 +82,10 @@ struct StubDisplayServer : public mir::frontend::detail::DisplayServer
         mir::protobuf::DisplayConfiguration const* /*request*/,
         mir::protobuf::Void* /*response*/,
         google::protobuf::Closure* /*done*/) {}
+    void cancel_base_display_configuration_preview(
+        mir::protobuf::Void const* /*request*/,
+        mir::protobuf::Void* /*response*/,
+        google::protobuf::Closure* /*done*/) {}
     void create_screencast(
         mir::protobuf::ScreencastParameters const* /*request*/,
         mir::protobuf::Screencast* /*response*/,
@@ -86,6 +94,10 @@ struct StubDisplayServer : public mir::frontend::detail::DisplayServer
         mir::protobuf::ScreencastId const* /*request*/,
         mir::protobuf::Buffer* /*response*/,
         google::protobuf::Closure* /*done*/) {}
+    void screencast_to_buffer(
+        mir::protobuf::ScreencastRequest const*,
+        mir::protobuf::Void*,
+        google::protobuf::Closure*) {}
     void release_screencast(
         mir::protobuf::ScreencastId const* /*request*/,
         mir::protobuf::Void* /*response*/,
@@ -108,10 +120,10 @@ struct StubDisplayServer : public mir::frontend::detail::DisplayServer
         google::protobuf::Closure* /*done*/) {}
     void start_prompt_session(
         mir::protobuf::PromptSessionParameters const* /*request*/,
-        mir::protobuf::Void* /*response*/,
+        mir::protobuf::PromptSession* /*response*/,
         google::protobuf::Closure* /*done*/) {}
     void stop_prompt_session(
-        mir::protobuf::Void const* /*request*/,
+        mir::protobuf::PromptSession const* /*request*/,
         mir::protobuf::Void* /*response*/,
         google::protobuf::Closure* /*done*/) {}
     void exchange_buffer(
