@@ -67,8 +67,8 @@ public:
     int buffers_ready_for_compositor(void const*) const override { return 0; }
 
     float alpha() const override { return 0.0f;}
-    MirSurfaceType type() const override { return mir_surface_type_normal; }
-    MirSurfaceState state() const override { return mir_surface_state_unknown; }
+    MirWindowType type() const override { return mir_window_type_normal; }
+    MirWindowState state() const override { return mir_window_state_unknown; }
 
     void hide() override {}
     void show() override {}
@@ -78,7 +78,7 @@ public:
     void resize(geometry::Size const&) override {}
     void set_transformation(glm::mat4 const&) override {}
     void set_alpha(float) override {}
-    void set_orientation(MirOrientation) {}
+    void set_orientation(MirOrientation) override {}
 
     void add_observer(std::shared_ptr<scene::SurfaceObserver> const&) override {}
     void remove_observer(std::weak_ptr<scene::SurfaceObserver> const&) override {}
@@ -86,15 +86,15 @@ public:
     void set_reception_mode(input::InputReceptionMode mode) override { input_mode = mode; }
     void consume(MirEvent const*) override {}
 
-    void set_cursor_image(std::shared_ptr<graphics::CursorImage> const& /* image */) {}
-    std::shared_ptr<graphics::CursorImage> cursor_image() const { return {}; }
+    void set_cursor_image(std::shared_ptr<graphics::CursorImage> const& /* image */) override {}
+    std::shared_ptr<graphics::CursorImage> cursor_image() const override { return {}; }
 
     void request_client_surface_close() override {}
 
     bool supports_input() const override { return true;}
     int client_input_fd() const override { return fd;}
-    int configure(MirSurfaceAttrib, int) override { return 0; }
-    int query(MirSurfaceAttrib) const override { return 0; }
+    int configure(MirWindowAttrib, int) override { return 0; }
+    int query(MirWindowAttrib) const override { return 0; }
     void with_most_recent_buffer_do(std::function<void(graphics::Buffer&)> const&) {}
 
     std::shared_ptr<mir::scene::Surface> parent() const override { return nullptr; }
@@ -102,12 +102,13 @@ public:
     void set_keymap(MirInputDeviceId, std::string const&, std::string const&, std::string const&, std::string const&) override
     {}
 
-    void set_cursor_stream(std::shared_ptr<frontend::BufferStream> const&, geometry::Displacement const&) {}
-    void rename(std::string const&) {}
+    void set_cursor_stream(std::shared_ptr<frontend::BufferStream> const&, geometry::Displacement const&) override {}
+    void rename(std::string const&) override {}
     std::shared_ptr<frontend::BufferStream> primary_buffer_stream() const override { return nullptr; }
 
     void set_confine_pointer_state(MirPointerConfinementState /*state*/) override {}
     MirPointerConfinementState confine_pointer_state() const override { return {}; }
+    void placed_relative(geometry::Rectangle const& /*placement*/) override {}
 };
 
 }
