@@ -51,16 +51,11 @@ struct StubServerTool : doubles::StubDisplayServer
         done->Run();
     }
 
-    virtual void exchange_buffer(
+    virtual void submit_buffer(
         mir::protobuf::BufferRequest const* /*request*/,
-        mir::protobuf::Buffer* response,
+        mir::protobuf::Void*,
         google::protobuf::Closure* done) override
     {
-        response->set_buffer_id(22);
-
-        std::lock_guard<std::mutex> lock(guard);
-        //FIXME: huh? What's the condition here?
-        signal.notify_one();
         done->Run();
     }
 
