@@ -49,6 +49,8 @@ void extension_glEGLImageTargetTexture2DOES(GLenum target, GLeglImageOES image);
 EGLSyncKHR extension_eglCreateSyncKHR(EGLDisplay dpy, EGLenum type, const EGLint *attrib_list);
 EGLBoolean extension_eglDestroySyncKHR(EGLDisplay dpy, EGLSyncKHR sync);
 EGLint extension_eglClientWaitSyncKHR(EGLDisplay dpy, EGLSyncKHR sync, EGLint flags, EGLTimeKHR timeout);
+EGLBoolean extension_eglGetSyncValuesCHROMIUM(EGLDisplay dpy,
+    EGLSurface surface, int64_t *ust, int64_t *msc, int64_t *sbc);
 
 /* EGL{Surface,Display,Config,Context} are all opaque types, so we can put whatever
    we want in them for testing */
@@ -144,6 +146,10 @@ mtd::MockEGL::MockEGL()
         .WillByDefault(Return(reinterpret_cast<func_ptr_t>(extension_eglDestroySyncKHR)));
     ON_CALL(*this, eglGetProcAddress(StrEq("eglClientWaitSyncKHR")))
         .WillByDefault(Return(reinterpret_cast<func_ptr_t>(extension_eglClientWaitSyncKHR)));
+    ON_CALL(*this, eglGetProcAddress(StrEq("eglGetSyncValuesCHROMIUM")))
+        .WillByDefault(Return(
+            reinterpret_cast<func_ptr_t>(extension_eglGetSyncValuesCHROMIUM)
+            ));
 }
 
 void mtd::MockEGL::provide_egl_extensions()
@@ -171,7 +177,7 @@ void mtd::MockEGL::provide_stub_platform_buffer_swapping()
             [&](EGLDisplay,EGLSurface surface) -> EGLBoolean
             {
                 auto mir_surf = reinterpret_cast<mir::client::EGLNativeSurface*>(surface);
-                mir_surf->request_and_wait_for_next_buffer();
+                mir_surf->swap_buffers_sync();
                 return true;
             }));
 }
@@ -416,3 +422,11 @@ EGLint extension_eglClientWaitSyncKHR(EGLDisplay dpy, EGLSyncKHR sync, EGLint fl
     CHECK_GLOBAL_MOCK(EGLint);
     return global_mock_egl->eglClientWaitSyncKHR(dpy, sync, flags, timeout);
 }
+
+EGLBoolean extension_eglGetSyncValuesCHROMIUM(EGLDisplay dpy,
+              EGLSurface surface, int64_t *ust, int64_t *msc, int64_t *sbc)
+{
+    CHECK_GLOBAL_MOCK(EGLBoolean);
+    return global_mock_egl->eglGetSyncValuesCHROMIUM(dpy, surface,
+                                                     ust, msc, sbc);
+}
