@@ -18,6 +18,7 @@
 
 #include "mir/test/doubles/stub_session.h"
 #include "mir/test/doubles/stub_buffer.h"
+#include "mir_test_framework/stub_platform_native_buffer.h"
 
 namespace mtd = mir::test::doubles;
 
@@ -64,6 +65,11 @@ void mtd::StubSession::send_display_config(
 {
 }
 
+void mtd::StubSession::send_error(
+    mir::ClientVisibleError const& /*error*/)
+{
+}
+
 void mtd::StubSession::hide()
 {
 }
@@ -137,7 +143,7 @@ void mtd::StubSession::destroy_surface(std::weak_ptr<scene::Surface> const& /*su
 {
 }
 
-void mtd::StubSession::send_input_device_change(std::vector<std::shared_ptr<mir::input::Device>> const& /*devices*/)
+void mtd::StubSession::send_input_config(MirInputConfig const& /*config*/)
 {
 }
 
@@ -146,13 +152,24 @@ mir::graphics::BufferID mtd::StubSession::create_buffer(mir::graphics::BufferPro
     return mir::graphics::BufferID(3);
 }
 
+mir::graphics::BufferID mtd::StubSession::create_buffer(mir::geometry::Size, uint32_t, uint32_t)
+{
+    return mir::graphics::BufferID(3);
+}
+
+mir::graphics::BufferID mtd::StubSession::create_buffer(mir::geometry::Size, MirPixelFormat)
+{
+    return mir::graphics::BufferID(3);
+}
+
 void mtd::StubSession::destroy_buffer(mir::graphics::BufferID)
 {
 }
 
 std::shared_ptr<mir::graphics::Buffer> mtd::StubSession::get_buffer(graphics::BufferID)
 {
-    return std::make_shared<mtd::StubBuffer>();
+    return std::make_shared<mtd::StubBuffer>(
+        std::make_shared<mir_test_framework::NativeBuffer>(graphics::BufferProperties{}));
 }
 
 namespace
