@@ -17,7 +17,6 @@
  */
 
 #include "mir_toolkit/mir_client_library.h"
-#include "src/client/client_buffer_depository.h"
 #include "src/client/buffer_vault.h"
 #include "src/client/buffer_factory.h"
 #include "src/client/connection_surface_map.h"
@@ -50,7 +49,7 @@ struct MockClientBufferFactory : public mcl::ClientBufferFactory
 {
     MockClientBufferFactory()
     {
-        ON_CALL(*this, create_buffer(_,_,_))
+        ON_CALL(*this, create_buffer(_,An<geom::Size>(),_))
             .WillByDefault(Invoke([](
                     std::shared_ptr<MirBufferPackage> const&, geom::Size size, MirPixelFormat)
                 {
@@ -62,6 +61,8 @@ struct MockClientBufferFactory : public mcl::ClientBufferFactory
     }
     MOCK_METHOD3(create_buffer, std::shared_ptr<mcl::ClientBuffer>(
         std::shared_ptr<MirBufferPackage> const&, geom::Size, MirPixelFormat));
+    MOCK_METHOD3(create_buffer, std::shared_ptr<mcl::ClientBuffer>(
+        std::shared_ptr<MirBufferPackage> const&, uint32_t, uint32_t));
 };
 
 struct MockServerRequests : mcl::ServerBufferRequests
@@ -173,13 +174,15 @@ TEST_F(BufferVault, withdrawing_and_never_filling_up_will_timeout)
     auto buffer_future = vault->withdraw();
     ASSERT_TRUE(buffer_future.valid());
     EXPECT_THAT(buffer_future.wait_for(20ms), Eq(std::future_status::timeout));
+
+    vault.reset();
 }
 
 TEST_F(StartedBufferVault, withdrawing_gives_a_valid_future)
 {
     auto buffer_future = vault.withdraw();
     ASSERT_TRUE(buffer_future.valid());
-    EXPECT_THAT(buffer_future.get(), Ne(nullptr));;
+    EXPECT_THAT(buffer_future.get(), Ne(nullptr));
 }
 
 TEST_F(StartedBufferVault, can_deposit_buffer)
@@ -271,7 +274,7 @@ TEST_F(BufferVault, ages_buffer_on_deposit)
     ON_CALL(*mock_buffer, size())
         .WillByDefault(Return(size));
     EXPECT_CALL(*mock_buffer, increment_age());
-    ON_CALL(mock_platform_factory, create_buffer(_,_,_))
+    ON_CALL(mock_platform_factory, create_buffer(_,An<uint32_t>(),_))
         .WillByDefault(Return(mock_buffer));
     auto b1 = std::make_shared<mcl::Buffer>(
         ignore, nullptr, package.buffer_id(), mock_buffer, nullptr, mir_buffer_usage_software);
@@ -288,7 +291,7 @@ TEST_F(BufferVault, marks_as_submitted_on_transfer)
     ON_CALL(*mock_buffer, size())
         .WillByDefault(Return(size));
     EXPECT_CALL(*mock_buffer, mark_as_submitted());
-    ON_CALL(mock_platform_factory, create_buffer(_,_,_))
+    ON_CALL(mock_platform_factory, create_buffer(_,An<uint32_t>(),_))
         .WillByDefault(Return(mock_buffer));
     auto b1 = std::make_shared<mcl::Buffer>(
         ignore, nullptr, package.buffer_id(), mock_buffer, nullptr, mir_buffer_usage_software);
@@ -448,7 +451,7 @@ TEST_F(StartedBufferVault, can_increase_allocation_count)
 {
     EXPECT_CALL(mock_requests, allocate_buffer(_,_,_))
         .Times(1);
-    vault.increase_buffer_count();
+    vault.set_interval(0);
 
     Mock::VerifyAndClearExpectations(&mock_requests);
     //no buffers
@@ -458,7 +461,8 @@ TEST_F(StartedBufferVault, cannot_decrease_allocation_count_below_initial)
 {
     EXPECT_CALL(mock_requests, free_buffer(_))
         .Times(0);
-    vault.decrease_buffer_count();
+    vault.set_interval(1);
+    vault.set_interval(1);
     Mock::VerifyAndClearExpectations(&mock_requests);
 }
 
@@ -468,8 +472,8 @@ TEST_F(StartedBufferVault, can_decrease_allocation_count)
         .Times(1);
     EXPECT_CALL(mock_requests, free_buffer(_))
         .Times(1);
-    vault.increase_buffer_count();
-    vault.decrease_buffer_count();
+    vault.set_interval(0);
+    vault.set_interval(1);
     Mock::VerifyAndClearExpectations(&mock_requests);
 }
 
@@ -492,7 +496,7 @@ TEST_F(StartedBufferVault, delayed_decrease_allocation_count)
     EXPECT_CALL(mock_requests, free_buffer(package.buffer_id()))
         .Times(1);
 
-    vault.increase_buffer_count();
+    vault.set_interval(0);
     vault.wire_transfer_inbound(requested_buffer.buffer_id());
     auto b = vault.withdraw().get();
     vault.deposit(b);
@@ -510,7 +514,7 @@ TEST_F(StartedBufferVault, delayed_decrease_allocation_count)
     vault.deposit(b);
     vault.wire_transfer_outbound(b, []{});
 
-    vault.decrease_buffer_count();
+    vault.set_interval(1);
     vault.wire_transfer_inbound(package.buffer_id());
     vault.wire_transfer_inbound(package2.buffer_id());
     Mock::VerifyAndClearExpectations(&mock_requests);
@@ -635,9 +639,13 @@ TEST_F(StartedBufferVault, can_increase_count_after_resize)
     EXPECT_CALL(mock_requests, allocate_buffer(_,_,_))
         .Times(num_allocations);
 
-    vault.increase_buffer_count();
+    vault.set_interval(0);
     vault.set_size(new_size);
 
     for(auto i = 0u; i < num_allocations + 1; i++)
-        vault.withdraw();
+    {
+        auto future = vault.withdraw();
+        // Do the minimum possible to actually satisfy this future...
+        vault.wire_transfer_inbound(package4.buffer_id());
+    }
 }
