@@ -18,7 +18,7 @@
 
 // TODO: There's a lot to suggest (usage of real connection most prevalent)
 // that this is perhaps a set of integration tests. But moving it there conflicts
-// with test_mirsurface.cpp. Client MirSurface testing probably needs to be reviewed
+// with test_mirsurface.cpp. Client MirWindow testing probably needs to be reviewed
 
 #include "mir_protobuf.pb.h"
 #include "mir_toolkit/mir_client_library.h"
@@ -52,7 +52,7 @@
 #include <boost/throw_exception.hpp>
 #include "mir/test/doubles/stub_client_buffer_factory.h"
 #include "mir/test/doubles/stub_buffer_stream.h"
-#include "mir/test/doubles/mock_client_buffer_stream.h"
+#include "mir/test/doubles/mock_mir_buffer_stream.h"
 
 #include "mir_test_framework/stub_client_platform_factory.h"
 
@@ -108,24 +108,15 @@ struct MockServerPackageGenerator : public mt::StubServerTool
     void release_surface(
         const mir::protobuf::SurfaceId*,
         mir::protobuf::Void*,
-        google::protobuf::Closure* done)
-    {
-        done->Run();
-    }
-
-    void exchange_buffer(
-        mir::protobuf::BufferRequest const* /*request*/,
-        mir::protobuf::Buffer* response,
         google::protobuf::Closure* done) override
     {
-        create_buffer_response(response);
         done->Run();
     }
 
     void modify_surface(
         const mir::protobuf::SurfaceModifications*, 
         mir::protobuf::Void*,
-        google::protobuf::Closure* done)
+        google::protobuf::Closure* done) override
     {
         done->Run();
     }
@@ -219,13 +210,13 @@ struct MockServerPackageGenerator : public mt::StubServerTool
 };
 
 std::map<int, int> MockServerPackageGenerator::sent_surface_attributes = {
-    { mir_surface_attrib_type, mir_surface_type_normal },
-    { mir_surface_attrib_state, mir_surface_state_restored },
-    { mir_surface_attrib_swapinterval, 1 },
-    { mir_surface_attrib_focus, mir_surface_focused },
-    { mir_surface_attrib_dpi, 19 },
-    { mir_surface_attrib_visibility, mir_surface_visibility_exposed },
-    { mir_surface_attrib_preferred_orientation, mir_orientation_mode_any }
+    { mir_window_attrib_type, mir_window_type_normal },
+    { mir_window_attrib_state, mir_window_state_restored },
+    { mir_window_attrib_swapinterval, 1 },
+    { mir_window_attrib_focus, mir_window_focus_state_focused },
+    { mir_window_attrib_dpi, 19 },
+    { mir_window_attrib_visibility, mir_window_visibility_exposed },
+    { mir_window_attrib_preferred_orientation, mir_orientation_mode_any }
 };
 
 struct StubClientInputPlatform : public mircv::InputPlatform
@@ -270,13 +261,19 @@ struct FakeRpcChannel : public mir::client::rpc::MirBasicRpcChannel
     {
         delete closure;
     }
+    void discard_future_calls() override
+    {
+    }
+    void wait_for_outstanding_calls() override
+    {
+    }
 };
 
 void null_connected_callback(MirConnection* /*connection*/, void * /*client_context*/)
 {
 }
 
-void null_event_callback(MirSurface*, MirEvent const*, void*)
+void null_event_callback(MirWindow*, MirEvent const*, void*)
 {
 }
 
@@ -323,9 +320,9 @@ struct MirClientSurfaceTest : public testing::Test
         client_comm_channel = std::make_shared<mclr::DisplayServer>(conf.the_rpc_channel());
     }
 
-    std::shared_ptr<MirSurface> create_surface_with(mclr::DisplayServer& server_stub)
+    std::shared_ptr<MirWindow> create_surface_with(mclr::DisplayServer& server_stub)
     {
-        return std::make_shared<MirSurface>(
+        return std::make_shared<MirWindow>(
             connection.get(),
             server_stub,
             nullptr,
@@ -336,11 +333,11 @@ struct MirClientSurfaceTest : public testing::Test
             wh);
     }
 
-    std::shared_ptr<MirSurface> create_surface_with(
+    std::shared_ptr<MirWindow> create_surface_with(
         mclr::DisplayServer& server_stub,
-        std::shared_ptr<mcl::ClientBufferStream> const& buffer_stream)
+        std::shared_ptr<MirBufferStream> const& buffer_stream)
     {
-        return std::make_shared<MirSurface>(
+        return std::make_shared<MirWindow>(
             connection.get(),
             server_stub,
             nullptr,
@@ -351,16 +348,16 @@ struct MirClientSurfaceTest : public testing::Test
             wh);
     }
 
-    std::shared_ptr<MirSurface> create_and_wait_for_surface_with(
+    std::shared_ptr<MirWindow> create_and_wait_for_surface_with(
         mclr::DisplayServer& server_stub)
     {
         auto surface = create_surface_with(server_stub);
         return surface;
     }
 
-    std::shared_ptr<MirSurface> create_and_wait_for_surface_with(
+    std::shared_ptr<MirWindow> create_and_wait_for_surface_with(
         mclr::DisplayServer& server_stub,
-        std::shared_ptr<mcl::ClientBufferStream> const& buffer_stream)
+        std::shared_ptr<MirBufferStream> const& buffer_stream)
     {
         auto surface = create_surface_with(server_stub, buffer_stream);
         return surface;
@@ -368,8 +365,8 @@ struct MirClientSurfaceTest : public testing::Test
 
     std::shared_ptr<MirConnection> connection;
 
-    MirSurfaceSpec const spec{nullptr, 33, 45, mir_pixel_format_abgr_8888};
-    std::shared_ptr<mtd::MockClientBufferStream> stub_buffer_stream{std::make_shared<mtd::MockClientBufferStream>()};
+    MirWindowSpec const spec{nullptr, 33, 45, mir_pixel_format_abgr_8888};
+    std::shared_ptr<mtd::MockMirBufferStream> stub_buffer_stream{std::make_shared<mtd::MockMirBufferStream>()};
     std::shared_ptr<StubClientInputPlatform> const input_platform =
         std::make_shared<StubClientInputPlatform>();
     std::shared_ptr<MockServerPackageGenerator> const mock_server_tool =
@@ -392,9 +389,9 @@ TEST_F(MirClientSurfaceTest, attributes_set_on_surface_creation)
 
     auto surface = create_and_wait_for_surface_with(*client_comm_channel);
     
-    for (int i = 0; i < mir_surface_attribs; i++)
+    for (int i = 0; i < mir_window_attribs; i++)
     {
-        EXPECT_EQ(MockServerPackageGenerator::sent_surface_attributes[i], surface->attrib(static_cast<MirSurfaceAttrib>(i)));
+        EXPECT_EQ(MockServerPackageGenerator::sent_surface_attributes[i], surface->attrib(static_cast<MirWindowAttrib>(i)));
     }
 }
 
@@ -410,7 +407,7 @@ TEST_F(MirClientSurfaceTest, creates_input_thread_with_input_dispatcher_when_del
     EXPECT_CALL(*mock_input_platform, create_input_receiver(_, _, _)).Times(1)
         .WillOnce(Return(mock_input_dispatcher));
 
-    MirSurface surface{connection.get(), *client_comm_channel, nullptr,
+    MirWindow surface{connection.get(), *client_comm_channel, nullptr,
         stub_buffer_stream, mock_input_platform, spec, surface_proto, wh};
     surface.set_event_handler(null_event_callback, nullptr);
 
@@ -419,6 +416,123 @@ TEST_F(MirClientSurfaceTest, creates_input_thread_with_input_dispatcher_when_del
     EXPECT_TRUE(dispatched->wait_for(std::chrono::seconds{5}));
 }
 
+TEST_F(MirClientSurfaceTest, adopts_the_default_stream)
+{
+    using namespace ::testing;
+
+    auto mock_input_platform = std::make_shared<MockClientInputPlatform>();
+    auto mock_stream = std::make_shared<mtd::MockMirBufferStream>(); 
+
+    MirWindow* adopted_by = nullptr;
+    MirWindow* unadopted_by = nullptr;
+    EXPECT_CALL(*mock_stream, adopted_by(_))
+        .WillOnce(SaveArg<0>(&adopted_by));
+    EXPECT_CALL(*mock_stream, unadopted_by(_))
+        .WillOnce(SaveArg<0>(&unadopted_by));
+
+    {
+        MirWindow win{connection.get(), *client_comm_channel, nullptr,
+            mock_stream, mock_input_platform, spec, surface_proto, wh};
+        EXPECT_EQ(&win,    adopted_by);
+        EXPECT_EQ(nullptr, unadopted_by);
+    }
+
+    EXPECT_NE(nullptr,    unadopted_by);
+    EXPECT_EQ(adopted_by, unadopted_by);
+}
+
+TEST_F(MirClientSurfaceTest, adopts_custom_streams_set_before_construction)
+{   // Regression test for nested server bugs LP: #1661128, LP: #1661072
+    using namespace ::testing;
+
+    mir::frontend::BufferStreamId const mock_stream_id(777888);
+    auto mock_input_platform = std::make_shared<MockClientInputPlatform>();
+    auto mock_stream = std::make_shared<mtd::MockMirBufferStream>(); 
+
+    MirWindow* adopted_by = nullptr;
+    MirWindow* unadopted_by = nullptr;
+    ON_CALL(*mock_stream, rpc_id())
+        .WillByDefault(Return(mock_stream_id));
+    EXPECT_CALL(*mock_stream, adopted_by(_))
+        .WillOnce(SaveArg<0>(&adopted_by));
+    EXPECT_CALL(*mock_stream, unadopted_by(_))
+        .WillOnce(SaveArg<0>(&unadopted_by));
+
+    surface_map->insert(mock_stream_id, mock_stream);
+
+    {
+        MirWindowSpec spec;
+        std::vector<ContentInfo> replacements
+        {
+            {geom::Displacement{0,0}, mock_stream_id.as_value(), geom::Size{1,1}}
+        };
+        spec.streams = replacements;
+        MirWindow win{connection.get(), *client_comm_channel, nullptr,
+            nullptr, mock_input_platform, spec, surface_proto, wh};
+        ASSERT_EQ(&win,    adopted_by);
+        ASSERT_EQ(nullptr, unadopted_by);
+    }
+
+    EXPECT_NE(nullptr,    unadopted_by);
+    EXPECT_EQ(adopted_by, unadopted_by);
+}
+
+TEST_F(MirClientSurfaceTest, adopts_custom_streams_set_after_construction)
+{
+    using namespace testing;
+
+    auto mock_input_platform = std::make_shared<MockClientInputPlatform>();
+
+    mir::frontend::BufferStreamId const mock_old_stream_id(11);
+    auto mock_old_stream = std::make_shared<mtd::MockMirBufferStream>(); 
+    MirWindow* old_adopted_by = nullptr;
+    MirWindow* old_unadopted_by = nullptr;
+    EXPECT_CALL(*mock_old_stream, adopted_by(_))
+        .WillOnce(SaveArg<0>(&old_adopted_by));
+    EXPECT_CALL(*mock_old_stream, unadopted_by(_))
+        .WillOnce(SaveArg<0>(&old_unadopted_by));
+    ON_CALL(*mock_old_stream, rpc_id())
+        .WillByDefault(Return(mock_old_stream_id));
+
+    mir::frontend::BufferStreamId const mock_new_stream_id(22);
+    auto mock_new_stream = std::make_shared<mtd::MockMirBufferStream>(); 
+    MirWindow* new_adopted_by = nullptr;
+    MirWindow* new_unadopted_by = nullptr;
+    EXPECT_CALL(*mock_new_stream, adopted_by(_))
+        .WillOnce(SaveArg<0>(&new_adopted_by));
+    EXPECT_CALL(*mock_new_stream, unadopted_by(_))
+        .WillOnce(SaveArg<0>(&new_unadopted_by));
+    ON_CALL(*mock_new_stream, rpc_id())
+        .WillByDefault(Return(mock_new_stream_id));
+
+    surface_map->insert(mock_old_stream_id, mock_old_stream);
+    surface_map->insert(mock_new_stream_id, mock_new_stream);
+    {
+        MirWindow win{connection.get(), *client_comm_channel, nullptr,
+            mock_old_stream, mock_input_platform, spec, surface_proto, wh};
+    
+        EXPECT_EQ(&win,    old_adopted_by);
+        EXPECT_EQ(nullptr, old_unadopted_by);
+        EXPECT_EQ(nullptr, new_adopted_by);
+        EXPECT_EQ(nullptr, new_unadopted_by);
+
+        MirWindowSpec spec;
+        std::vector<ContentInfo> replacements
+        {
+            {geom::Displacement{0,0}, mock_new_stream_id.as_value(), geom::Size{1,1}}
+        };
+        spec.streams = replacements;
+        win.modify(spec)->wait_for_all();
+
+        EXPECT_EQ(&win,    old_unadopted_by);
+        EXPECT_EQ(&win,    new_adopted_by);
+        EXPECT_EQ(nullptr, new_unadopted_by);
+    }
+    EXPECT_EQ(new_adopted_by, new_unadopted_by);
+    surface_map->erase(mock_old_stream_id);
+    surface_map->erase(mock_new_stream_id);
+}
+
 TEST_F(MirClientSurfaceTest, replacing_delegate_with_nullptr_prevents_further_dispatch)
 {
     using namespace ::testing;
@@ -431,7 +545,7 @@ TEST_F(MirClientSurfaceTest, replacing_delegate_with_nullptr_prevents_further_di
     EXPECT_CALL(*mock_input_platform, create_input_receiver(_, _, _)).Times(1)
         .WillOnce(Return(mock_input_dispatcher));
 
-    MirSurface surface{connection.get(), *client_comm_channel, nullptr,
+    MirWindow surface{connection.get(), *client_comm_channel, nullptr,
         stub_buffer_stream, mock_input_platform, spec, surface_proto, wh};
     surface.set_event_handler(null_event_callback, nullptr);
 
@@ -452,7 +566,7 @@ TEST_F(MirClientSurfaceTest, does_not_create_input_dispatcher_when_no_delegate_s
 
     EXPECT_CALL(*mock_input_platform, create_input_receiver(_, _, _)).Times(0);
 
-    MirSurface surface{connection.get(), *client_comm_channel, nullptr,
+    MirWindow surface{connection.get(), *client_comm_channel, nullptr,
         stub_buffer_stream, mock_input_platform, spec, surface_proto, wh};
 }
 
@@ -460,7 +574,7 @@ TEST_F(MirClientSurfaceTest, valid_surface_is_valid)
 {
     auto const surface = create_and_wait_for_surface_with(*client_comm_channel);
 
-    EXPECT_TRUE(MirSurface::is_valid(surface.get()));
+    EXPECT_TRUE(MirWindow::is_valid(surface.get()));
 }
 
 TEST_F(MirClientSurfaceTest, configure_cursor_wait_handle_really_blocks)
@@ -472,7 +586,10 @@ TEST_F(MirClientSurfaceTest, configure_cursor_wait_handle_really_blocks)
 
     auto const surface = create_surface_with(unresponsive_server);
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
     auto cursor_config = mir_cursor_configuration_from_name(mir_default_cursor_name);
+#pragma GCC diagnostic pop
     auto cursor_wait_handle = surface->configure_cursor(cursor_config);
 
     auto expected_end = std::chrono::steady_clock::now() + pause_time;
@@ -492,7 +609,7 @@ TEST_F(MirClientSurfaceTest, configure_wait_handle_really_blocks)
 
     auto const surface = create_surface_with(unresponsive_server);
 
-    auto configure_wait_handle = surface->configure(mir_surface_attrib_dpi, 100);
+    auto configure_wait_handle = surface->configure(mir_window_attrib_dpi, 100);
 
     auto expected_end = std::chrono::steady_clock::now() + pause_time;
     configure_wait_handle->wait_for_pending(pause_time);
@@ -503,50 +620,54 @@ TEST_F(MirClientSurfaceTest, configure_wait_handle_really_blocks)
 TEST_F(MirClientSurfaceTest, resizes_streams_and_calls_callback_if_no_customized_streams)
 {
     using namespace testing;
-    auto mock_stream = std::make_shared<mtd::MockClientBufferStream>(); 
+    auto mock_stream = std::make_shared<mtd::MockMirBufferStream>(); 
+    mir::frontend::BufferStreamId const mock_stream_id(2);
     auto mock_input_platform = std::make_shared<NiceMock<MockClientInputPlatform>>();
     ON_CALL(*mock_input_platform, create_input_receiver(_,_,_))
         .WillByDefault(Return(std::make_shared<mt::TestDispatchable>([]{})));
-    ON_CALL(*mock_stream, rpc_id()).WillByDefault(Return(mir::frontend::BufferStreamId(2)));
+    ON_CALL(*mock_stream, rpc_id()).WillByDefault(Return(mock_stream_id));
 
     geom::Size size(120, 124);
     EXPECT_CALL(*mock_stream, set_size(size));
     auto ev = mir::events::make_event(mir::frontend::SurfaceId(2), size);
 
-    MirSurface surface{connection.get(), *client_comm_channel, nullptr,
+    MirWindow surface{connection.get(), *client_comm_channel, nullptr,
         mock_stream, mock_input_platform, spec, surface_proto, wh};
+    surface_map->insert(mock_stream_id, mock_stream);
     surface.handle_event(*ev);
-    surface_map->erase(mir::frontend::BufferStreamId(2));
+    surface_map->erase(mock_stream_id);
 }
 
 TEST_F(MirClientSurfaceTest, resizes_streams_and_calls_callback_if_customized_streams)
 {
     using namespace testing;
-    auto mock_stream = std::make_shared<NiceMock<mtd::MockClientBufferStream>>();
+    auto mock_stream = std::make_shared<NiceMock<mtd::MockMirBufferStream>>();
+    mir::frontend::BufferStreamId const mock_stream_id(2);
     auto mock_input_platform = std::make_shared<NiceMock<MockClientInputPlatform>>();
-    ON_CALL(*mock_stream, rpc_id()).WillByDefault(Return(mir::frontend::BufferStreamId(2)));
+    ON_CALL(*mock_stream, rpc_id()).WillByDefault(Return(mock_stream_id));
     ON_CALL(*mock_input_platform, create_input_receiver(_,_,_))
         .WillByDefault(Return(std::make_shared<mt::TestDispatchable>([]{})));
 
     geom::Size size(120, 124);
     EXPECT_CALL(*mock_stream, set_size(size)).Times(0);
     auto ev = mir::events::make_event(mir::frontend::SurfaceId(2), size);
-    MirSurface surface{connection.get(), *client_comm_channel, nullptr,
+    MirWindow surface{connection.get(), *client_comm_channel, nullptr,
         mock_stream, mock_input_platform, spec, surface_proto, wh};
 
-    MirSurfaceSpec spec;
+    MirWindowSpec spec;
     std::vector<ContentInfo> info =
         { ContentInfo{ geom::Displacement{0,0}, 2, geom::Size{1,1}} };
     spec.streams = info;
+    surface_map->insert(mock_stream_id, mock_stream);
     surface.modify(spec)->wait_for_all();
     surface.handle_event(*ev);
-    surface_map->erase(mir::frontend::BufferStreamId(2));
+    surface_map->erase(mock_stream_id);
 }
 
 TEST_F(MirClientSurfaceTest, parameters_are_unhooked_from_stream_sizes)
 {
     using namespace testing;
-    auto mock_stream = std::make_shared<mtd::MockClientBufferStream>(); 
+    auto mock_stream = std::make_shared<mtd::MockMirBufferStream>(); 
     auto mock_input_platform = std::make_shared<NiceMock<MockClientInputPlatform>>();
     ON_CALL(*mock_input_platform, create_input_receiver(_,_,_))
         .WillByDefault(Return(std::make_shared<mt::TestDispatchable>([]{})));
@@ -558,7 +679,7 @@ TEST_F(MirClientSurfaceTest, parameters_are_unhooked_from_stream_sizes)
     surface_proto.set_width(size.width.as_int());
     surface_proto.set_height(size.height.as_int());
 
-    MirSurface surface{connection.get(), *client_comm_channel, nullptr,
+    MirWindow surface{connection.get(), *client_comm_channel, nullptr,
         mock_stream, mock_input_platform, spec, surface_proto, wh};
 
     auto params = surface.get_parameters();
@@ -574,7 +695,7 @@ TEST_F(MirClientSurfaceTest, parameters_are_unhooked_from_stream_sizes)
 TEST_F(MirClientSurfaceTest, initial_sizes_are_from_response_from_server)
 {
     using namespace testing;
-    auto mock_stream = std::make_shared<mtd::MockClientBufferStream>(); 
+    auto mock_stream = std::make_shared<mtd::MockMirBufferStream>(); 
     auto mock_input_platform = std::make_shared<NiceMock<MockClientInputPlatform>>();
     ON_CALL(*mock_input_platform, create_input_receiver(_,_,_))
         .WillByDefault(Return(std::make_shared<mt::TestDispatchable>([]{})));
@@ -583,7 +704,7 @@ TEST_F(MirClientSurfaceTest, initial_sizes_are_from_response_from_server)
 
     surface_proto.set_width(size.width.as_int());
     surface_proto.set_height(size.height.as_int());
-    MirSurface surface{connection.get(), *client_comm_channel, nullptr,
+    MirWindow surface{connection.get(), *client_comm_channel, nullptr,
         mock_stream, mock_input_platform, spec, surface_proto, wh};
 
     auto params = surface.get_parameters();
