@@ -23,6 +23,7 @@
 #include "src/server/frontend/event_sender.h"
 
 #include "mir/events/event_builders.h"
+#include "mir/client_visible_error.h"
 
 #include "mir/test/display_config_matchers.h"
 #include "mir/test/input_devices_matcher.h"
@@ -33,12 +34,15 @@
 #include "mir/test/doubles/mock_platform_ipc_operations.h"
 #include "mir/input/device.h"
 #include "mir/input/device_capability.h"
-#include "mir/input/pointer_configuration.h"
-#include "mir/input/touchpad_configuration.h"
+#include "mir/input/mir_input_config.h"
+#include "mir/input/mir_input_config_serialization.h"
+#include "mir/input/mir_pointer_config.h"
+#include "mir/input/mir_touchpad_config.h"
 #include "mir/variable_length_array.h"
 
 #include <gtest/gtest.h>
 #include <gmock/gmock.h>
+#include <mir_protobuf.pb.h>
 
 namespace mt = mir::test;
 namespace mi = mir::input;
@@ -71,27 +75,27 @@ struct StubDevice : mi::Device
     {
         return device_unique_id;
     }
-    mir::optional_value<mi::PointerConfiguration> pointer_configuration() const override
+    mir::optional_value<MirPointerConfig> pointer_configuration() const override
     {
         return {};
     }
-    void apply_pointer_configuration(mi::PointerConfiguration const&) override
+    void apply_pointer_configuration(MirPointerConfig const&) override
     {
     }
 
-    mir::optional_value<mi::TouchpadConfiguration> touchpad_configuration() const override
+    mir::optional_value<MirTouchpadConfig> touchpad_configuration() const override
     {
         return {};
     }
-    void apply_touchpad_configuration(mi::TouchpadConfiguration const&) override
+    void apply_touchpad_configuration(MirTouchpadConfig const&) override
     {
     }
 
-    mir::optional_value<mi::KeyboardConfiguration> keyboard_configuration() const override
+    mir::optional_value<MirKeyboardConfig> keyboard_configuration() const override
     {
         return {};
     }
-    void apply_keyboard_configuration(mi::KeyboardConfiguration const&) override
+    void apply_keyboard_configuration(MirKeyboardConfig const&) override
     {
     }
 
@@ -115,6 +119,20 @@ struct EventSender : public testing::Test
     mtd::MockPlatformIpcOperations mock_buffer_packer;
     mfd::EventSender event_sender;
 };
+
+std::function<void(char const*, size_t, mir::frontend::FdSets)>
+make_validator(std::function<void(mir::protobuf::EventSequence const&)> const& sequence_validator)
+{
+    return [sequence_validator](char const* data, size_t len, mir::frontend::FdSets)
+        {
+            mir::protobuf::wire::Result wire;
+            wire.ParseFromArray(data, len);
+            std::string str = wire.events(0);
+            mir::protobuf::EventSequence seq;
+            seq.ParseFromString(str);
+            sequence_validator(seq);
+        };
+}
 }
 
 TEST_F(EventSender, display_send)
@@ -123,18 +141,15 @@ TEST_F(EventSender, display_send)
 
     mtd::StubDisplayConfig config;
 
-    auto msg_validator = [&config](char const* data, size_t len){
-        mir::protobuf::wire::Result wire;
-        wire.ParseFromArray(data, len);
-        std::string str = wire.events(0);
-        mir::protobuf::EventSequence seq;
-        seq.ParseFromString(str);
-        EXPECT_THAT(seq.display_configuration(), mt::DisplayConfigMatches(std::cref(config)));
-    };
+    auto msg_validator = make_validator(
+        [&config](auto const& seq)
+        {
+            EXPECT_THAT(seq.display_configuration(), mt::DisplayConfigMatches(std::cref(config)));
+        });
 
     EXPECT_CALL(mock_msg_sender, send(_, _, _))
         .Times(1)
-        .WillOnce(WithArgs<0,1>(Invoke(msg_validator)));
+        .WillOnce(Invoke(msg_validator));
 
     event_sender.handle_display_config_change(config);
 }
@@ -143,7 +158,7 @@ TEST_F(EventSender, sends_noninput_events)
 {
     using namespace testing;
 
-    auto surface_ev = mev::make_event(mf::SurfaceId{1}, mir_surface_attrib_focus, mir_surface_focused);
+    auto surface_ev = mev::make_event(mf::SurfaceId{1}, mir_window_attrib_focus, mir_window_focus_state_focused);
     auto resize_ev = mev::make_event(mf::SurfaceId{1}, {10, 10});
 
     EXPECT_CALL(mock_msg_sender, send(_, _, _))
@@ -181,48 +196,46 @@ TEST_F(EventSender, sends_input_devices)
 {
     using namespace testing;
 
-    std::vector<std::shared_ptr<mi::Device>> devices{
-        std::make_shared<StubDevice>(3, mi::DeviceCapability::pointer | mi::DeviceCapability::touchpad, "touchpad",
-                                     "5352"),
-        std::make_shared<StubDevice>(23, mi::DeviceCapability::keyboard | mi::DeviceCapability::alpha_numeric,
-                                     "keybaord", "7853")};
+    MirInputDevice tpd(3, mi::DeviceCapability::pointer | mi::DeviceCapability::touchpad, "touchpad", "5352");
+    MirInputDevice kbd(23, mi::DeviceCapability::keyboard | mi::DeviceCapability::alpha_numeric, "keyboard",
+                                "5352");
 
-    auto msg_validator = [&devices](char const* data, size_t len){
-        mir::protobuf::wire::Result wire;
-        wire.ParseFromArray(data, len);
-        std::string str = wire.events(0);
-        mir::protobuf::EventSequence seq;
-        seq.ParseFromString(str);
-        EXPECT_THAT(seq.input_devices(), mt::InputDevicesMatch(devices));
-    };
+    MirInputConfig devices;
+    devices.add_device_config(tpd);
+    devices.add_device_config(kbd);
+
+    auto msg_validator = make_validator(
+        [&devices](auto const& seq)
+        {
+            auto received_input_config = mi::deserialize_input_config(seq.input_configuration());
+            EXPECT_THAT(received_input_config, Eq(devices));
+        });
 
     EXPECT_CALL(mock_msg_sender, send(_, _, _))
         .Times(1)
-        .WillOnce(WithArgs<0,1>(Invoke(msg_validator)));
+        .WillOnce(Invoke(msg_validator));
 
-    event_sender.handle_input_device_change(devices);
+    event_sender.handle_input_config_change(devices);
 }
 
 TEST_F(EventSender, sends_empty_sequence_of_devices)
 {
     using namespace testing;
 
-    std::vector<std::shared_ptr<mi::Device>> devices;
+    MirInputConfig empty;
 
-    auto msg_validator = [&devices](char const* data, size_t len){
-        mir::protobuf::wire::Result wire;
-        wire.ParseFromArray(data, len);
-        std::string str = wire.events(0);
-        mir::protobuf::EventSequence seq;
-        seq.ParseFromString(str);
-        EXPECT_THAT(seq.input_devices(), mt::InputDevicesMatch(devices));
-    };
+    auto msg_validator = make_validator(
+        [&empty](auto const& seq)
+        {
+            auto received_input_config = mi::deserialize_input_config(seq.input_configuration());
+            EXPECT_THAT(received_input_config, Eq(empty));
+        });
 
     EXPECT_CALL(mock_msg_sender, send(_, _, _))
         .Times(1)
-        .WillOnce(WithArgs<0,1>(Invoke(msg_validator)));
+        .WillOnce(Invoke(msg_validator));
 
-    event_sender.handle_input_device_change(devices);
+    event_sender.handle_input_config_change(empty);
 }
 
 TEST_F(EventSender, can_send_error_buffer)
@@ -253,7 +266,45 @@ TEST_F(EventSender, can_send_error_buffer)
                 sent_buffer.resize(size);
                 memcpy(sent_buffer.data(), data, size); 
             }));
-    event_sender.error_buffer(properties, error_msg);
+    event_sender.error_buffer(properties.size, properties.format, error_msg);
     ASSERT_THAT(sent_buffer.size(), Eq(expected_buffer.size()));
     EXPECT_FALSE(memcmp(sent_buffer.data(), expected_buffer.data(), sent_buffer.size()));
 }
+
+TEST_F(EventSender, sends_errors)
+{
+    using namespace testing;
+
+    class TestError : public mir::ClientVisibleError
+    {
+    public:
+        TestError()
+            : ClientVisibleError("An explosion of delight")
+        {
+        }
+
+        MirErrorDomain domain() const noexcept override
+        {
+            return static_cast<MirErrorDomain>(32);
+        }
+
+        uint32_t code() const noexcept override
+        {
+            return 0xDEADBEEF;
+        }
+    } error;
+
+    auto msg_validator = make_validator(
+        [&error](auto const& seq)
+        {
+            EXPECT_TRUE(seq.has_structured_error());
+            EXPECT_THAT(seq.structured_error().domain(), Eq(error.domain()));
+            EXPECT_THAT(seq.structured_error().code(), Eq(error.code()));
+        });
+
+    EXPECT_CALL(mock_msg_sender, send(_, _, _))
+        .Times(1)
+        .WillOnce(Invoke(msg_validator));
+
+    event_sender.handle_error(error);
+}
