@@ -18,8 +18,8 @@
 
 #include "src/server/input/default_device.h"
 #include "mir/input/input_device.h"
-#include "mir/input/touchpad_configuration.h"
-#include "mir/input/pointer_configuration.h"
+#include "mir/input/mir_touchpad_config.h"
+#include "mir/input/mir_pointer_config.h"
 #include "mir/test/doubles/mock_key_mapper.h"
 #include "mir/dispatch/action_queue.h"
 #include "mir/test/fake_shared.h"
@@ -83,7 +83,7 @@ struct DefaultDevice : Test
 TEST_F(DefaultDevice, refuses_touchpad_config_on_mice)
 {
     mi::DefaultDevice dev(MirInputDeviceId{17}, queue, mouse, mt::fake_shared(key_mapper));
-    mi::TouchpadConfiguration touch_conf;
+    MirTouchpadConfig touch_conf;
 
     EXPECT_THROW({dev.apply_touchpad_configuration(touch_conf);}, std::invalid_argument);
 }
@@ -91,8 +91,8 @@ TEST_F(DefaultDevice, refuses_touchpad_config_on_mice)
 TEST_F(DefaultDevice, refuses_touchpad_and_pointer_settings_on_keyboards)
 {
     mi::DefaultDevice dev(MirInputDeviceId{17}, queue, keyboard, mt::fake_shared(key_mapper));
-    mi::TouchpadConfiguration touch_conf;
-    mi::PointerConfiguration pointer_conf;
+    MirTouchpadConfig touch_conf;
+    MirPointerConfig pointer_conf;
 
     EXPECT_THROW({dev.apply_touchpad_configuration(touch_conf);}, std::invalid_argument);
     EXPECT_THROW({dev.apply_pointer_configuration(pointer_conf);}, std::invalid_argument);
@@ -102,7 +102,7 @@ TEST_F(DefaultDevice, refuses_touchpad_and_pointer_settings_on_keyboards)
 TEST_F(DefaultDevice, accepts_pointer_config_on_mice)
 {
     mi::DefaultDevice dev(MirInputDeviceId{17}, queue, mouse, mt::fake_shared(key_mapper));
-    mi::PointerConfiguration pointer_conf;
+    MirPointerConfig pointer_conf;
 
     EXPECT_CALL(mouse, apply_settings(Matcher<mi::PointerSettings const&>(_)));
 
@@ -113,8 +113,8 @@ TEST_F(DefaultDevice, accepts_pointer_config_on_mice)
 TEST_F(DefaultDevice, accepts_touchpad_and_pointer_config_on_touchpads)
 {
     mi::DefaultDevice dev(MirInputDeviceId{17}, queue, touchpad, mt::fake_shared(key_mapper));
-    mi::TouchpadConfiguration touch_conf;
-    mi::PointerConfiguration pointer_conf;
+    MirTouchpadConfig touch_conf;
+    MirPointerConfig pointer_conf;
 
     EXPECT_CALL(touchpad, apply_settings(Matcher<mi::PointerSettings const&>(_)));
     EXPECT_CALL(touchpad, apply_settings(Matcher<mi::TouchpadSettings const&>(_)));
@@ -129,13 +129,13 @@ TEST_F(DefaultDevice, ensures_cursor_accleration_bias_is_in_range)
 {
     mi::DefaultDevice dev(MirInputDeviceId{17}, queue, touchpad, mt::fake_shared(key_mapper));
 
-    mi::PointerConfiguration pointer_conf;
-    pointer_conf.cursor_acceleration_bias = 3.0;
+    MirPointerConfig pointer_conf;
+    pointer_conf.cursor_acceleration_bias(3.0);
     EXPECT_THROW({dev.apply_pointer_configuration(pointer_conf);}, std::invalid_argument);
-    pointer_conf.cursor_acceleration_bias = -2.0;
+    pointer_conf.cursor_acceleration_bias(-2.0);
     EXPECT_THROW({dev.apply_pointer_configuration(pointer_conf);}, std::invalid_argument);
 
-    pointer_conf.cursor_acceleration_bias = 1.0;
+    pointer_conf.cursor_acceleration_bias(1.0);
     EXPECT_NO_THROW(dev.apply_pointer_configuration(pointer_conf));
 }
 
@@ -155,3 +155,26 @@ TEST_F(DefaultDevice, when_managing_a_keyboard_us_layout_is_set_by_default)
 
     queue->dispatch(md::FdEvent::readable);
 }
+
+TEST_F(DefaultDevice, disable_queue_ends_config_processing)
+{
+    mi::DefaultDevice dev(MirInputDeviceId{17}, queue, touchpad, mt::fake_shared(key_mapper));
+
+    dev.disable_queue();
+    MirPointerConfig pointer_conf;
+    pointer_conf.cursor_acceleration_bias(1.0);
+
+    EXPECT_CALL(touchpad, apply_settings(Matcher<mi::TouchpadSettings const&>(_))).Times(0);
+    dev.apply_pointer_configuration(pointer_conf);
+}
+
+TEST_F(DefaultDevice, disable_queue_removes_reference_to_queue)
+{
+    std::weak_ptr<md::ActionQueue> ref = queue;
+    mi::DefaultDevice dev(MirInputDeviceId{17}, queue, touchpad, mt::fake_shared(key_mapper));
+    queue.reset();
+
+    EXPECT_THAT(ref.lock(), Ne(nullptr));
+    dev.disable_queue();
+    EXPECT_THAT(ref.lock(), Eq(nullptr));
+}
