@@ -22,6 +22,7 @@
 #include "mir/scene/surface_creation_parameters.h"
 #include "mir/scene/surface_factory.h"
 #include "mir/scene/null_session_listener.h"
+#include "mir/client_visible_error.h"
 #include "mir/test/fake_shared.h"
 #include "mir/test/doubles/mock_surface_stack.h"
 #include "mir/test/doubles/mock_surface.h"
@@ -111,9 +112,9 @@ MATCHER_P(HasParent, parent, "")
     return arg.parent.lock() == parent;
 }
 
-MATCHER(IsSurfaceOutputEvent, "")
+MATCHER(IsMirWindowOutputEvent, "")
 {
-    return mir_event_get_type(&arg) == mir_event_type_surface_output;
+    return mir_event_get_type(&arg) == mir_event_type_window_output;
 }
 
 struct StubSurfaceStack : public msh::SurfaceStack
@@ -130,7 +131,7 @@ struct StubSurfaceStack : public msh::SurfaceStack
     void remove_surface(std::weak_ptr<ms::Surface> const&) override
     {
     }
-    auto surface_at(mir::geometry::Point) const -> std::shared_ptr<ms::Surface>
+    auto surface_at(mir::geometry::Point) const -> std::shared_ptr<ms::Surface> override
     {
         return std::shared_ptr<ms::Surface>{};
     }
@@ -695,7 +696,7 @@ TEST_F(ApplicationSession, surface_uses_prexisting_buffer_stream_if_set)
 
     ms::SurfaceCreationParameters params = ms::SurfaceCreationParameters{}
         .of_name("Aardavks")
-        .of_type(mir_surface_type_normal)
+        .of_type(mir_window_type_normal)
         .with_buffer_stream(id);
 
     session->create_surface(params, event_sink);
@@ -736,6 +737,34 @@ TEST_F(ApplicationSessionSender, display_config_sender)
     app_session.send_display_config(stub_config);
 }
 
+TEST_F(ApplicationSessionSender, error_sender)
+{
+    using namespace ::testing;
+
+    class TestError : public mir::ClientVisibleError
+    {
+    public:
+        TestError()
+            : ClientVisibleError("Hello")
+        {
+        }
+
+        MirErrorDomain domain() const noexcept override
+        {
+            return static_cast<MirErrorDomain>(42);
+        }
+
+        uint32_t code() const noexcept override
+        {
+            return 8u;
+        }
+    } error;
+    EXPECT_CALL(sender, handle_error(Ref(error)))
+        .Times(1);
+
+    app_session.send_error(error);
+}
+
 TEST_F(ApplicationSessionSender, lifecycle_event_sender)
 {
     using namespace ::testing;
@@ -807,8 +836,8 @@ int calculate_dpi(geom::Size const& resolution, geom::Size const& size)
 struct ApplicationSessionSurfaceOutput : public ApplicationSession
 {
     ApplicationSessionSurfaceOutput() :
-        high_dpi(static_cast<mg::DisplayConfigurationOutputId>(5), {3840, 2160}, {509, 286}, 2.5f, mir_form_factor_monitor),
-        projector(static_cast<mg::DisplayConfigurationOutputId>(2), {1280, 1024}, {800, 600}, 0.5f, mir_form_factor_projector),
+        high_dpi(static_cast<mg::DisplayConfigurationOutputId>(5), {3840, 2160}, {509, 286}, 2.5f, 60.0, mir_form_factor_monitor),
+        projector(static_cast<mg::DisplayConfigurationOutputId>(2), {1280, 1024}, {800, 600}, 0.5f, 50.0, mir_form_factor_projector),
         stub_surface_factory{std::make_shared<ObserverPreservingSurfaceFactory>()},
         sender{std::make_shared<testing::NiceMock<mtd::MockEventSink>>()},
         app_session(
@@ -831,8 +860,9 @@ struct ApplicationSessionSurfaceOutput : public ApplicationSession
             geom::Size const& resolution,
             geom::Size const& physical_size,
             float scale,
+            double hz,
             MirFormFactor form_factor) :
-            output{id, resolution, physical_size, mir_pixel_format_argb_8888, 60.0, true},
+            output{id, resolution, physical_size, mir_pixel_format_argb_8888, hz, true},
             form_factor{form_factor},
             scale{scale},
             dpi{calculate_dpi(resolution, physical_size)},
@@ -865,30 +895,39 @@ MATCHER_P(SurfaceOutputEventFor, output, "")
 {
     using namespace testing;
 
-    if (mir_event_get_type(arg) != mir_event_type_surface_output)
+    if (mir_event_get_type(arg) != mir_event_type_window_output)
     {
-        *result_listener << "Event is not a MirSurfaceOutputEvent";
+        *result_listener << "Event is not a MirWindowOutputEvent";
         return 0;
     }
 
-    auto const event = mir_event_get_surface_output_event(arg);
+    auto const event = mir_event_get_window_output_event(arg);
+
+    if (output.output.current_mode_index >= output.output.modes.size())
+        return false;
+
+    auto const& mode = output.output.modes[output.output.current_mode_index];
 
     return
         ExplainMatchResult(
             Eq(output.dpi),
-            mir_surface_output_event_get_dpi(event),
+            mir_window_output_event_get_dpi(event),
             result_listener) &&
         ExplainMatchResult(
             Eq(output.form_factor),
-            mir_surface_output_event_get_form_factor(event),
+            mir_window_output_event_get_form_factor(event),
             result_listener) &&
         ExplainMatchResult(
             Eq(output.scale),
-            mir_surface_output_event_get_scale(event),
+            mir_window_output_event_get_scale(event),
+            result_listener) &&
+        ExplainMatchResult(
+            Eq(mode.vrefresh_hz),
+            mir_window_output_event_get_refresh_rate(event),
             result_listener) &&
         ExplainMatchResult(
             Eq(output.id),
-            mir_surface_output_event_get_output_id(event),
+            mir_window_output_event_get_output_id(event),
             result_listener);
 }
 }
@@ -897,14 +936,16 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_events_to_surfaces)
 {
     using namespace ::testing;
 
-    //MirEvent* event;
-    std::unique_ptr<MirEvent> event;
+    MirWindowOutputEvent event;
     bool event_received{false};
 
-    EXPECT_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
+    EXPECT_CALL(*sender, handle_event(IsMirWindowOutputEvent()))
         .WillOnce(Invoke([&event, &event_received](MirEvent const& ev)
                          {
-                             event.reset(ev.clone());
+                             if (event.type() == mir_event_type_window_output)
+                             {
+                                 event = *ev.to_window_output();
+                             }
                              event_received = true;
                          }));
 
@@ -922,7 +963,7 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_events_to_surfaces)
     auto surface = app_session.surface(surf_id);
 
     ASSERT_TRUE(event_received);
-    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
 }
 
 TEST_F(ApplicationSessionSurfaceOutput, sends_correct_surface_details_to_surface)
@@ -931,13 +972,16 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_correct_surface_details_to_surface
 
     std::array<TestOutput const*, 2> outputs{{ &high_dpi, &projector }};
 
-    std::array<std::unique_ptr<MirEvent>, 2> event;
+    std::array<MirWindowOutputEvent, 2> event;
     int events_received{0};
 
-    ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
+    ON_CALL(*sender, handle_event(IsMirWindowOutputEvent()))
         .WillByDefault(Invoke([&event, &events_received](MirEvent const& ev)
                          {
-                             event[events_received].reset(ev.clone());
+                             if (ev.type() == mir_event_type_window_output)
+                             {
+                                 event[events_received] = *ev.to_window_output();
+                             }
                              ++events_received;
                          }));
 
@@ -974,8 +1018,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_correct_surface_details_to_surface
 
     for (int i = 0; i < 2 ; ++i)
     {
-        EXPECT_THAT(event[i]->to_surface_output()->surface_id(), Eq(ids[i].as_value()));
-        EXPECT_THAT(event[i].get(), SurfaceOutputEventFor(*outputs[i]));
+        EXPECT_THAT(event[i].to_window_output()->surface_id(), Eq(ids[i].as_value()));
+        EXPECT_THAT(&event[i], SurfaceOutputEventFor(*outputs[i]));
     }
 }
 
@@ -985,13 +1029,16 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_details_of_the_hightest_scale_fact
 
     std::array<TestOutput const*, 2> outputs{{ &projector, &high_dpi }};
 
-    std::unique_ptr<MirEvent> event;
+    MirWindowOutputEvent event;
     bool event_received{false};
 
-    ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
+    ON_CALL(*sender, handle_event(IsMirWindowOutputEvent()))
         .WillByDefault(Invoke([&event, &event_received](MirEvent const& ev)
                               {
-                                  event.reset(ev.clone());
+                                  if (ev.type() == mir_event_type_window_output)
+                                  {
+                                      event = *ev.to_window_output();
+                                  }
                                   event_received = true;
                               }));
 
@@ -1020,8 +1067,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_details_of_the_hightest_scale_fact
 
     ASSERT_TRUE(event_received);
 
-    EXPECT_THAT(event->to_surface_output()->surface_id(), Eq(id.as_value()));
-    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event.to_window_output()->surface_id(), Eq(id.as_value()));
+    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
 }
 
 TEST_F(ApplicationSessionSurfaceOutput, surfaces_on_edges_get_correct_values)
@@ -1030,13 +1077,16 @@ TEST_F(ApplicationSessionSurfaceOutput, surfaces_on_edges_get_correct_values)
 
     std::array<TestOutput const*, 2> outputs{{ &projector, &high_dpi }};
 
-    std::unique_ptr<MirEvent> event;
+    MirWindowOutputEvent event;
     bool event_received{false};
 
-    ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
+    ON_CALL(*sender, handle_event(IsMirWindowOutputEvent()))
         .WillByDefault(Invoke([&event, &event_received](MirEvent const& ev)
                               {
-                                  event.reset(ev.clone());
+                                  if (ev.type() == mir_event_type_window_output)
+                                  {
+                                      event = *ev.to_window_output();
+                                  }
                                   event_received = true;
                               }));
 
@@ -1064,21 +1114,21 @@ TEST_F(ApplicationSessionSurfaceOutput, surfaces_on_edges_get_correct_values)
     surface->move_to({outputs[0]->width - ((surface->size().width.as_uint32_t()) / 2), 100});
 
     ASSERT_TRUE(event_received);
-    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
 
     event_received = false;
     // This should be *just* entirely on the projector
     surface->move_to({outputs[0]->width - surface->size().width.as_uint32_t(), 100});
 
     ASSERT_TRUE(event_received);
-    EXPECT_THAT(event.get(), SurfaceOutputEventFor(projector));
+    EXPECT_THAT(&event, SurfaceOutputEventFor(projector));
 
     event_received = false;
     // This should have a single pixel overlap on the high_dpi
     surface->move_to({outputs[0]->width - (surface->size().width.as_uint32_t() - 1), 100});
 
     ASSERT_TRUE(event_received);
-    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
 }
 
 TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
@@ -1087,14 +1137,17 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
 
     std::array<TestOutput const*, 2> outputs {{ &projector, &high_dpi }};
 
-    std::unique_ptr<MirEvent> event;
+    MirWindowOutputEvent event;
     int events_received{0};
 
 
-    ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
+    ON_CALL(*sender, handle_event(IsMirWindowOutputEvent()))
         .WillByDefault(Invoke([&event, &events_received](MirEvent const& ev)
                               {
-                                  event.reset(ev.clone());
+                                  if (ev.type() == mir_event_type_window_output)
+                                  {
+                                      event = *ev.to_window_output();
+                                  }
                                   events_received++;
                               }));
 
@@ -1125,8 +1178,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
     ASSERT_THAT(events_received, Ge(1));
     auto events_expected = events_received + 1;
 
-    EXPECT_THAT(event->to_surface_output()->surface_id(), Eq(id.as_value()));
-    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event.to_window_output()->surface_id(), Eq(id.as_value()));
+    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
 
     // Now solely on the left output
     surface->move_to({0, 0});
@@ -1134,8 +1187,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
     ASSERT_THAT(events_received, Eq(events_expected));
     events_expected++;
 
-    EXPECT_THAT(event->to_surface_output()->surface_id(), Eq(id.as_value()));
-    EXPECT_THAT(event.get(), SurfaceOutputEventFor(projector));
+    EXPECT_THAT(event.to_window_output()->surface_id(), Eq(id.as_value()));
+    EXPECT_THAT(&event, SurfaceOutputEventFor(projector));
 
     // Now solely on the right output
     surface->move_to({outputs[0]->width + 100, 100});
@@ -1143,8 +1196,8 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move)
     ASSERT_THAT(events_received, Eq(events_expected));
     events_expected++;
 
-    EXPECT_THAT(event->to_surface_output()->surface_id(), Eq(id.as_value()));
-    EXPECT_THAT(event.get(), SurfaceOutputEventFor(high_dpi));
+    EXPECT_THAT(event.to_window_output()->surface_id(), Eq(id.as_value()));
+    EXPECT_THAT(&event, SurfaceOutputEventFor(high_dpi));
 }
 
 TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move_only_if_changed)
@@ -1155,7 +1208,7 @@ TEST_F(ApplicationSessionSurfaceOutput, sends_surface_output_event_on_move_only_
 
     int events_received{0};
 
-    ON_CALL(*sender, handle_event(IsSurfaceOutputEvent()))
+    ON_CALL(*sender, handle_event(IsMirWindowOutputEvent()))
         .WillByDefault(Invoke([&events_received](MirEvent const&)
                               {
                                   events_received++;
