@@ -17,9 +17,9 @@
  */
 
 #include "src/server/scene/mediating_display_changer.h"
-#include "src/server/scene/session_container.h"
+#include "mir/scene/session_container.h"
 #include "mir/graphics/display_configuration_policy.h"
-#include "mir/graphics/display_configuration_report.h"
+#include "mir/graphics/display_configuration_observer.h"
 #include "mir/geometry/rectangles.h"
 #include "src/server/scene/broadcasting_session_event_sink.h"
 #include "mir/server_action_queue.h"
@@ -31,11 +31,13 @@
 #include "mir/test/doubles/mock_scene_session.h"
 #include "mir/test/doubles/mock_input_region.h"
 #include "mir/test/doubles/stub_session.h"
+#include "mir/test/doubles/stub_session_container.h"
 #include "mir/test/fake_shared.h"
 #include "mir/test/display_config_matchers.h"
 #include "mir/test/doubles/fake_alarm_factory.h"
 
 #include <mutex>
+#include <boost/throw_exception.hpp>
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -71,7 +73,11 @@ auto display_output(
                                               mode,
                                               mir_orientation_normal,
                                               1.0f,
-                                              mir_form_factor_phone
+                                              mir_form_factor_phone,
+                                              mir_subpixel_arrangement_unknown,
+                                              {},
+                                              mir_output_gamma_unsupported,
+                                              {}
         };
 }
 
@@ -98,42 +104,22 @@ public:
     MOCK_METHOD1(apply_to, void(mg::DisplayConfiguration&));
 };
 
-class StubSessionContainer : public ms::SessionContainer
-{
-public:
-    void insert_session(std::shared_ptr<ms::Session> const& session)
-    {
-        sessions.push_back(session);
-    }
-
-    void remove_session(std::shared_ptr<ms::Session> const&)
-    {
-    }
-
-    void for_each(std::function<void(std::shared_ptr<ms::Session> const&)> f) const
-    {
-        for (auto const& session : sessions)
-            f(session);
-    }
-
-    std::shared_ptr<ms::Session> successor_of(std::shared_ptr<ms::Session> const&) const
-    {
-        return {};
-    }
-
-private:
-    std::vector<std::shared_ptr<ms::Session>> sessions;
-};
 
 struct MockDisplay : public mtd::MockDisplay
 {
-    std::unique_ptr<mg::DisplayConfiguration> configuration() const override
+    MockDisplay()
+        : config{std::make_unique<mtd::StubDisplayConfig>()}
     {
-        conf_ptr = new mtd::StubDisplayConfig{};
-        return std::unique_ptr<mg::DisplayConfiguration>(conf_ptr);
+        ON_CALL(*this, configure(_))
+            .WillByDefault(Invoke([this](auto& conf) { config = conf.clone(); }));
     }
 
-    mutable mg::DisplayConfiguration* conf_ptr;
+    std::unique_ptr<mg::DisplayConfiguration> configuration() const override
+    {
+        return config->clone();
+    }
+
+    std::unique_ptr<mg::DisplayConfiguration> config;
 };
 
 struct StubServerActionQueue : mir::ServerActionQueue
@@ -158,10 +144,21 @@ struct MockServerActionQueue : mir::ServerActionQueue
     MOCK_METHOD1(resume_processing_for, void(void const*));
 };
 
-struct StubDisplayConfigurationReport : mg::DisplayConfigurationReport
+struct StubDisplayConfigurationObserver : mg::DisplayConfigurationObserver
 {
-    void initial_configuration(mg::DisplayConfiguration const&) override {}
-    void new_configuration(mg::DisplayConfiguration const&) override {}
+    void initial_configuration(std::shared_ptr<mg::DisplayConfiguration const> const&) override {}
+    void configuration_applied(std::shared_ptr<mg::DisplayConfiguration const> const&) override {}
+    void base_configuration_updated(std::shared_ptr<mg::DisplayConfiguration const> const&) override {}
+    void session_configuration_applied(
+        std::shared_ptr<mf::Session> const&,
+        std::shared_ptr<mg::DisplayConfiguration> const&) override {};
+    void session_configuration_removed(std::shared_ptr<mf::Session> const&) override {};
+    void configuration_failed(
+        std::shared_ptr<mg::DisplayConfiguration const> const&,
+        std::exception const&) override {}
+    void catastrophic_configuration_error(
+        std::shared_ptr<mg::DisplayConfiguration const> const&,
+        std::exception const&) override { }
 };
 
 
@@ -186,11 +183,11 @@ struct MediatingDisplayChangerTest : public ::testing::Test
     testing::NiceMock<MockDisplay> mock_display;
     testing::NiceMock<mtd::MockCompositor> mock_compositor;
     testing::NiceMock<MockDisplayConfigurationPolicy> mock_conf_policy;
-    StubSessionContainer stub_session_container;
+    mtd::StubSessionContainer stub_session_container;
     ms::BroadcastingSessionEventSink session_event_sink;
     mtd::StubDisplayConfig base_config;
     StubServerActionQueue server_action_queue;
-    StubDisplayConfigurationReport display_configuration_report;
+    StubDisplayConfigurationObserver display_configuration_report;
     testing::NiceMock<mtd::MockInputRegion> mock_input_region;
     mtd::FakeAlarmFactory alarm_factory;
     std::shared_ptr<ms::MediatingDisplayChanger> changer;
@@ -203,15 +200,18 @@ TEST_F(MediatingDisplayChangerTest, returns_base_configuration_from_display_at_s
     using namespace testing;
 
     auto const base_conf = changer->base_configuration();
-    EXPECT_THAT(*base_conf, mt::DisplayConfigMatches(std::ref(*mock_display.conf_ptr)));
+    EXPECT_THAT(*base_conf, mt::DisplayConfigMatches(std::ref(*mock_display.configuration())));
 }
 
-TEST_F(MediatingDisplayChangerTest, pauses_system_when_applying_new_configuration_for_focused_session)
+TEST_F(MediatingDisplayChangerTest, pauses_system_when_applying_new_configuration_for_focused_session_would_invalidate_display_buffers)
 {
     using namespace testing;
     mtd::NullDisplayConfiguration conf;
     auto session = std::make_shared<mtd::StubSession>();
 
+    ON_CALL(mock_display, apply_if_configuration_preserves_display_buffers(_))
+        .WillByDefault(Return(false));
+
     InSequence s;
     EXPECT_CALL(mock_compositor, stop());
 
@@ -224,6 +224,56 @@ TEST_F(MediatingDisplayChangerTest, pauses_system_when_applying_new_configuratio
                        mt::fake_shared(conf));
 }
 
+TEST_F(MediatingDisplayChangerTest, does_not_pause_system_when_applying_new_configuration_for_focused_session_would_preserve_display_buffers)
+{
+    using namespace testing;
+    mtd::NullDisplayConfiguration conf;
+    auto session = std::make_shared<mtd::StubSession>();
+
+    EXPECT_CALL(mock_display, apply_if_configuration_preserves_display_buffers(Ref(conf)))
+        .WillOnce(Return(true));
+
+    EXPECT_CALL(mock_compositor, stop()).Times(0);
+    EXPECT_CALL(mock_compositor, start()).Times(0);
+    EXPECT_CALL(mock_display, configure(_)).Times(0);
+
+    session_event_sink.handle_focus_change(session);
+    changer->configure(session,
+                       mt::fake_shared(conf));
+}
+
+TEST_F(MediatingDisplayChangerTest, sends_error_when_applying_new_configuration_for_focused_session_fails)
+{
+    using namespace testing;
+    mtd::NullDisplayConfiguration conf;
+    auto session = std::make_shared<mtd::MockSceneSession>();
+
+    ON_CALL(mock_display, configure(Ref(conf)))
+        .WillByDefault(InvokeWithoutArgs([]() { BOOST_THROW_EXCEPTION(std::runtime_error{"Ducks!"}); }));
+    EXPECT_CALL(*session, send_error(_));
+
+    session_event_sink.handle_focus_change(session);
+    changer->configure(session,
+                       mt::fake_shared(conf));
+}
+
+TEST_F(MediatingDisplayChangerTest, reapplies_old_config_when_applying_new_configuration_fails)
+{
+    using namespace testing;
+    mtd::NullDisplayConfiguration conf;
+    auto session = std::make_shared<NiceMock<mtd::MockSceneSession>>();
+    auto existing_configuration = changer->base_configuration();
+
+    InSequence s;
+    EXPECT_CALL(mock_display, configure(Ref(conf)))
+        .WillOnce(InvokeWithoutArgs([]() { BOOST_THROW_EXCEPTION(std::runtime_error{"Ducks!"}); }));
+    EXPECT_CALL(mock_display, configure(mt::DisplayConfigMatches(std::cref(*existing_configuration))));
+
+    session_event_sink.handle_focus_change(session);
+    changer->configure(session, mt::fake_shared(conf));
+}
+
+
 TEST_F(MediatingDisplayChangerTest, doesnt_apply_config_for_unfocused_session)
 {
     using namespace testing;
@@ -244,18 +294,20 @@ TEST_F(MediatingDisplayChangerTest, returns_updated_base_configuration_after_har
     mtd::StubDisplayConfig conf{2};
 
     changer->configure_for_hardware_change(
-        mt::fake_shared(conf),
-        mir::DisplayChanger::PauseResumeSystem);
+        mt::fake_shared(conf));
 
     auto const base_conf = changer->base_configuration();
     EXPECT_THAT(*base_conf, mt::DisplayConfigMatches(conf));
 }
 
-TEST_F(MediatingDisplayChangerTest, handles_hardware_change_properly_when_pausing_system)
+TEST_F(MediatingDisplayChangerTest, handles_hardware_change_when_display_buffers_are_invalidated)
 {
     using namespace testing;
     mtd::NullDisplayConfiguration conf;
 
+    ON_CALL(mock_display, apply_if_configuration_preserves_display_buffers(_))
+        .WillByDefault(Return(false));
+
     InSequence s;
     EXPECT_CALL(mock_conf_policy, apply_to(Ref(conf)));
 
@@ -263,24 +315,81 @@ TEST_F(MediatingDisplayChangerTest, handles_hardware_change_properly_when_pausin
     EXPECT_CALL(mock_display, configure(Ref(conf)));
     EXPECT_CALL(mock_compositor, start());
 
-    changer->configure_for_hardware_change(mt::fake_shared(conf),
-                                           mir::DisplayChanger::PauseResumeSystem);
+    changer->configure_for_hardware_change(mt::fake_shared(conf));
 }
 
-TEST_F(MediatingDisplayChangerTest, handles_hardware_change_properly_when_retaining_system_state)
+TEST_F(MediatingDisplayChangerTest, handles_error_when_applying_hardware_change)
 {
     using namespace testing;
     mtd::NullDisplayConfiguration conf;
 
+
+    EXPECT_CALL(mock_display, configure(Ref(conf)))
+        .WillOnce(InvokeWithoutArgs([]() { BOOST_THROW_EXCEPTION(std::runtime_error{"Avocado!"}); }));
+    EXPECT_CALL(mock_display, configure(Not(Ref(conf))))
+        .Times(AnyNumber());
+
+    auto const previous_base_config = changer->base_configuration();
+    ASSERT_THAT(conf, Not(mt::DisplayConfigMatches(std::cref(*previous_base_config))));
+
+    changer->configure_for_hardware_change(mt::fake_shared(conf));
+
+    EXPECT_THAT(*changer->base_configuration(), mt::DisplayConfigMatches(std::cref(*previous_base_config)));
+}
+
+TEST_F(MediatingDisplayChangerTest, handles_hardware_change_when_display_buffers_are_preserved)
+{
+    using namespace testing;
+    mtd::NullDisplayConfiguration conf;
+
+    {
+        InSequence s;
+        EXPECT_CALL(mock_conf_policy, apply_to(Ref(conf)));
+
+        EXPECT_CALL(mock_display, apply_if_configuration_preserves_display_buffers(Ref(conf)))
+            .WillOnce(Return(true));
+    }
+
     EXPECT_CALL(mock_compositor, stop()).Times(0);
+    EXPECT_CALL(mock_display, configure(_)).Times(0);
     EXPECT_CALL(mock_compositor, start()).Times(0);
 
-    InSequence s;
-    EXPECT_CALL(mock_conf_policy, apply_to(Ref(conf)));
-    EXPECT_CALL(mock_display, configure(Ref(conf)));
+    changer->configure_for_hardware_change(mt::fake_shared(conf));
+}
 
-    changer->configure_for_hardware_change(mt::fake_shared(conf),
-                                           mir::DisplayChanger::RetainSystemState);
+TEST_F(MediatingDisplayChangerTest, handles_hardware_change_when_display_buffers_are_preserved_but_new_outputs_are_enabled)
+{
+    using namespace testing;
+
+    auto conf = changer->base_configuration();
+
+    bool new_output_enabled{false};
+    conf->for_each_output(
+        [&new_output_enabled](mg::UserDisplayConfigurationOutput& output)
+        {
+            if (!output.used)
+            {
+                new_output_enabled = true;
+                output.used = true;
+            }
+        });
+    ASSERT_TRUE(new_output_enabled);
+
+    ON_CALL(mock_display, apply_if_configuration_preserves_display_buffers(_))
+        .WillByDefault(Return(true));
+
+    InSequence s;
+    EXPECT_CALL(mock_conf_policy, apply_to(Ref(*conf)));
+
+    /*
+     * Currently we have to tear down and recreate the compositor in order to add
+     * a new output. This is not an inherent limitation, and we might do better later.
+     */
+    EXPECT_CALL(mock_compositor, stop()).Times(1);
+    EXPECT_CALL(mock_display, configure(Ref(*conf)));
+    EXPECT_CALL(mock_compositor, start()).Times(1);
+
+    changer->configure_for_hardware_change(conf);
 }
 
 TEST_F(MediatingDisplayChangerTest, hardware_change_doesnt_apply_base_config_if_per_session_config_is_active)
@@ -303,8 +412,7 @@ TEST_F(MediatingDisplayChangerTest, hardware_change_doesnt_apply_base_config_if_
     EXPECT_CALL(mock_display, configure(_)).Times(0);
     EXPECT_CALL(mock_compositor, start()).Times(0);
 
-    changer->configure_for_hardware_change(conf,
-                                           mir::DisplayChanger::PauseResumeSystem);
+    changer->configure_for_hardware_change(conf);
 }
 
 TEST_F(MediatingDisplayChangerTest, notifies_all_sessions_on_hardware_config_change)
@@ -320,11 +428,33 @@ TEST_F(MediatingDisplayChangerTest, notifies_all_sessions_on_hardware_config_cha
     EXPECT_CALL(mock_session1, send_display_config(_));
     EXPECT_CALL(mock_session2, send_display_config(_));
 
-    changer->configure_for_hardware_change(mt::fake_shared(conf),
-                                           mir::DisplayChanger::PauseResumeSystem);
+    changer->configure_for_hardware_change(mt::fake_shared(conf));
 }
 
-TEST_F(MediatingDisplayChangerTest, focusing_a_session_with_attached_config_applies_config)
+TEST_F(MediatingDisplayChangerTest, notifies_all_sessions_when_hardware_config_change_fails)
+{
+    using namespace testing;
+    mtd::NullDisplayConfiguration conf;
+    mtd::MockSceneSession mock_session1;
+    mtd::MockSceneSession mock_session2;
+
+    auto const previous_base_config = changer->base_configuration();
+    ASSERT_THAT(conf, Not(mt::DisplayConfigMatches(std::cref(*previous_base_config))));
+
+    stub_session_container.insert_session(mt::fake_shared(mock_session1));
+    stub_session_container.insert_session(mt::fake_shared(mock_session2));
+
+    EXPECT_CALL(mock_display, configure(Ref(conf)))
+        .WillOnce(InvokeWithoutArgs([]() { BOOST_THROW_EXCEPTION(std::runtime_error{"Avocado!"}); }));
+    EXPECT_CALL(mock_display, configure(Not(Ref(conf))))
+        .Times(AnyNumber());
+    EXPECT_CALL(mock_session1, send_display_config(mt::DisplayConfigMatches(std::cref(*previous_base_config))));
+    EXPECT_CALL(mock_session2, send_display_config(mt::DisplayConfigMatches(std::cref(*previous_base_config))));
+
+    changer->configure_for_hardware_change(mt::fake_shared(conf));
+}
+
+TEST_F(MediatingDisplayChangerTest, focusing_a_session_with_db_preserving_attached_config_applies_config)
 {
     using namespace testing;
     auto conf = std::make_shared<mtd::NullDisplayConfiguration>();
@@ -333,6 +463,65 @@ TEST_F(MediatingDisplayChangerTest, focusing_a_session_with_attached_config_appl
     stub_session_container.insert_session(session1);
     changer->configure(session1, conf);
 
+    EXPECT_CALL(mock_display, apply_if_configuration_preserves_display_buffers(Ref(*conf)))
+        .WillOnce(Return(true));
+
+    EXPECT_CALL(mock_compositor, stop()).Times(0);
+    EXPECT_CALL(mock_display, configure(_)).Times(0);
+    EXPECT_CALL(mock_compositor, start()).Times(0);
+
+    session_event_sink.handle_focus_change(session1);
+}
+
+TEST_F(MediatingDisplayChangerTest, focusing_a_session_with_db_preserving_but_output_adding_attached_config_applies_config)
+{
+    using namespace testing;
+    auto session1 = std::make_shared<mtd::StubSession>();
+
+    auto conf = changer->base_configuration();
+
+    bool new_output_enabled{false};
+    conf->for_each_output(
+        [&new_output_enabled](mg::UserDisplayConfigurationOutput& output)
+        {
+            if (!output.used)
+            {
+                new_output_enabled = true;
+                output.used = true;
+            }
+        });
+    ASSERT_TRUE(new_output_enabled);
+
+    ON_CALL(mock_display, apply_if_configuration_preserves_display_buffers(_))
+        .WillByDefault(Return(true));
+
+    stub_session_container.insert_session(session1);
+    changer->configure(session1, conf);
+
+    /*
+     * Currently we have to tear down and recreate the compositor in order to add
+     * a new output. This is not an inherent limitation, and we might do better later.
+     */
+    InSequence s;
+    EXPECT_CALL(mock_compositor, stop()).Times(1);
+    EXPECT_CALL(mock_display, configure(Ref(*conf)));
+    EXPECT_CALL(mock_compositor, start()).Times(1);
+
+    session_event_sink.handle_focus_change(session1);
+}
+
+TEST_F(MediatingDisplayChangerTest, focusing_a_session_with_db_invalidating_attached_config_applies_config)
+{
+    using namespace testing;
+    auto conf = std::make_shared<mtd::NullDisplayConfiguration>();
+    auto session1 = std::make_shared<mtd::StubSession>();
+
+    ON_CALL(mock_display, apply_if_configuration_preserves_display_buffers(_))
+        .WillByDefault(Return(false));
+
+    stub_session_container.insert_session(session1);
+    changer->configure(session1, conf);
+
     InSequence s;
     EXPECT_CALL(mock_compositor, stop());
     EXPECT_CALL(mock_display, configure(Ref(*conf)));
@@ -341,13 +530,38 @@ TEST_F(MediatingDisplayChangerTest, focusing_a_session_with_attached_config_appl
     session_event_sink.handle_focus_change(session1);
 }
 
-TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_applies_base_config)
+TEST_F(MediatingDisplayChangerTest, failure_to_apply_session_config_on_focus_sends_error)
+{
+    using namespace testing;
+    auto conf = std::make_shared<mtd::NullDisplayConfiguration>();
+    auto session1 = std::make_shared<mtd::MockSceneSession>();
+
+    stub_session_container.insert_session(session1);
+    changer->configure(session1, conf);
+
+    EXPECT_CALL(
+        mock_display,
+        configure(mt::DisplayConfigMatches(std::cref(*conf))))
+            .WillOnce(InvokeWithoutArgs([]() { BOOST_THROW_EXCEPTION(std::runtime_error{"Banana"}); }));
+    EXPECT_CALL(
+        mock_display,
+        configure(Not(mt::DisplayConfigMatches(std::cref(*conf)))))
+            .Times(AnyNumber());
+    EXPECT_CALL(*session1, send_error(_));
+
+    session_event_sink.handle_focus_change(session1);
+}
+
+TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_applies_base_config_pausing_if_db_invalidated)
 {
     using namespace testing;
     auto conf = std::make_shared<mtd::NullDisplayConfiguration>();
     auto session1 = std::make_shared<mtd::StubSession>();
     auto session2 = std::make_shared<mtd::StubSession>();
 
+    ON_CALL(mock_display, apply_if_configuration_preserves_display_buffers(_))
+        .WillByDefault(Return(false));
+
     stub_session_container.insert_session(session1);
     changer->configure(session1, conf);
 
@@ -364,6 +578,43 @@ TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_a
     session_event_sink.handle_focus_change(session2);
 }
 
+TEST_F(MediatingDisplayChangerTest, focusing_a_session_without_attached_config_applies_base_config_not_pausing_if_db_preserved)
+{
+    using namespace testing;
+    std::shared_ptr<mg::DisplayConfiguration> conf = base_config.clone();
+    conf->for_each_output(
+        [](mg::UserDisplayConfigurationOutput& output)
+        {
+            if (output.used)
+            {
+                output.orientation =
+                    output.orientation == mir_orientation_normal ? mir_orientation_left : mir_orientation_normal;
+            }
+        });
+
+    auto session1 = std::make_shared<mtd::StubSession>();
+    auto session2 = std::make_shared<mtd::StubSession>();
+
+    stub_session_container.insert_session(session1);
+    changer->configure(session1, conf);
+
+    session_event_sink.handle_focus_change(session1);
+
+    Mock::VerifyAndClearExpectations(&mock_compositor);
+    Mock::VerifyAndClearExpectations(&mock_display);
+
+    EXPECT_CALL(
+        mock_display,
+        apply_if_configuration_preserves_display_buffers(mt::DisplayConfigMatches(std::cref(base_config))))
+            .WillOnce(Return(true));
+
+    EXPECT_CALL(mock_compositor, stop()).Times(0);
+    EXPECT_CALL(mock_display, configure(_)).Times(0);
+    EXPECT_CALL(mock_compositor, start()).Times(0);
+
+    session_event_sink.handle_focus_change(session2);
+}
+
 TEST_F(MediatingDisplayChangerTest, losing_focus_applies_base_config)
 {
     using namespace testing;
@@ -414,8 +665,7 @@ TEST_F(MediatingDisplayChangerTest, hardware_change_invalidates_session_configs)
     stub_session_container.insert_session(session1);
     changer->configure(session1, conf);
 
-    changer->configure_for_hardware_change(conf,
-                                           mir::DisplayChanger::PauseResumeSystem);
+    changer->configure_for_hardware_change(conf);
 
     Mock::VerifyAndClearExpectations(&mock_compositor);
     Mock::VerifyAndClearExpectations(&mock_display);
@@ -491,9 +741,7 @@ TEST_F(MediatingDisplayChangerTest, uses_server_action_queue_for_configuration_a
     Mock::VerifyAndClearExpectations(&mock_server_action_queue);
 
     EXPECT_CALL(mock_server_action_queue, enqueue(owner, _));
-    display_changer.configure_for_hardware_change(
-        conf,
-        mir::DisplayChanger::PauseResumeSystem);
+    display_changer.configure_for_hardware_change(conf);
     Mock::VerifyAndClearExpectations(&mock_server_action_queue);
 
     EXPECT_CALL(mock_server_action_queue, enqueue(owner, _));
@@ -646,6 +894,33 @@ TEST_F(MediatingDisplayChangerTest, notifies_all_sessions_on_set_base_configurat
     changer->set_base_configuration(mt::fake_shared(conf));
 }
 
+TEST_F(MediatingDisplayChangerTest, notifies_observer_on_set_base_configuration)
+{
+    using namespace testing;
+
+    struct MockDisplayConfigurationObserver : StubDisplayConfigurationObserver
+    {
+        MOCK_METHOD1(base_configuration_updated, void (std::shared_ptr<mg::DisplayConfiguration const> const& base_config));
+    } display_configuration_observer;
+
+    changer = std::make_shared<ms::MediatingDisplayChanger>(
+        mt::fake_shared(mock_display),
+        mt::fake_shared(mock_compositor),
+        mt::fake_shared(mock_conf_policy),
+        mt::fake_shared(stub_session_container),
+        mt::fake_shared(session_event_sink),
+        mt::fake_shared(server_action_queue),
+        mt::fake_shared(display_configuration_observer),
+        mt::fake_shared(mock_input_region),
+        mt::fake_shared(alarm_factory));
+
+    mtd::NullDisplayConfiguration conf;
+
+    EXPECT_CALL(display_configuration_observer, base_configuration_updated(_));
+
+    changer->set_base_configuration(mt::fake_shared(conf));
+}
+
 TEST_F(MediatingDisplayChangerTest, input_region_receives_display_configuration_on_start)
 {
     using namespace testing;
