@@ -17,7 +17,6 @@
  */
 
 #include "src/server/scene/surface_stack.h"
-#include "src/server/compositor/buffer_stream_surfaces.h"
 #include "mir/graphics/buffer_properties.h"
 #include "mir/geometry/rectangle.h"
 #include "mir/scene/observer.h"
@@ -26,13 +25,15 @@
 #include "mir/compositor/decoration.h"
 #include "src/server/report/null_report_factory.h"
 #include "src/server/scene/basic_surface.h"
+#include "src/server/compositor/stream.h"
 #include "mir/input/input_channel_factory.h"
 #include "mir/test/doubles/stub_input_channel.h"
 #include "mir/test/fake_shared.h"
 #include "mir/test/doubles/stub_buffer_stream.h"
+#include "mir/test/doubles/stub_buffer_stream_factory.h"
 #include "mir/test/doubles/stub_renderable.h"
 #include "mir/test/doubles/mock_buffer_stream.h"
-#include "mir/test/doubles/mock_buffer_bundle.h"
+#include "mir/test/doubles/stub_frame_dropping_policy_factory.h"
 
 #include <gmock/gmock.h>
 #include <gtest/gtest.h>
@@ -58,8 +59,7 @@ namespace
 
 void post_a_frame(mc::BufferStream& s)
 {
-    mtd::StubBuffer old_buffer;
-    s.swap_buffers(&old_buffer, [](mg::Buffer*){});
+    s.submit_buffer(std::make_shared<mtd::StubBuffer>());
 }
 
 MATCHER_P(SurfaceWithInputReceptionMode, mode, "")
@@ -128,7 +128,6 @@ struct SurfaceStack : public ::testing::Test
             std::string("stub"),
             geom::Rectangle{{},{}},
             mir_pointer_unconfined,
-            false,
             std::list<ms::StreamInfo> { { stub_buffer_stream1, {}, {} } },
             std::shared_ptr<mir::input::InputChannel>(),
             std::shared_ptr<mir::input::InputSender>(),
@@ -139,7 +138,6 @@ struct SurfaceStack : public ::testing::Test
             std::string("stub"),
             geom::Rectangle{{},{}},
             mir_pointer_unconfined,
-            false,
             std::list<ms::StreamInfo> { { stub_buffer_stream2, {}, {} } },
             std::shared_ptr<mir::input::InputChannel>(),
             std::shared_ptr<mir::input::InputSender>(),
@@ -151,7 +149,6 @@ struct SurfaceStack : public ::testing::Test
             std::string("stub"),
             geom::Rectangle{{},{}},
             mir_pointer_unconfined,
-            false,
             std::list<ms::StreamInfo> { { stub_buffer_stream3, {}, {} } },
             std::shared_ptr<mir::input::InputChannel>(),
             std::shared_ptr<mir::input::InputSender>(),
@@ -163,7 +160,6 @@ struct SurfaceStack : public ::testing::Test
             std::string("stub"),
             geom::Rectangle{{},{}},
             mir_pointer_unconfined,
-            false,
             std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
             std::shared_ptr<mir::input::InputChannel>(),
             std::shared_ptr<mir::input::InputSender>(),
@@ -279,15 +275,14 @@ TEST_F(SurfaceStack, decor_name_is_surface_name)
         std::string("Mary had a little lamb"),
         geom::Rectangle{{},{}},
         mir_pointer_unconfined,
-        false,
         std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
         std::shared_ptr<mir::input::InputChannel>(),
         std::shared_ptr<mir::input::InputSender>(),
         std::shared_ptr<mg::CursorImage>(),
         report);
     stack.add_surface(surface, default_params.input_mode);
-    surface->configure(mir_surface_attrib_visibility,
-                       mir_surface_visibility_exposed);
+    surface->configure(mir_window_attrib_visibility,
+                       mir_window_visibility_exposed);
     
     auto elements = stack.scene_elements_for(compositor_id);
     ASSERT_EQ(1, elements.size());
@@ -309,15 +304,14 @@ TEST_F(SurfaceStack, gets_surface_renames)
         std::string("username@hostname: /"),
         geom::Rectangle{{},{}},
         mir_pointer_unconfined,
-        false,
         std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
         std::shared_ptr<mir::input::InputChannel>(),
         std::shared_ptr<mir::input::InputSender>(),
         std::shared_ptr<mg::CursorImage>(),
         report);
     stack.add_surface(surface, default_params.input_mode);
-    surface->configure(mir_surface_attrib_visibility,
-                       mir_surface_visibility_exposed);
+    surface->configure(mir_window_attrib_visibility,
+                       mir_window_visibility_exposed);
     
     // (change directory in shell app)
     surface->rename("username@hostname: ~/Documents");
@@ -338,45 +332,45 @@ TEST_F(SurfaceStack, scene_counts_pending_accurately)
     using namespace testing;
     ms::SurfaceStack stack{report};
     stack.register_compositor(this);
-    mtd::StubBuffer stub_buffer;
-    int ready = 0;
-    auto mock_queue = std::make_shared<testing::NiceMock<mtd::MockBufferBundle>>();
-    ON_CALL(*mock_queue, buffers_ready_for_compositor(_))
-        .WillByDefault(InvokeWithoutArgs([&]{return ready;}));
-    ON_CALL(*mock_queue, client_release(_))
-        .WillByDefault(InvokeWithoutArgs([&]{ready++;}));
-    ON_CALL(*mock_queue, compositor_acquire(_))
-        .WillByDefault(InvokeWithoutArgs([&]{ready--; return mt::fake_shared(stub_buffer); }));
 
-    auto stream = std::make_shared<mc::BufferStreamSurfaces>(mock_queue);
+    struct StubBuffers : mtd::StubClientBuffers
+    {
+        std::shared_ptr<mg::Buffer>& operator[](mg::BufferID) override
+        {
+            return buffer;
+        }
+        std::shared_ptr<mg::Buffer> buffer {std::make_shared<mtd::StubBuffer>()};
+    };
+
+    auto buffers = std::make_shared<StubBuffers>();
+    mtd::StubFrameDroppingPolicyFactory factory;
+    auto stream = std::make_shared<mc::Stream>(factory, buffers, geom::Size{ 1, 1 }, mir_pixel_format_abgr_8888);
+
     auto surface = std::make_shared<ms::BasicSurface>(
         std::string("stub"),
         geom::Rectangle{{},{}},
         mir_pointer_unconfined,
-        false,
         std::list<ms::StreamInfo> { { stream, {}, {} } },
         std::shared_ptr<mir::input::InputChannel>(),
         std::shared_ptr<mir::input::InputSender>(),
         std::shared_ptr<mg::CursorImage>(),
         report);
     stack.add_surface(surface, default_params.input_mode);
-    surface->configure(mir_surface_attrib_visibility,
-                       mir_surface_visibility_exposed);
+    surface->configure(mir_window_attrib_visibility,
+                       mir_window_visibility_exposed);
 
     EXPECT_EQ(0, stack.frames_pending(this));
-    post_a_frame(*stream);
-    post_a_frame(*stream);
-    post_a_frame(*stream);
-    EXPECT_EQ(3, stack.frames_pending(this));
 
-    for (int expect = 3; expect >= 0; --expect)
+    unsigned int num_posts = 3;
+
+    for (auto i = 0u; i != num_posts; i++)
+        post_a_frame(*stream);
+
+    for (auto expect = 0u; expect != num_posts; expect++)
     {
-        ASSERT_EQ(expect, stack.frames_pending(this));
-        auto snap = stack.scene_elements_for(compositor_id);
-        for (auto& element : snap)
-        {
-            auto consumed = element->renderable()->buffer();
-        }
+        ASSERT_EQ(expect >= num_posts ? 0 : 1, stack.frames_pending(this));
+        for (auto& element : stack.scene_elements_for(compositor_id))
+            element->renderable()->buffer();
     }
 }
 
@@ -391,7 +385,6 @@ TEST_F(SurfaceStack, scene_doesnt_count_pending_frames_from_occluded_surfaces)
         std::string("stub"),
         geom::Rectangle{{},{}},
         mir_pointer_unconfined,
-        false,
         std::list<ms::StreamInfo> { { stream, {}, {} } },
         std::shared_ptr<mir::input::InputChannel>(),
         std::shared_ptr<mir::input::InputSender>(),
@@ -426,7 +419,6 @@ TEST_F(SurfaceStack, scene_doesnt_count_pending_frames_from_partially_exposed_su
         std::string("stub"),
         geom::Rectangle{{},{}},
         mir_pointer_unconfined,
-        false,
         std::list<ms::StreamInfo> { { stream, {}, {} } },
         std::shared_ptr<mir::input::InputChannel>(),
         std::shared_ptr<mir::input::InputSender>(),
@@ -541,7 +533,6 @@ TEST_F(SurfaceStack, generate_elementelements)
             std::string("stub"),
             geom::Rectangle{geom::Point{3 * i, 4 * i},geom::Size{1 * i, 2 * i}},
             mir_pointer_unconfined,
-            true,
             std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
             std::shared_ptr<mir::input::InputChannel>(),
             std::shared_ptr<mir::input::InputSender>(),
@@ -727,7 +718,6 @@ TEST_F(SurfaceStack, scene_elements_hold_snapshot_of_positioning_info)
             std::string("stub"),
             geom::Rectangle{geom::Point{3 * i, 4 * i},geom::Size{1 * i, 2 * i}},
             mir_pointer_unconfined,
-            true,
             std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
             std::shared_ptr<mir::input::InputChannel>(),
             std::shared_ptr<mir::input::InputSender>(),
@@ -761,7 +751,6 @@ TEST_F(SurfaceStack, generates_scene_elements_that_delay_buffer_acquisition)
         std::string("stub"),
         geom::Rectangle{geom::Point{3, 4},geom::Size{1, 2}},
         mir_pointer_unconfined,
-        true,
         std::list<ms::StreamInfo> { { mock_stream, {}, {} } },
         std::shared_ptr<mir::input::InputChannel>(),
         std::shared_ptr<mir::input::InputSender>(),
@@ -792,7 +781,6 @@ TEST_F(SurfaceStack, generates_scene_elements_that_allow_only_one_buffer_acquisi
         std::string("stub"),
         geom::Rectangle{geom::Point{3, 4},geom::Size{1, 2}},
         mir_pointer_unconfined,
-        true,
         std::list<ms::StreamInfo> { { mock_stream, {}, {} } },
         std::shared_ptr<mir::input::InputChannel>(),
         std::shared_ptr<mir::input::InputSender>(),
@@ -816,7 +804,6 @@ struct MockConfigureSurface : public ms::BasicSurface
             {},
             {{},{}},
             mir_pointer_unconfined,
-            true,
             std::list<ms::StreamInfo> { { std::make_shared<mtd::StubBufferStream>(), {}, {} } },
             {},
             {},
@@ -824,7 +811,7 @@ struct MockConfigureSurface : public ms::BasicSurface
             mir::report::null_scene_report())
     {
     }
-    MOCK_METHOD2(configure, int(MirSurfaceAttrib, int));
+    MOCK_METHOD2(configure, int(MirWindowAttrib, int));
 };
 }
 
@@ -847,7 +834,7 @@ TEST_F(SurfaceStack, occludes_not_rendered_surface)
     auto const elements2 = stack.scene_elements_for(compositor_id2);
     ASSERT_THAT(elements2.size(), Eq(1u));
 
-    EXPECT_CALL(*mock_surface, configure(mir_surface_attrib_visibility, mir_surface_visibility_occluded));
+    EXPECT_CALL(*mock_surface, configure(mir_window_attrib_visibility, mir_window_visibility_occluded));
 
     elements.back()->occluded();
     elements2.back()->occluded();
@@ -870,7 +857,7 @@ TEST_F(SurfaceStack, exposes_rendered_surface)
     auto const elements2 = stack.scene_elements_for(compositor_id2);
     ASSERT_THAT(elements2.size(), Eq(1u));
 
-    EXPECT_CALL(*mock_surface, configure(mir_surface_attrib_visibility, mir_surface_visibility_exposed));
+    EXPECT_CALL(*mock_surface, configure(mir_window_attrib_visibility, mir_window_visibility_exposed));
 
     elements.back()->occluded();
     elements2.back()->rendered();
@@ -897,7 +884,7 @@ TEST_F(SurfaceStack, occludes_surface_when_unregistering_all_compositors_that_re
     auto const elements3 = stack.scene_elements_for(compositor_id3);
     ASSERT_THAT(elements3.size(), Eq(1u));
 
-    EXPECT_CALL(*mock_surface, configure(mir_surface_attrib_visibility, mir_surface_visibility_exposed))
+    EXPECT_CALL(*mock_surface, configure(mir_window_attrib_visibility, mir_window_visibility_exposed))
         .Times(2);
 
     elements.back()->occluded();
@@ -906,7 +893,7 @@ TEST_F(SurfaceStack, occludes_surface_when_unregistering_all_compositors_that_re
 
     Mock::VerifyAndClearExpectations(mock_surface.get());
 
-    EXPECT_CALL(*mock_surface, configure(mir_surface_attrib_visibility, mir_surface_visibility_occluded));
+    EXPECT_CALL(*mock_surface, configure(mir_window_attrib_visibility, mir_window_visibility_occluded));
 
     stack.unregister_compositor(compositor_id2);
     stack.unregister_compositor(compositor_id3);
@@ -991,8 +978,8 @@ TEST_F(SurfaceStack, overlays_do_not_appear_in_input_enumeration)
     stack.add_surface(stub_surface2, default_params.input_mode);
 
     // Configure surface1 and surface2 to appear in input enumeration.
-    stub_surface1->configure(mir_surface_attrib_visibility, MirSurfaceVisibility::mir_surface_visibility_exposed);
-    stub_surface2->configure(mir_surface_attrib_visibility, MirSurfaceVisibility::mir_surface_visibility_exposed);
+    stub_surface1->configure(mir_window_attrib_visibility, MirWindowVisibility::mir_window_visibility_exposed);
+    stub_surface2->configure(mir_window_attrib_visibility, MirWindowVisibility::mir_window_visibility_exposed);
 
     stack.add_input_visualization(mt::fake_shared(r));
 
@@ -1069,9 +1056,9 @@ TEST_F(SurfaceStack, only_enumerates_exposed_input_surfaces)
     stack.add_surface(stub_surface2, default_params.input_mode);
     stack.add_surface(stub_surface3, default_params.input_mode);
 
-    stub_surface1->configure(mir_surface_attrib_visibility, MirSurfaceVisibility::mir_surface_visibility_exposed);
-    stub_surface2->configure(mir_surface_attrib_visibility, MirSurfaceVisibility::mir_surface_visibility_occluded);
-    stub_surface3->configure(mir_surface_attrib_visibility, MirSurfaceVisibility::mir_surface_visibility_occluded);
+    stub_surface1->configure(mir_window_attrib_visibility, MirWindowVisibility::mir_window_visibility_exposed);
+    stub_surface2->configure(mir_window_attrib_visibility, MirWindowVisibility::mir_window_visibility_occluded);
+    stub_surface3->configure(mir_window_attrib_visibility, MirWindowVisibility::mir_window_visibility_occluded);
 
     int num_exposed_surfaces = 0;
     auto const count_exposed_surfaces = [&num_exposed_surfaces](std::shared_ptr<mi::Surface> const&){
